/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* NEXTSTEP PPC Control Center.                                            */
/* BrowserController class interface.                                      */
/* One object of this class is instanciated for each opened project.       */
/* It controls the project files browser.                                  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 30.07.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: BrowserController.h                                           */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import <appkit/appkit.h>

#define MAX_NAME_LEN 512
#define MAX_FILES_ENTRIES 255      /* maximum of 255 files per files array */

typedef enum                  /* possible fields (first column of browser) */
{
  VIRTUAL_FIELD,
  HARDWARE_FIELD,
  MODULES_FIELD,
  NB_FIELDS
} filesField;

@interface BrowserController:Object
{
  id filesBrowser;                            /* objects initialized by IB */
  id projectController;

  id menuFilesUpdate;                 /* menu options to disable or enable */
  id menuFilesOpenInWorkspace;       
  id menuFilesSelectInWorkspace;
  id menuFilesAddExisting;
  id menuFilesCreateNew;
  id menuFilesDelete;

  char projectName[MAX_NAME_LEN]; /* proj. name without path and extension */
  char projectPath[MAX_NAME_LEN];             /* project path without name */
  char *files[NB_FIELDS][MAX_FILES_ENTRIES];      /* arrays of files names */
  filesField currentField;   /* current selected field or NB_FIELD if none */
  char currentFile[MAX_NAME_LEN];                 /* current selected file */
}

/*-------------------------------------------------------------------------*/
/* initForProject:                                                         */
/* Initializes the browser for the specified project.                      */
/* Message sent by the project controller when a new project is loaded.    */
/* A enableArch: message will be sent to the project controller when a     */
/* hardware configuration file is found.                                   */
/* In addition to the name and path of the project, this                   */
/* method takes as argument the menu options which can be eventually       */
/* modified depending on the actions performed on the browser.             */
/*-------------------------------------------------------------------------*/
- initForProject: (char *)projectNameParam
  path: (char *)projectPathParam
  : menuFilesUpdateP
  : menuFilesOpenInWorkspaceP
  : menuFilesSelectInWorkspaceP
  : menuFilesAddExistingP
  : menuFilesCreateNewP
  : menuFilesDeleteP;

/*-------------------------------------------------------------------------*/
/* updateAllFields:                                                        */
/* Update all fields of the browser.                                       */
/*-------------------------------------------------------------------------*/
- updateAllFields;

/*-------------------------------------------------------------------------*/
/* updateField:                                                            */
/* Initializes the specified field.                                        */
/*-------------------------------------------------------------------------*/
- updateField: (filesField) field;

/*-------------------------------------------------------------------------*/
/* updateCurrentField:                                                     */
/* Update currently selected field.                                        */
/* All fields are updated if none is selected.                             */
/*-------------------------------------------------------------------------*/
- updateCurrentField;

/*-------------------------------------------------------------------------*/
/* filesMenuSelected:                                                      */
/* Message sent by the project controller when one of the Files menu       */
/* option is selected.                                                     */
/*-------------------------------------------------------------------------*/
- filesMenuSelected: sender;

/*-------------------------------------------------------------------------*/
/* clicked:                                                                */
/* Message sent by the browser when the user clicks on a cell of the       */
/* browser.                                                                */
/*-------------------------------------------------------------------------*/
- clicked: sender;

/*-------------------------------------------------------------------------*/
/* doubleClicked:                                                          */
/* Message sent by the browser when the user double-clicks on a cell of    */
/* the browser.                                                            */
/*-------------------------------------------------------------------------*/
- doubleClicked: sender;

/*-------------------------------------------------------------------------*/
/* browser:                                                                */
/* NXBrowser delegate method sent when the browsers needs the              */
/* contents of the cells.                                                  */
/*-------------------------------------------------------------------------*/
- (int) browser: sender
  fillMatrix: matrix
  inColumn: (int)column;

/*-------------------------------------------------------------------------*/
/* free:                                                                   */
/* Release dynamically allocated instance variable.                        */
/*-------------------------------------------------------------------------*/
- free;

@end
