/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* NEXTSTEP PPC Control Center.                                            */
/* BrowserController class implementation.                                 */
/* One object of this class is instanciated for each opened project.       */
/* It controls the project files browser.                                  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 30.07.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: BrowserController.h                                           */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import "BrowserController.h"
#import "ProjectController.h"
#import "architectures.h"
#import <sys/dir.h>
#import <syslog.h>

#define UPDATE_ALL_STRING "Update All"
#define UPDATE_STRING     "Update"
#define MAX_EXTENSION_LEN 100             /* maximum file extension length */

static char *fields[] =              /* fields of the first browser column */
{      /* each element of this array correspond to one filesField constant */
  "Virtual",                                /* virtual configuration files */
  "Hardware",                              /* hardware configuration files */
  "Modules"                               /* tasks implementations modules */
};

@implementation BrowserController

/*-------------------------------------------------------------------------*/
/* initForProject:                                                         */
/* Initializes the browser for the specified project.                      */
/* Message sent by the project controller when a new project is loaded.    */
/* A enableArch: message will be sent to the project controller when a     */
/* hardware configuration file is found.                                   */
/* In addition to the name and path of the project, this                   */
/* method takes as argument the menu options which can be eventually       */
/* modified depending on the actions performed on the browser.             */
/*-------------------------------------------------------------------------*/
- initForProject: (char *)projectNameParam
  path: (char *)projectPathParam
  : menuFilesUpdateP
  : menuFilesOpenInWorkspaceP
  : menuFilesSelectInWorkspaceP
  : menuFilesAddExistingP
  : menuFilesCreateNewP
  : menuFilesDeleteP
{
  int actField;

  strcpy(projectName, projectNameParam);
  strcpy(projectPath, projectPathParam);

  for (actField = 0; actField < NB_FIELDS; actField++)
    files[actField][0] = NULL;

  [filesBrowser setTarget: self];
  [filesBrowser setAction: @selector(clicked:)];
  [filesBrowser setDoubleAction: @selector(doubleClicked:)];
  [filesBrowser setTitle: projectName ofColumn: 0];
  [filesBrowser loadColumnZero];

  menuFilesUpdate = menuFilesUpdateP;
  menuFilesOpenInWorkspace = menuFilesOpenInWorkspaceP;
  menuFilesSelectInWorkspace = menuFilesSelectInWorkspaceP;
  menuFilesAddExisting = menuFilesAddExistingP;
  menuFilesCreateNew = menuFilesCreateNewP;
  menuFilesDelete = menuFilesDeleteP;
  
  currentField = NB_FIELDS;
  currentFile[0] = '\0';

  return self;
}

/*-------------------------------------------------------------------------*/
/* updateAllFields:                                                        */
/* Update all fields of the browser.                                       */
/*-------------------------------------------------------------------------*/
- updateAllFields
{
  int actField;

  for (actField = 0; actField < NB_FIELDS; actField++)
    [self updateField: actField];
  
  return self;
}

/*-------------------------------------------------------------------------*/
/* updateField:                                                            */
/* Initializes the specified field.                                        */
/*-------------------------------------------------------------------------*/
- updateField: (filesField) field
{
  DIR *dir;
  struct direct *dirEnt;
  int actEnt;
  BOOL fileAccepted;
  char ext [MAX_EXTENSION_LEN];
  char *point_ptr;
  int idx;
  char virtualConfName[MAX_NAME_LEN];
  char virtualIntName[MAX_NAME_LEN];
  int actHardwareName;
  char actHardwareFileName[MAX_NAME_LEN];
  
  if ((dir = opendir(projectPath)) == NULL)              /* open directory */
    return nil;
    
              /* first release the memory used by the field if initialized */    
  for (actEnt = 0; files[field][actEnt] != NULL; actEnt++)
  {
    free(files[field][actEnt]);
    files[field][actEnt] = NULL;
  }
  
  actEnt = 0;                                      /* misc initializations */
  sprintf(virtualConfName, "%s.vmc", projectName);
  sprintf(virtualIntName, "%s.h", projectName);
  
  for (dirEnt = readdir(dir); dirEnt != NULL; dirEnt = readdir(dir))
  {
    fileAccepted = NO;
         /* determine file extension in order to filtrate non-source files */
    point_ptr = dirEnt->d_name + strlen(dirEnt->d_name) - 1;
    idx = strlen(dirEnt->d_name);
    while(*point_ptr != '.' && idx--)
      point_ptr--;
    for (idx = 0; *point_ptr != '\0'; idx++, point_ptr++)
      ext[idx] = *point_ptr;
    ext[idx] = '\0';	

    if ((strcmp(dirEnt->d_name, ".") && 
        strcmp(dirEnt->d_name, "..")) &&
	(!strcmp(ext, ".c") || !strcmp(ext, ".h") || !strcmp(ext, ".vmc")))
    {         /* the file is a source file with .vmc, .c,  or .h extension */
      switch(field)
      {
	case VIRTUAL_FIELD:
	  fileAccepted = !strcmp(dirEnt->d_name, virtualConfName) ||
	                 !strcmp(dirEnt->d_name, virtualIntName);
	  break;
	case HARDWARE_FIELD:
	  for (actHardwareName = 0; 
	       archNames[actHardwareName] != NULL && !fileAccepted; 
	       actHardwareName++)
	  {
	    int c;
	    strcpy(actHardwareFileName, dirEnt->d_name);
	    for(c = 0; actHardwareFileName[c] != '.' && 
	        actHardwareFileName[c] != '\0'; c++);
	    actHardwareFileName[c] = '\0';
	    fileAccepted = !strcmp(actHardwareFileName, 
	      archNames[actHardwareName]);
	    if (fileAccepted)
	      [projectController enableArch: actHardwareName: YES];
	  }
	  break;
	case MODULES_FIELD:
	  fileAccepted = strcmp(dirEnt->d_name, virtualConfName) &&
	                 strcmp(dirEnt->d_name, virtualIntName);
	  for (actHardwareName = 0; 
	       archNames[actHardwareName] != NULL; 
	       actHardwareName++)
	  {
	    int c;
	    strcpy(actHardwareFileName, dirEnt->d_name);
	    for(c = 0; actHardwareFileName[c] != '.' && 
	        actHardwareFileName[c] != '\0'; c++);
	    actHardwareFileName[c] = '\0';
	    fileAccepted = fileAccepted && strcmp(actHardwareFileName, 
	      archNames[actHardwareName]);
	  }
	  break;
	default:
	  break;
      }
    }

    if (fileAccepted)
    {                                                  /* add file in list */
      files[field][actEnt] = malloc(strlen(dirEnt->d_name)+1);
      strcpy(files[field][actEnt], dirEnt->d_name);
      ++actEnt;
      files[field][actEnt] = NULL;
    }
  }

  closedir(dir);
  [filesBrowser reloadColumn: [filesBrowser lastColumn]];

  return self;
}

/*-------------------------------------------------------------------------*/
/* updateCurrentField:                                                     */
/* Update currently selected field.                                        */
/* All fields are updated if none is selected.                             */
/*-------------------------------------------------------------------------*/
- updateCurrentField
{
  switch(currentField)
  {
    case NB_FIELDS:
      [self updateAllFields];
      [projectController updateArchitectures];
      break;
    case HARDWARE_FIELD:
      [projectController updateArchitectures];
      break;
    default:
      [self updateField: currentField];
      break;
  }

  return self;
}

/*-------------------------------------------------------------------------*/
/* filesMenuSelected:                                                      */
/* Message sent by the project controller when one of the Files menu       */
/* option is selected.                                                     */
/*-------------------------------------------------------------------------*/
- filesMenuSelected: sender
{
  id menuItem = [sender selectedCell];
  int senderTag;

  if (menuItem == nil)
    return self;
    
  senderTag = [menuItem tag];
  
  if (senderTag == [menuFilesUpdate tag])
  {
    [self updateCurrentField];
    return self;
  }
  
  if (senderTag == [menuFilesOpenInWorkspace tag])
  {
    char fileName [MAX_NAME_LEN];

    sprintf(fileName, "%s%s", projectPath, currentFile);
    
    if (![[Application workspace] 
      openFile: fileName
      withApplication: "Edit"
      andDeactivate: YES])
    {
      NXRunAlertPanel([menuItem title],
	"Cannot open file %s with Edit",
	NULL, NULL, NULL,
	currentFile);
    }
    
    return self;
  }

  if (senderTag == [menuFilesSelectInWorkspace tag])
  {
    char fileName [MAX_NAME_LEN];

    sprintf(fileName, "%s%s", projectPath, currentFile);
    
    if (![[Application workspace] selectFile: fileName 
      inFileViewerRootedAt: ""])
    {
      NXRunAlertPanel([menuItem title],
	"Cannot select file %s in Workspace",
	NULL, NULL, NULL,
	currentFile);
    }
    
    return self;
  }

  NXRunAlertPanel([menuItem title],
    "This feature is not yet implemented. %s",
    NULL, NULL, NULL,
    "Remember that you're using a Beta version.");
  return self;
  
  if (senderTag == [menuFilesAddExisting tag])
  {
    return self;
  }

  if (senderTag == [menuFilesCreateNew tag])
  {
    return self;
  }

  if (senderTag == [menuFilesDelete tag])
  {
    return self;
  }

  return self;
}

/*-------------------------------------------------------------------------*/
/* clicked:                                                                */
/* Message sent by the browser when the user clicks on a cell of the       */
/* browser.                                                                */
/* It updates the instance variables currentField and currentFile.         */
/* The menu options of the File sub-menu are also updated.                 */
/*-------------------------------------------------------------------------*/
- clicked: sender
{
  char *selectedString;
  int actField;
  char updateMenuTitle [100];

  selectedString = (char *)[[filesBrowser selectedCell] stringValue];
  
  if (selectedString == NULL)
  {    /* user release the mouse-button outside the browser, no field sel. */
    currentField = NB_FIELDS;
    currentFile[0] = '\0';                          /* no file is selected */

    [menuFilesUpdate setTitle: UPDATE_ALL_STRING];
    [menuFilesOpenInWorkspace setEnabled: NO];
    [menuFilesSelectInWorkspace setEnabled: NO];
    [menuFilesAddExisting setEnabled: NO];
    [menuFilesCreateNew setEnabled: NO];
    [menuFilesDelete setEnabled: NO];
    
    return self;
  }
  
  for (actField = 0; 
	actField < NB_FIELDS && 
	strcmp(selectedString, fields[actField]);
	actField++);
	
  if (actField < NB_FIELDS)
  {          /* The user clicked on one of the fields of the first column. */
    currentField = actField;
    currentFile[0] = '\0';  /* no file is selected */

    sprintf(updateMenuTitle, UPDATE_STRING " %s", fields[currentField]);
    [menuFilesUpdate setTitle: updateMenuTitle];
    [menuFilesOpenInWorkspace setEnabled: NO];
    [menuFilesSelectInWorkspace setEnabled: NO];
    [menuFilesAddExisting setEnabled: YES];
    [menuFilesCreateNew setEnabled: YES];
    [menuFilesDelete setEnabled: NO];

    return self;
  }
  
  strcpy(currentFile, selectedString);       /* the user clicked on a file */

  sprintf(updateMenuTitle, UPDATE_STRING " %s", fields[currentField]);
  [menuFilesUpdate setTitle: updateMenuTitle];
  [menuFilesOpenInWorkspace setEnabled: YES];
  [menuFilesSelectInWorkspace setEnabled: YES];
  [menuFilesAddExisting setEnabled: YES];
  [menuFilesCreateNew setEnabled: YES];
  [menuFilesDelete setEnabled: YES];
  
  return self;
}

/*-------------------------------------------------------------------------*/
/* doubleClicked:                                                          */
/* Message sent by the browser when the user double-clicks on a cell of    */
/* the browser.                                                            */
/*-------------------------------------------------------------------------*/
- doubleClicked: sender
{
  char fileName [MAX_NAME_LEN];
  char selectedCellString [MAX_NAME_LEN];
  int actField;
  
  strcpy(selectedCellString, [[sender selectedCell] stringValue]);
  for (actField = 0; 
	actField < NB_FIELDS && 
	strcmp(selectedCellString, fields[actField]);
	actField++);
	
  if (actField < NB_FIELDS)
  {
    /* The user double clicked on one of the fields of the first column.   */
    /* The field must be reloaded.                                         */
    
    switch(actField)
    {
      case HARDWARE_FIELD:
        [projectController updateArchitectures];
	break;
      default:
        [self updateField: actField];
        break;
    }
    
    return self;
  }
  
  sprintf(fileName, "%s%s", projectPath, selectedCellString);
  
  if (![[Application workspace] 
    openFile: fileName
    withApplication: "Edit"
    andDeactivate: YES])
  {
    NXRunAlertPanel("Edit File",
      "Cannot open file %s with Edit",
      NULL, NULL, NULL,
      [[sender selectedCell] stringValue]);
  }
    
  return self;
}

/*-------------------------------------------------------------------------*/
/* browser:                                                                */
/* NXBrowser delegate method sent when the browsers needs the              */
/* contents of the cells.                                                  */
/*-------------------------------------------------------------------------*/
- (int) browser: sender
  fillMatrix: matrix
  inColumn: (int)column
{
  if (column == 0)
  {                        /* column zero contains the fields descriptions */
    int actField;
    id actCell;
    
    for (actField = 0; actField < NB_FIELDS; actField++)
    {
      [matrix addRow];
      actCell = [matrix cellAt: actField: column];
      [actCell setStringValue: fields[actField]];
      [actCell setLeaf: NO];
    }
    
    return actField;
  }
  else
  {                           /* fill the matrix with the field file names */
    int actName;
    filesField actField;
    id actFieldId;
    id currentCell;

    actFieldId = [sender selectedCell];
    for (actField = 0; 
         actField < NB_FIELDS && 
	 strcmp([actFieldId stringValue], fields[actField]);
	 actField++);
	 
    if (actField >= NB_FIELDS)
      return 0;  

    for (actName = 0; files[actField][actName] != NULL; actName++)
    {
      [matrix addRow];
      currentCell = [matrix cellAt: actName: 0];
      [currentCell setStringValue: files[actField][actName]];
      [currentCell setLeaf: YES];
    }
    
    return actName;
  }
  
  return 0;
}

/*-------------------------------------------------------------------------*/
/* free:                                                                   */
/* Release dynamically allocated instance variable.                        */
/*-------------------------------------------------------------------------*/
- free
{
  int actField;
  int actEnt;
  
  for (actField = 0; actField < NB_FIELDS; actField++)
  {
    for (actEnt = 0; files[actField][actEnt] != NULL; actEnt++)
    {
      free(files[actField][actEnt]);
      files[actField][actEnt] = NULL;
    }
  }
	
  return [super free];
}

@end
