/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* NEXTSTEP PPC Control Center.                                            */
/* CenterController class interface.                                       */
/* One object of this class is instanciated at startup. It controls the    */
/* interface objects.                                                      */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 26.07.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: CenterController.h                                            */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import <appkit/appkit.h>

#define MAX_PATH_LEN 200                            /* maximum path lentgh */

typedef enum
{
  MENU_INFO_TAG = 0,
  MENU_PROJECT_TAG = 10,
  MENU_EDIT_TAG = 20,
  MENU_ACTIONS_TAG = 30,
  MENU_FILES_TAG = 40,
  MENU_WINDOW_TAG = 50
} submenusTags;

typedef enum
{
  MENU_ACTIONS_CONFIGURE_TAG,
  MENU_ACTIONS_MAKE_TAG,
  MENU_ACTIONS_INSTALL_TAG,
  MENU_ACTIONS_RUN_TAG,
  MENU_ACTIONS_STOP_TAG,
  MENU_ACTIONS_VIEW_CONFIG_TAG,
  MENU_ACTIONS_CLEAN_TAG,
  MENU_ACTIONS_CMI_TAG,
  MENU_ACTIONS_CMR_TAG,
} actionsTags;

#define MENU_OPTION_TAG(menu, option) (menu+1+option)

@interface CenterController:Object
{
  id menuInfoHelp;                                    /* interface objects */
  id menuProjectOpen;
  id menuProjectSave;                                 
  id menuProjectSaveAll;
  id menuProjectRevertToSaved;
  id menuProjectClose;
  id menuEditCut;
  id menuEditCopy;
  id menuEditPaste;
  id menuEditClearBuffer;
  id menuEditSelectAll;
  id menuFilesUpdate;
  id menuFilesOpenInWorkspace;
  id menuFilesSelectInWorkspace;
  id menuFilesAddExisting;
  id menuFilesCreateNew;
  id menuFilesDelete;
  id menuActionsConfigure;
  id menuActionsMake;
  id menuActionsInstall;
  id menuActionsRun;
  id menuActionsStop;
  id menuActionsViewConfiguration;
  id menuActionsClean;
  id menuActionsConfigureMakeInstall;
  id menuActionsConfigureMakeRun;
  id menuQuit;
                                 /* objects instanciated when loading nibs */
  id infoPanel;                  /* info panel created by loading info.nib */
  id newProject; /* new project object just created by loading project.nib */
  id configFixer;     /* configuration fixer created by loading doctor.nib */
                                               /* other instance variables */
  float windowOffset; /* X and Y offsets to add to each new project window */
  id projectsList;                  /* list containing all opened projects */
  char PPCHome[MAX_PATH_LEN];                        /* PPC home directory */
  BOOL saveCanceled;            /* YES if save operation has been canceled */
  id optionsToEnableWhenFirstProject;    /* a list containing menu options */
  id optionsToDisableWhenLastProject;                              /* idem */
  id activeProject;               /* current active project or nil if none */
  BOOL limitedMode;         /* YES if application executes in limited mode */
  char PPCArch[MAX_PATH_LEN];                          /* PPC architecture */
}

/*-------------------------------------------------------------------------*/
/* infoPanel:                                                              */
/* Shows the application info panel.                                       */
/*-------------------------------------------------------------------------*/
- infoPanel:sender;

/*-------------------------------------------------------------------------*/
/* menuActionsStop:                                                        */
/* Return the above object.                                                */
/*-------------------------------------------------------------------------*/
- menuActionsStop;

/*-------------------------------------------------------------------------*/
/* projectOpened:                                                          */
/* Check if the specified project is already opened. Returns its id if it  */
/* is the case, nil otherwise.                                             */
/*-------------------------------------------------------------------------*/
- projectOpened: (const char *)projectName;

/*-------------------------------------------------------------------------*/
/* createProject:                                                          */
/* Create a new project.nib copy and initialize it.                        */
/*-------------------------------------------------------------------------*/
- createProject: (const char *)projectName;

/*-------------------------------------------------------------------------*/
/* openProject:                                                            */
/* Message sent by the menu when a project must be opened.                 */
/*-------------------------------------------------------------------------*/
- openProject: sender;

/*-------------------------------------------------------------------------*/
/* saveProject:                                                            */
/* Message sent by the menu when a project must be saved.                  */
/*-------------------------------------------------------------------------*/
- saveProject: sender;

/*-------------------------------------------------------------------------*/
/* saveAllProjects:                                                        */
/* Message sent by the menu when all projects must be saved.               */
/*-------------------------------------------------------------------------*/
- saveAllProjects: sender;

/*-------------------------------------------------------------------------*/
/* revertToSaved:                                                          */
/* Message sent by the menu when a project must be reverted to its saved   */
/* state.                                                                  */
/*-------------------------------------------------------------------------*/
- revertToSaved: sender;

/*-------------------------------------------------------------------------*/
/* closeProject:                                                           */
/* Message sent by the menu when a project must be closed.                 */
/*-------------------------------------------------------------------------*/
- closeProject: sender;

/*-------------------------------------------------------------------------*/
/* closeAllProjects:                                                       */
/* Close all projects.                                                     */
/* Returns nil if not all projects have been closed.                       */
/*-------------------------------------------------------------------------*/
- closeAllProjects;

/*-------------------------------------------------------------------------*/
/* clearBuffer:                                                            */
/* Sent when the clear buffer menu option is selected.                     */
/*-------------------------------------------------------------------------*/
- clearBuffer: sender;

/*-------------------------------------------------------------------------*/
/* filesMenuSelected:                                                      */
/* Message sent by the menu when one of the Files option is selected.      */
/*-------------------------------------------------------------------------*/
- filesMenuSelected: sender;

/*-------------------------------------------------------------------------*/
/* actionsMenuSelected:                                                    */
/* Message sent by the menu when one of the Actions options is selected.   */
/*-------------------------------------------------------------------------*/
- actionsMenuSelected: sender;

/*-------------------------------------------------------------------------*/
/* setActiveProject:                                                       */
/* Message sent by a project when it becomes active.                       */
/*-------------------------------------------------------------------------*/
- setActiveProject: sender;

/*-------------------------------------------------------------------------*/
/* removeProject:                                                          */
/* Remove the project given as parameter.                                  */
/*-------------------------------------------------------------------------*/
- removeProject: project;

/*-------------------------------------------------------------------------*/
/* saveCanceled:                                                           */
/* Message sent by a project when a save operation is canceled.            */
/*-------------------------------------------------------------------------*/
- saveCanceled;

/*-------------------------------------------------------------------------*/
/* enableMenuOptionsForFirstProject:                                       */
/* Enable the menu options stored in the optionsToEnableWhenFirstProject   */
/* list. This method must be called when the first project is created.     */
/*-------------------------------------------------------------------------*/
- enableMenuOptionsForFirstProject;

/*-------------------------------------------------------------------------*/
/* disableMenuOptionsForLastProject:                                       */
/* Disable the menu options stored in the optionsToDisableWhenLastProject  */
/* list. This method must be called when the last project is closed.       */
/*-------------------------------------------------------------------------*/
- disableMenuOptionsForLastProject;

/*-------------------------------------------------------------------------*/
/* runInLimitedMode:                                                       */
/* Set the application to be executed in limited mode. In this mode, it is */
/* not possible to open projects. This method sets the instance variable   */
/* limitedMode to YES.                                                     */
/*-------------------------------------------------------------------------*/
- runInLimitedMode: (char *)title: (char *)firstPhrase;

/*-------------------------------------------------------------------------*/
/* initPPCHome:                                                            */
/* Get the PPC home directory from the default environment variable        */
/* PPC Home and store it in the PPCHome instance variable.                 */
/* Returns nil if not defined.                                             */
/*-------------------------------------------------------------------------*/
- initPPCHome;

/*-------------------------------------------------------------------------*/
/* PPCHome:                                                                */
/* Return the PPC home directory.                                          */
/*-------------------------------------------------------------------------*/
- (char *)PPCHome;

/*-------------------------------------------------------------------------*/
/* PPCArch:                                                                */
/* Return the PPC architecture.                                            */
/*-------------------------------------------------------------------------*/
- (char *)PPCArch;

/*-------------------------------------------------------------------------*/
/* initPPCArch:                                                            */
/* Get the PPC architecture.                                               */
/* Returns nil if not defined.                                             */
/*-------------------------------------------------------------------------*/
- initPPCArch;

/*-------------------------------------------------------------------------*/
/* checkPPCHome:                                                           */
/* Check if the PPC directory specified by the instance variable PPCHome   */
/* exists.                                                                 */
/* Returns nil if it is not the case.                                      */
/*-------------------------------------------------------------------------*/
- checkPPCHome;

/*-------------------------------------------------------------------------*/
/* getPPCEnvironment:                                                      */
/* Get the PPC environment and initialize the corresponding instance       */
/* variables. If an error occured, an error panel is shown and the method  */
/* returns nil.                                                            */
/*-------------------------------------------------------------------------*/
- getPPCEnvironment;

@end

@interface CenterController(ApplicationDelegate)

/*-------------------------------------------------------------------------*/
/* appWillInit:                                                            */
/* Called just before application initialization.                          */
/*-------------------------------------------------------------------------*/
- appWillInit: sender;

/*-------------------------------------------------------------------------*/
/* appDidInit:                                                             */
/* Called just after application initialization.                           */
/*-------------------------------------------------------------------------*/
- appDidInit: sender;

/*-------------------------------------------------------------------------*/
/* appWillTerminate:                                                       */
/* Called just before the application terminates.                          */
/*-------------------------------------------------------------------------*/
- appWillTerminate: sender;

/*-------------------------------------------------------------------------*/
/* appAcceptsAnotherFile:                                                  */
/* PPCCenter accepts files from the Workspace.                             */
/*-------------------------------------------------------------------------*/
- (BOOL)appAcceptsAnotherFile: sender;

/*-------------------------------------------------------------------------*/
/* app:openFile:type:                                                      */
/* Message sent when the user opens a virtual machine configuration file   */
/* from the Workspace.                                                     */
/*-------------------------------------------------------------------------*/
- (int)app: sender
  openFile: (const char *)filename
  type: (const char *)aType;

@end

