/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* NEXTSTEP PPC Control Center.                                            */
/* DragTextField class implementation.                                     */
/* TextField subclass implementing the dragging destination protocol. It   */
/* lets the user drag a file into the text field.                          */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 29.07.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: DragTextField.m                                               */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import "DragTextField.h"
#import "ProjectController.h"

@implementation DragTextField

/*-------------------------------------------------------------------------*/
/* initFrame:                                                              */
/* This method is automatically called when a new DragView object is       */
/* created.                                                                */
/* View class.                                                             */
/*-------------------------------------------------------------------------*/
- initFrame:(const NXRect *)theFrame
{
  const char *const types[] = {NXFilenamePboardType};
	
  [super initFrame:theFrame];
  [self setEditable: YES];
  [self setSelectable: YES];
  
  [self registerForDraggedTypes:(const char *const*)&types count:1];
  
  return self;
}

/*-------------------------------------------------------------------------*/
/* includesType:                                                           */
/* Returns YES if the <type> is included within <types>.                   */
/*-------------------------------------------------------------------------*/
static BOOL includesType (const NXAtom *types, NXAtom type)
{
   if (types) while (*types) if (*types++ == type) return YES;
   return NO;
}

/*-------------------------------------------------------------------------*/
/* draggingEntered:                                                        */
/* Invoked when the dragged image enters the destination.                  */
/* NXDraggingDestination informal protocol.                                */
/*-------------------------------------------------------------------------*/
- (NXDragOperation)draggingEntered:sender
{
  return NX_DragOperationGeneric;
}

/*-------------------------------------------------------------------------*/
/* draggingUpdated:                                                        */
/* Invoked periodically as the image is poised within the destination.     */
/* NXDraggingDestination informal protocol.                                */
/*-------------------------------------------------------------------------*/
- (NXDragOperation)draggingUpdated:sender
{
  return NX_DragOperationGeneric;
}

/*-------------------------------------------------------------------------*/
/* draggingExited:                                                         */
/* Invoked when the dragged image exits the destination.                   */
/* NXDraggingDestination informal protocol.                                */
/*-------------------------------------------------------------------------*/
- draggingExited:sender
{
  return self;
}

/*-------------------------------------------------------------------------*/
/* prepareForDragOperation:                                                */
/* Invoked when the image is released.                                     */
/* NXDraggingDestination informal protocol.                                */
/*-------------------------------------------------------------------------*/
- (BOOL)prepareForDragOperation:sender
{
  return YES;
}

/*-------------------------------------------------------------------------*/
/* performDragOperation:                                                   */
/* Invoked after the released image has been removed from the screen (but  */
/* only if the previous prepareForDragOperation: message returned YES.     */
/* NXDraggingDestination informal protocol.                                */
/*-------------------------------------------------------------------------*/
- (BOOL)performDragOperation:sender
{
  id pBoard = [sender draggingPasteboard];          /* dragging pasteboard */
  char *fileName;                                     /* dragged file name */
  int fileNameLen;                            /* file name length in bytes */
  
  if (!includesType([pBoard types], NXFilenamePboardType))
     return NO;       /* does not contain a file name, reject dragging op. */
     
  if (![pBoard readType: NXFilenamePboardType 
               data: &fileName 
               length: &fileNameLen])
    return NO;                    /* pastboard error, cannot get file name */
    
  if ([controller setExecPath: fileName] == nil)
    return NO;                                    /* not a valid file name */
      
  return YES;
}

/*-------------------------------------------------------------------------*/
/* concludeDragOperation:                                                  */
/* Invoked when the dragging operation is complete (but only if the        */
/* previous performDragOperation: returned YES).                           */
/* NXDraggingDestination informal protocol.                                */
/*-------------------------------------------------------------------------*/
- concludeDragOperation:sender
{
  return self;
}

@end
