/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* NEXTSTEP PPC Control Center.                                            */
/* LogController class implementation.                                     */
/* One instance of this class is associated with each project. It controls */
/* the contents of the log scroll text.                                    */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 09.10.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: LogController.m                                               */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import "LogController.h"
#import <syslog.h>

@implementation LogController

/*-------------------------------------------------------------------------*/
/* initWithScroll:                                                         */
/* Instance initialization.                                                */
/*-------------------------------------------------------------------------*/
- initWithScroll: scrollViewP;
{
  [super init];
  scrollView = scrollViewP;
  docView = [scrollView docView];
  [docView setDelegate: self];
  machineFont = [Font newFont:"Helvetica" size:12];
  userFont = [Font newFont:"Helvetica-Bold" size:12];
  
  return self;
}

/*-------------------------------------------------------------------------*/
/* appendString:                                                           */
/* Append the given string to the log.                                     */
/*-------------------------------------------------------------------------*/
- appendString: (char *)string
  user: (BOOL)isUser
{
  int length = [docView textLength];

  [docView setSel: length: length];
  if (isUser)
    [docView setSelFont:userFont];
  else
    [docView setSelFont:machineFont];
  [docView replaceSel: string];
  [docView scrollSelToVisible];
  [docView display];
  lastTextCount = [docView textLength];
  
  return self;
}

/*-------------------------------------------------------------------------*/
/* appendString:                                                           */
/* Append the given string to the log.                                     */
/*-------------------------------------------------------------------------*/
- appendString: (char *)string
{
  [self appendString: string
    user: NO];
  
  return self;
}

/*-------------------------------------------------------------------------*/
/* setDelegate:                                                            */
/* Set the object's delegate.                                              */
/*-------------------------------------------------------------------------*/
- setDelegate: sender
{
  delegate = sender;
  
  return self;
}

/*-------------------------------------------------------------------------*/
/* textWillChange:                                           Text Delegate */
/* Moving the selection to the end before the change means the keys        */
/* the user hits will always appear at the end of the scroll view          */
/* (From Subprocess example by Joe Freeman at NeXT)                        */
/*-------------------------------------------------------------------------*/
- (BOOL)textWillChange:textObject 
{
    [docView setSel:[docView textLength] :0];
    [docView scrollSelToVisible];
    return NO;
}

/*-------------------------------------------------------------------------*/
/* textDidEnd:                                               Text Delegate */
/* Get this on every Return if using NXFieldFilter                         */
/* really simplistic.  Assumes user does not type ahead                    */
/* (From Subprocess example by Joe Freeman at NeXT)                        */
/*-------------------------------------------------------------------------*/
- textDidEnd:textObject endChar:(unsigned short)whyEnd  
{
    [docView setSel:[docView textLength] :0];
    [docView replaceSel: "\n"];
    lastTextCount = [docView textLength];
    if (delegate && [delegate respondsTo:@selector(userEntered:)])
	[delegate perform:@selector(userEntered:) with:(void *)"\n"];
    return self;
}

/*-------------------------------------------------------------------------*/
/* textDidGetKeys:                                           Text Delegate */
/* Send each character as it is entered.                                   */
/* The delta calculation is in case the user has backspaced.               */
/* (From Subprocess example by Joe Freeman at NeXT)                        */
/*-------------------------------------------------------------------------*/
- textDidGetKeys:textObject isEmpty:(BOOL)flag 
{	
    int	delta;		// the difference in this length from previous
    int	theLength;	// current length of text object
    char *theText, miniBuffer[2];
    BOOL mallocedTheText = NO;
    
    theLength = [docView textLength];
    delta = theLength - lastTextCount;
    if (delta > 0)
    {
        theText = (char *)malloc(delta + 1);
        mallocedTheText = YES;
	[docView
	    getSubstring:theText
	    start:([docView textLength]-delta)
	    length:delta];
	theText[delta] = '\0';
    }
    else
    {
        theText = miniBuffer;
	strcpy(theText,"\177");
    }
    if (delegate && [delegate respondsTo:@selector(userEntered:)])
	[delegate perform:@selector(userEntered:) with:(void *)theText];
    lastTextCount = theLength;
    
    if (mallocedTheText) {
        free(theText);
    }
    
    return self;
}

/*-------------------------------------------------------------------------*/
/* clearBuffer:                                                            */
/* Sent when the clear buffer menu option is selected.                     */
/*-------------------------------------------------------------------------*/
- clearBuffer: sender
{
  [docView selectAll: self];
  [docView replaceSel: ""];
  
  return self;
}

@end
