/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* NEXTSTEP PPC Control Center.                                            */
/* ProjectController class interface.                                      */
/* One object of this class is instanciated for each opened project.       */
/* It controls the project objects.                                        */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 28.07.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: ProjectController.h                                           */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import <appkit/appkit.h>
#import "architectures.h"

#define MAX_NAME_LEN 512

@interface ProjectController:Object
{
  id archSwitches;                            /* objects initialized by IB */
  id statusTextField;
  id optionsPanel;
  id projectWindow;
  id execPathTextField;
  id restoreDefaultButton;
  id optionsButton;
  id browserController;
  id miniTerminal;
  id procCoordinator;
  id commandScroll;
  id subProcessStringTable;
  id verboseSwitch;
  id vmcFileButtonMatrix;
  id hwFilesButtonMatrix;
  id hwFilesFormCellMatrix;
  id vmcFileFormCellMatrix;
  id logController;
  id scrollView;
  
  id centerController;                           /* application controller */
  char actProjectName[MAX_NAME_LEN];      /* name of the project with path */
  char projectPath[MAX_NAME_LEN];             /* project path without name */
  char projectName[MAX_NAME_LEN]; /* proj. name without path and extension */
  char configFileName[MAX_NAME_LEN];          /* absolute config file name */
  BOOL defaultExecPath;             /* YES if default executable directory */
  BOOL canBeRevertedToSaved;  /* YES if the proj. can be reverted to saved */
  char execPath[MAX_NAME_LEN];       /* if defaultExecPath == NO, the path */
  int selectedArch;              /* currently selected target architecture */
  char vmcFile[MAX_NAME_LEN];                         /* optional VMC file */
  char hwFiles[NB_ARCH][MAX_NAME_LEN];               /* optional HWC files */
}

/*-------------------------------------------------------------------------*/
/* setUp:                                                                  */
/* Set up a new project. Initialize the project, shows a new window and    */
/* initializes its contents.                                               */
/* In addition to the name of the project and offset of the window, this   */
/* method takes as argument the menu options which can be eventually       */
/* modified depending on the state of the project.                         */
/*-------------------------------------------------------------------------*/
- setUp: sender
  projectName: (const char *)name 
  windowOffset: (float)offset
  : menuFilesUpdate
  : menuFilesOpenInWorkspace
  : menuFilesSelectInWorkspace
  : menuFilesAddExisting
  : menuFilesCreateNew
  : menuFilesDelete;

/*-------------------------------------------------------------------------*/
/* selectedArch:                                                           */
/* Returns the selected architecture.                                      */
/*-------------------------------------------------------------------------*/
- (archEnum) selectedArch;

/*-------------------------------------------------------------------------*/
/* unselectUnsupportedArch:                                                */
/* Uselect the selected switches which are not enabled.                    */
/*-------------------------------------------------------------------------*/
- unselectUnsupportedArch;

/*-------------------------------------------------------------------------*/
/* updateExecPath:                                                         */
/* Update executable path contents and text field.                         */
/*-------------------------------------------------------------------------*/
- updateExecPath: (char *)path
  default: (BOOL)isDefault;

/*-------------------------------------------------------------------------*/
/* clearBuffer:                                                            */
/* Sent when the clear buffer menu option is selected.                     */
/*-------------------------------------------------------------------------*/
- clearBuffer: sender;

/*-------------------------------------------------------------------------*/
/* filesMenuSelected:                                                      */
/* Message sent by the center controller when one of the Files menu option */
/* is selected.                                                            */
/*-------------------------------------------------------------------------*/
- filesMenuSelected: sender;

/*-------------------------------------------------------------------------*/
/* actionsMenuSelected:                                                    */
/* Message sent by the menu when one of the Actions options is selected.   */
/*-------------------------------------------------------------------------*/
- actionsMenuSelected: sender;

/*-------------------------------------------------------------------------*/
/* updateArchitectures:                                                    */
/* Update all architectures switches and field.                            */
/*-------------------------------------------------------------------------*/
- updateArchitectures;

/*-------------------------------------------------------------------------*/
/* showOptions:                                                            */
/* Shows the options panel. This message is sent when the "Options..."     */
/* button of the project window is clicked.                                */
/*-------------------------------------------------------------------------*/
- showOptions:sender;

/*-------------------------------------------------------------------------*/
/* restoreDefaultExecPath:                                                 */
/* Restore the default executable path. This message is sent when the      */
/* "Restore default..." button of the Build options window is clicked.     */
/*-------------------------------------------------------------------------*/
- restoreDefaultExecPath: sender;

/*-------------------------------------------------------------------------*/
/* archStateChanged:                                                       */
/* Message sent by the architecture matrix when the state of a switch is   */
/* changed.                                                                */
/*-------------------------------------------------------------------------*/
- archStateChanged: sender;

/*-------------------------------------------------------------------------*/
/* setExecPath:                                                            */
/* Set the given path as the executable path.                              */
/*-------------------------------------------------------------------------*/
- setExecPath: (char *)path;

/*-------------------------------------------------------------------------*/
/* execPath:                                                               */
/* Returns the PPC servers executable path or NULL if default.             */
/*-------------------------------------------------------------------------*/
- (char *) execPath;

/*-------------------------------------------------------------------------*/
/* verboseSwitched:                                                        */
/* Sent when the verbose switch is toggled.                                */
/*-------------------------------------------------------------------------*/
- verboseSwitched: sender;

/*-------------------------------------------------------------------------*/
/* verboseSelected:                                                        */
/* Returns wether the verbose switch is enabled or not.                    */
/*-------------------------------------------------------------------------*/
- (BOOL) verboseSelected;

/*-------------------------------------------------------------------------*/
/* vmcOptionalFile:                                                        */
/* Returns the VMC file if defined, or an empty string if it is not the    */
/* case.                                                                   */
/*-------------------------------------------------------------------------*/
- (char *)vmcOptionalFile;

/*-------------------------------------------------------------------------*/
/* hwOptionalFile:                                                         */
/* Returns the hardware configuration file for the given architecture if   */
/* defined, or an empty string if it is not the case.                      */
/*-------------------------------------------------------------------------*/
- (char *)hwOptionalFile: (archEnum)arch;

/*-------------------------------------------------------------------------*/
/* vmcFileButtonMatrixClicked:                                             */
/* Sent when one of the virtual machine configuration file buttons is      */
/* clicked.                                                                */
/*-------------------------------------------------------------------------*/
- vmcFileButtonMatrixClicked: sender;

/*-------------------------------------------------------------------------*/
/* hwFilesButtonMatrixClicked:                                             */
/* Sent when one of the hardware configuration files buttons is            */
/* clicked.                                                                */
/*-------------------------------------------------------------------------*/
- hwFilesButtonMatrixClicked: sender;

/*-------------------------------------------------------------------------*/
/* projectName:                                                            */
/* Return the name of the project.                                         */
/*-------------------------------------------------------------------------*/
- (const char *)projectName;

/*-------------------------------------------------------------------------*/
/* projectNameWithoutPath:                                                 */
/* Return the name of the project but without its path.                    */
/*-------------------------------------------------------------------------*/
- (const char *)projectNameWithoutPath;

/*-------------------------------------------------------------------------*/
/* projectPath:                                                            */
/* Return the path of the project.                                         */
/*-------------------------------------------------------------------------*/
- (const char *)projectPath;

/*-------------------------------------------------------------------------*/
/* makeActive:                                                             */
/* Make the project active by making the project window the key and main   */
/* window of the application.                                              */
/*-------------------------------------------------------------------------*/
- makeActive;

/*-------------------------------------------------------------------------*/
/* isActive:                                                               */
/* Return YES if the project is active. Only one project may be active at  */
/* a time.                                                                 */
/*-------------------------------------------------------------------------*/
- (BOOL) isActive;

/*-------------------------------------------------------------------------*/
/* splitPath:                                                              */
/* Split a path into a directory and a file name.                          */
/*-------------------------------------------------------------------------*/
- splitPath: (const char *)path
  directory: (char *)dir
  fileName: (char *)file;

/*-------------------------------------------------------------------------*/
/* splitProjectName:                                                       */
/* This extract the path and the file name components from the project     */
/* full name and store the corresponding values in the instance variables. */
/*-------------------------------------------------------------------------*/
- splitProjectName;

/*-------------------------------------------------------------------------*/
/* setDefaults:                                                            */
/* Set default configuration parameters.                                   */
/*-------------------------------------------------------------------------*/
- setDefaults;

/*-------------------------------------------------------------------------*/
/* projectCorrupted:                                                       */
/* Must be sent when a project configuration file is corrupted.            */
/*-------------------------------------------------------------------------*/
- projectCorrupted: (FILE *) configFile
  status: (char *) msg;

/*-------------------------------------------------------------------------*/
/* loadConfig:                                                             */
/* Load the configuration informations of the project.                     */
/* If no configuration is defined, then define defaults and set the        */
/* project to edited.                                                      */
/*-------------------------------------------------------------------------*/
- loadConfig;

/*-------------------------------------------------------------------------*/
/* saveConfig:                                                             */
/* Save the project configuration.                                         */
/*-------------------------------------------------------------------------*/
- saveConfig;

/*-------------------------------------------------------------------------*/
/* revertToSaved:                                                          */
/* Revert project to its saved state.                                      */
/*-------------------------------------------------------------------------*/
- revertToSaved;

/*-------------------------------------------------------------------------*/
/* isEdited:                                                               */
/* Returns YES if the project needs to be saved, NO otherwise.             */
/*-------------------------------------------------------------------------*/
- (BOOL) isEdited;

/*-------------------------------------------------------------------------*/
/* setNotEdited:                                                           */
/* Set the project to be not edited.                                       */
/*-------------------------------------------------------------------------*/
- setNotEdited;

/*-------------------------------------------------------------------------*/
/* close:                                                                  */
/* Simulates a close operation on the project window. This message is      */
/* sent to the controller when closing a project.                          */
/*-------------------------------------------------------------------------*/
- close;

/*-------------------------------------------------------------------------*/
/* enableArch:                                                             */
/* Enable or disable the specified architecture. This method is called by  */
/* the browser controller when it updates the list of the available        */
/* hardware configuration file names.                                      */
/*-------------------------------------------------------------------------*/
- enableArch: (archEnum) arch: (BOOL)flag;

/*-------------------------------------------------------------------------*/
/* canBeRevertedToSaved:                                                   */
/* Return YES if the project can be reverted to saved.                     */
/*-------------------------------------------------------------------------*/
- (BOOL) canBeRevertedToSaved;

/*-------------------------------------------------------------------------*/
/* windowWillClose:                                                        */
/* Project window delegate window. Message sent when the window is closed  */
/* manually by the user or by the close: method.                           */
/*-------------------------------------------------------------------------*/
- windowWillClose: sender;

/*-------------------------------------------------------------------------*/
/* windowWillMiniaturize:                                                  */
/* Message sent when the project window is miniaturized. Allows the        */
/* definition of the miniaturized window image.                            */
/*-------------------------------------------------------------------------*/
- windowWillMiniaturize: sender 
  toMiniwindow: miniwindow;

/*-------------------------------------------------------------------------*/
/* windowDidBecomeMain:                                                    */
/* Message sent when the project window become the main window.            */
/*-------------------------------------------------------------------------*/
- windowDidBecomeMain:sender;

/*-------------------------------------------------------------------------*/
/* free:                                                                   */
/* Release dynamically allocated instance variable.                        */
/*-------------------------------------------------------------------------*/
- free;

@end
