/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* NEXTSTEP PPC Control Center.                                            */
/* Application strings definition.                                         */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 27.07.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: strings.c                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#define STRINGS_SERV

#include <stdlib.h>
#include <stdio.h>
#include "strings.h"

#define LPROJECT_NAME "English.lproj"
#define STRING_FILE_NAME "strings"
#define STRING_FILE_NAME_LEN 1000
#define MAX_STRING_LEN 10000

char **strings;

/*-------------------------------------------------------------------------*/
/* init_strings:                                                           */
/* This function must be called at least once before accessing the string  */
/* array.                                                                  */
/* It initializes the string array, and returns -1 if an error occurs, 0   */
/* otherwise.                                                              */
/*-------------------------------------------------------------------------*/
int init_strings (char *app_directory)
{
  FILE *strings_file;
  char strings_file_name [STRING_FILE_NAME_LEN];
  int act_string;
  char tmp_string [MAX_STRING_LEN];
  
  /* The strings file is assumed to be stored in the English.lproj bundle. */
  /* It is of course assumed that the application is not relocalizable.    */
  sprintf(strings_file_name, "%s/%s/%s",
    app_directory,
    LPROJECT_NAME,
    STRING_FILE_NAME);
    
  strings_file = fopen(strings_file_name, "r");
  if (strings_file == NULL)
    return -1;                                   /* strings file not found */

  strings = (char **)malloc(STR_END * sizeof(char *));
  if (strings == NULL)
    return -1;                           /* error allocating strings array */

  for (act_string = 0; act_string < STR_END; act_string++)
    strings[act_string] = NULL;               /* initializes strings array */
    
  for (act_string = 0; act_string < STR_END; act_string++)
  {
    if (feof(strings_file))
      return -1;                                 /* unexpected end of file */
    fgets(tmp_string, MAX_STRING_LEN, strings_file);
    strings[act_string] = (char *)malloc(strlen(tmp_string) + 1);
    if (strings[act_string] == NULL)
      return -1;
    strcpy(strings[act_string], tmp_string);
  }
    
  fclose(strings_file);
  return 0;
}

/*-------------------------------------------------------------------------*/
/* end_strings:                                                            */
/* This function must be called before the application termination.        */
/*-------------------------------------------------------------------------*/
void end_strings (void)
{
  int act_string;

  if (strings == NULL)
    return;
  
  for (act_string = 0; act_string < STR_END; act_string++)
    if (strings[act_string] != NULL)
      free(strings[act_string]);
      
  free(strings);
}
