# ------------------------------------------------------------------------- #
#  Mathematics on Parallel and Distributed Systems                          #
#  School of Engineering of Biel, Switzerland.                              #
#  Computer Science Dpt.                                                    #
#  Swiss National Science Foundation project 5003-034331.                   #
#  Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.     #
# ------------------------------------------------------------------------- #
#  Description:                                                             #
#  PPC: Portable Parallel C                                                 #
#  Configurer make file.                                                    #
#  This make file, called automatically by PPC when creating the            #
#  configuration informations for the parallel program, creates the         #
#  configurer.                                                              #
#  This makefile is used exclusively when configuring PACTC programs.       #
# ------------------------------------------------------------------------- #
#  History:                                                                 #
#  Date        By          Description                                      #
#  ----------  ----------- -----------------------------------------------  #
#  24.01.1994  Droux       Created                                          #
#  15.07.1994  Droux       Added conf_$(ARCH) for hardware configuration    #
#  17.08.1994  Droux       Support for custom hardware configuration file   #
#  28.08.1994  Droux       Added "." prefix to architecture directory       #
#  29.08.1994  Droux       clean target suppressed                          #
#  29.08.1994  Droux       Server installation from this makefile           #
# ------------------------------------------------------------------------- #
#  filename: config.pactc.mak                                               #
#  Tested on :                                                              #
#    Linux 0.99.pl9, Intel Computers                                        #
#    NEXTSTEP 3.2, Intel Computers                                          #
# ------------------------------------------------------------------------- #
#  Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.      #
#                                                                           #
#  This program is free software; you can redistribute it and/or modify it  #
#  under the terms of the GNU General Public License as published by the    #
#  Free Software Foundation; either version 2 of the License, or (at your   #
#  option) any later version.                                               #
#                                                                           #
#  This program is distributed in the hope that it will be useful, but      #
#  WITHOUT ANY WARRANTY; without even the implied warranty of               #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                     #
#  See the GNU General Public License for more details.                     #
# ------------------------------------------------------------------------- #

# ------------------------------------------------------------------------- #
# Custom C compiler flags.                                                  #
# ------------------------------------------------------------------------- #
CFLAGS=-I$(PPC_HOME)/include -D$(HOST_ARCH)
PCC=pcc
PCFLAGS=-I$(PPC_HOME)/include -DPACTC
LFLAGS=-lconf -lconf$(ARCHLIB)  \
  -L$(PPC_HOME)/lib/$(HOST_ARCH) $(OTHERLIB)

# ------------------------------------------------------------------------- #
# Object files for current host and PACTC.                                  #
# We have to compile two version of the configuration files: one version    #
# for the current used host such that it can be able to generate the        #
# configuration files and one version for PACTC such that the tasks can     #
# use the virtual configuration informations in order to open the channels. #
# ------------------------------------------------------------------------- #
OBJ=.$(ARCH)/$(HOST_ARCH)/$(APP_NAME).o .$(ARCH)/$(HOST_ARCH)/$(ARCH_FILE).o
PACTC_OBJ=.$(ARCH)/PACTC/$(APP_NAME).o .$(ARCH)/PACTC/$(ARCH_FILE).o

# ------------------------------------------------------------------------- #
# Main dependency.                                                          #
# ------------------------------------------------------------------------- #
all: .$(ARCH)/$(HOST_ARCH)/$(APP_NAME).con .$(ARCH)/PACTC $(PACTC_OBJ)

# ------------------------------------------------------------------------- #
# Current host binaries.                                                    #
# ------------------------------------------------------------------------- #
.$(ARCH)/$(HOST_ARCH)/$(APP_NAME).con: $(OBJ) \
  $(PPC_HOME)/lib/$(HOST_ARCH)/libconf.a \
  $(PPC_HOME)/lib/$(HOST_ARCH)/libconf$(ARCHLIB).a
	$(CC) $(OBJ) -o $@ $(LFLAGS)

.$(ARCH)/$(HOST_ARCH)/$(ARCH_FILE).o: $(ARCH_FILE).c
	$(CC) $(CFLAGS) -Dconf_$(ARCH) -c $? -o $@

.$(ARCH)/$(HOST_ARCH)/$(APP_NAME).o: .$(ARCH)/$(APP_NAME).c
	$(CC) $(CFLAGS) -Dconf_$(ARCH) -c $? -I. -o $@

.$(ARCH)/$(APP_NAME).c: $(APP_NAME).vmc
	@ cp $? $@
	@ echo "}" >> $@

# ------------------------------------------------------------------------- #
# PACTC binaries.                                                           #
# ------------------------------------------------------------------------- #
.$(ARCH)/PACTC/$(ARCH_FILE).o: $(ARCH_FILE).c
	$(PCC) $(PCFLAGS) -Dconf_$(ARCH) -c $? -o $(ARCH_FILE).o
	@mv $(ARCH_FILE).o .$(ARCH)/PACTC/$(ARCH_FILE).o

.$(ARCH)/PACTC/$(APP_NAME).o: .$(ARCH)/$(APP_NAME).c
	$(PCC) $(PCFLAGS) -Dconf_$(ARCH) -c $? -o $(APP_NAME).o
	@mv $(APP_NAME).o .$(ARCH)/PACTC/$(APP_NAME).o

# ------------------------------------------------------------------------- #
# Misc dependencies.                                                        #
# ------------------------------------------------------------------------- #
.$(ARCH)/PACTC:
	@mkdir .$(ARCH)/PACTC

install:
	@ cp .$(ARCH)/PACTC/$(APP_NAME).run $(INSTALLDIR)
