/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* School of Engineering of Biel, Switzerland.                             */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* Asynchronous reception of messages.                                     */
/* Client source.                                                          */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 22.06.1994  Droux       Created                                         */
/* 14.09.1994  Droux       Arguments processing                            */
/*-------------------------------------------------------------------------*/
/* filename: async_client.c                                                */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 School of Engineering of Biel-Bienne, Switzerland.   */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <signal.h>
#include <stdio.h>
#include <ppc/cslib.h>
#include "../include/cst.h"

void process_arguments (int, char **, architecture *, char **);

channel to_server;
channel from_server;
int count = 0;

/*-------------------------------------------------------------------------*/
/* channel_handler:                                                        */
/* This function is called each time a message is send by the sever        */
/* program.                                                                */
/*-------------------------------------------------------------------------*/
void channel_handler (
  int session_id,
  channel chan
)
{
  int val;                                   /* value received from server */

  receive_channel(chan,                         /* get message from server */
    INT, &val,
    END);

  send_channel(to_server,             /* tell server to continue execution */
    INT, CONTINUE,
    END);
    
  printf("%03d\n", val);
  fflush(stdout);
  
  count++;
}

/*-------------------------------------------------------------------------*/
/* main:                                                                   */
/* Client program entry point.                                             */
/*-------------------------------------------------------------------------*/
void main (int argc, char *argv[])
{
  int session_id;
  int val;
  char *server_host;
  architecture arch;
  
  process_arguments(argc, argv, &arch, &server_host);
    
  session_id = connect_to_server(           /* start and connect to server */
    &to_server,
    &from_server,
    "async",
    server_host,
    arch,
    "");
    
  if (session_id == -1)
  {                                     /* an error occured during startup */
    printf("Cannot connect to server %s-%s: %s\n", 
      last_server_host_name(),
      arch_string(last_server_arch()),
      ppc_strerror(ppc_errno));
    exit(1);
  }
  
  printf("connected to %s, %s\n",               /* print connection status */
    server_host_name(session_id),
    arch_string(server_arch(session_id)));
  printf("this application will terminate after 20 messages\n");

  /* Install channel handler. The function simple_handler() will be        */
  /* called each time a message has been send by the server program.       */
  if (install_channel_handler(session_id, channel_handler) == -1)
  {
    printf("Cannot install channel handler: %s\n", 
      ppc_strerror(ppc_errno));
    disconnect_from_server(session_id);    
    exit(1);
  }
  
  send_channel(to_server,                /* tell server to start execution */
    INT, CONTINUE,
    END); 

  while(count < 20)
  {
    usleep(10000);
    printf("."); fflush(stdout);
  }

  remove_channel_handler(session_id);                    /* remove handler */ 
  
  printf("\nterminating application\n");
  
  receive_channel(from_server,             /* get last message from server */
    INT, &val,
    END);
  
  send_channel(to_server,            /* tell server to terminate execution */
    INT, TERMINATE,
    END); 

  disconnect_from_server(session_id);    
  exit(0);
}

/*-------------------------------------------------------------------------*/
/* process_arguments:                                                      */
/* Extract optional server host name and architecture from arguments.      */ 
/*-------------------------------------------------------------------------*/
void process_arguments (
  int argc,
  char *argv[],
  architecture *arch,
  char **server_host
)
{
  int c;
  
  *server_host = NULL;
  *arch = UNIX;
  
  for (c = 1; c < argc; c++)
  {
    switch(argv[c][0])
    {
      case '-':
        *server_host = argv[c]+1; 
	break;
      case '+':
        if (!strcmp(argv[c]+1, "unix"))
	  *arch = UNIX;
	else if (!strcmp(argv[c]+1, "pvm"))
	  *arch = PVM;
	else if (!strcmp(argv[c]+1, "pactc"))
	  *arch = PACTC;
	else 
	{
	  printf("%s: unknown \"%s\" architecture\n", argv[0], argv[c]+1);
	  exit(1);
	}
	break;
      default: 
        break;
    }
  }
}
