/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* Asynchronous reception of messages from multiple servers.               */
/* Client source.                                                          */
/* Usage: multi_client <host1> <host2> ... <hostn>                         */
/* To simplify the usage of the client application, it is assumed that the */
/* given host do not require some password.                                */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 26.06.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: asyncm_client.c                                               */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <signal.h>
#include <stdio.h>
#include <ppc/cslib.h>
#include "../include/cst.h"

#define MAX_SESSIONS  10
#define HOST_NAME_LEN 15

channel to_server [MAX_SESSIONS];
channel from_server [MAX_SESSIONS];
int count = 0;

/*-------------------------------------------------------------------------*/
/* channel_handler:                                                        */
/* This function is called each time a message is send by the sever        */
/* program.                                                                */
/*-------------------------------------------------------------------------*/
void channel_handler (
  int session_id,
  channel chan
)
{
  int val;                                   /* value received from server */

  receive_channel(chan,                         /* get message from server */
    INT, &val,
    END);

  send_channel(to_server[session_id], /* tell server to continue execution */
    INT, CONTINUE,
    END);
    
  printf("%03d:%03d ", session_id, val);
  fflush(stdout);
  
  count++;
}

/*-------------------------------------------------------------------------*/
/* main:                                                                   */
/* Client program entry point.                                             */
/*-------------------------------------------------------------------------*/
void main (int argc, char *argv[])
{
  int session_id [MAX_SESSIONS];
  int nb_sessions;
  int act_session;
  int val;
  
  if (argc == 1)                                    
  {                                                         /* print usage */
    printf("Usage: %s <host1> <host2> ... <hostn>\n", argv[0]);
    exit(1);
  }
  
  nb_sessions = argc - 1;
  
  printf("connecting to servers...\n");
  
  for (act_session = 0; act_session < nb_sessions; act_session++)
  {
    session_id[act_session] = connect_to_server(          /* start session */
      &to_server[act_session],
      &from_server[act_session],
      "async_server",
      argv[act_session+1],
      UNKNOWN,
      NULL);
    
    if (session_id[act_session] == -1)
    {                                   /* an error occured during startup */
      printf("Cannot connect to server %s-%s: %s\n", 
        last_server_host_name(),
        arch_string(last_server_arch()),
        ppc_strerror(ppc_errno));
	
      exit(1);
    }
  
    printf("connected to %s, %s\n",             /* print connection status */
      server_host_name(session_id[act_session]),
      arch_string(server_arch(session_id[act_session])));
  }
  
  printf("all servers connected to client\n");
  
  /* Install channel handler. The function simple_handler() will be        */
  /* called each time a message has been send by the server program.       */
  printf("installing channels handlers...\n");
  for (act_session = 0; act_session < nb_sessions; act_session++)
  {  
    if (install_channel_handler(session_id[act_session], channel_handler) 
        == -1)
    {
      printf("Cannot install channel handler for session %d: %s\n", 
	act_session, ppc_strerror(ppc_errno));
      exit(1);
    }
    printf("channel handler for session %d installed\n", act_session);
  }
  
  printf("this application will terminate after 50 messages\n");

  printf("sending jobs to servers...\n");
  for (act_session = 0; act_session < nb_sessions; act_session++)
  {  
    /* Send jobs to servers. Here, we simply send to the server an integer */
    /* and we wait for the results it should send back to us.              */
    
    printf("server[%d]: sending... ", act_session);
    fflush(stdout);
    
    send_channel(to_server[act_session],
      INT, CONTINUE,
      END);
      
    printf("sent\n");
  }

  while(count < (50 * nb_sessions))
    sleep(1);

  for (act_session = 0; act_session < nb_sessions; act_session++)
    remove_channel_handler(session_id[act_session]);     /* remove handler */ 
  
  printf("\nterminating application\n");
  
  for (act_session = 0; act_session < nb_sessions; act_session++)
  {  
    receive_channel(from_server[act_session],
      INT, &val,
      END);
  
    send_channel(to_server[act_session],       /* tell server to terminate */
      INT, TERMINATE,
      END);

    printf("disconnecting from server[%d]... ", act_session);
    fflush(stdout);
    disconnect_from_server(session_id[act_session]);
    printf("ok\n");
  }
  exit(0);
}
