/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* Simple Client-Server application.                                       */
/* Client source.                                                          */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 10.09.1994  Droux       Created                                         */
/* 14.09.1994  Droux       Arguments processing                            */
/*-------------------------------------------------------------------------*/
/* filename: datatype_client.c                                             */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <limits.h>
#include <ppc/cslib.h>

#define INT_FACTOR 		2
#define INT_VALUE 		-1234
#define SHORT_VALUE 		-4321
#define UNSIGNED_SHORT_VALUE 	6789
#define LONG_VALUE 		-123456789
#define UNSIGNED_LONG_VALUE 	987654321
#define FLOAT_VALUE 		123.456789
#define DOUBLE_VALUE 		-321.987654321
#define CHAR_VALUE 		'n'
#define DELTA			0.00000001
#define ARRAY_SIZE		1000

void process_arguments (int, char **, architecture *, char **);

/*-------------------------------------------------------------------------*/
/* main:                                                                   */
/* Client program entry point.                                             */
/*-------------------------------------------------------------------------*/
void main (int argc, char *argv[])
{
  channel to_server;
  channel from_server;
  int session_id;
  char *server_host;
  architecture arch;
  
  process_arguments(argc, argv, &arch, &server_host);
    
  session_id = connect_to_server(           /* start and connect to server */
    &to_server,
    &from_server,
    "datatypes",
    server_host,
    arch,
    "");
    
  if (session_id == -1)
  {                                     /* an error occured during startup */
    printf("Cannot connect to server %s-%s: %s\n", 
      last_server_host_name(),
      arch_string(last_server_arch()),
      ppc_strerror(ppc_errno));
    exit(1);
  }
  
  printf("connected to %s, %s\n",               /* print connection status */
    server_host_name(session_id),
    arch_string(server_arch(session_id)));
    
  /* - - - - - - - - - - - - - - - - - -  atomic types in the same message */
  {
    int int_value;
    short short_value;
    unsigned short unsigned_short_value;
    long long_value;
    unsigned long unsigned_long_value;
    float float_value;
    double double_value;
    char char_value;
    
    printf("atomic types:\n");
    printf("  sending data ... "); fflush(stdout);
    send_channel(to_server,
      INT, INT_VALUE,
      SHORT, SHORT_VALUE,
      UNSIGNED_SHORT, UNSIGNED_SHORT_VALUE,
      LONG, LONG_VALUE,
      UNSIGNED_LONG, UNSIGNED_LONG_VALUE,
      FLOAT, FLOAT_VALUE,
      DOUBLE, DOUBLE_VALUE,
      CHAR, CHAR_VALUE,
      INT, INT_FACTOR,
      END);
    printf("ok\n");

    printf("  receiving ... "); fflush(stdout);
    receive_channel(from_server,
      INT, &int_value,
      SHORT, &short_value,
      UNSIGNED_SHORT, &unsigned_short_value,
      LONG, &long_value,
      UNSIGNED_LONG, &unsigned_long_value,
      FLOAT, &float_value,
      DOUBLE, &double_value,
      CHAR, &char_value,
      END);
    printf("ok\n");
    
    printf("  %d * %d = %d -> ", INT_FACTOR, INT_VALUE, int_value); 
    fflush(stdout);
    if (int_value != INT_VALUE * INT_FACTOR)
      printf("error transmitting INT value\n");
    else
      printf("ok\n");

    printf("  %d * %hd = %hd -> ", INT_FACTOR, SHORT_VALUE, short_value); 
    fflush(stdout);
    if (short_value != SHORT_VALUE * INT_FACTOR)
      printf("error transmitting SHORT value\n");
    else
      printf("ok\n");

    printf("  %d * %hu = %hu -> ", 
      INT_FACTOR, UNSIGNED_SHORT_VALUE, unsigned_short_value); 
    if (unsigned_short_value != UNSIGNED_SHORT_VALUE * INT_FACTOR)
      printf("error transmitting UNSIGNED_SHORT value\n");
    else
      printf("ok\n");

    printf("  %d * %ld = %ld -> ", INT_FACTOR, LONG_VALUE, long_value); 
    fflush(stdout);
    if (long_value != LONG_VALUE * INT_FACTOR)
      printf("error transmitting LONG value\n");
    else
      printf("ok\n");

    printf("  %d * %lu = %lu -> ", 
      INT_FACTOR, UNSIGNED_LONG_VALUE, unsigned_long_value); 
    fflush(stdout);
    if (unsigned_long_value != UNSIGNED_LONG_VALUE * INT_FACTOR)
      printf("error transmitting UNSIGNED_LONG value\n");
    else
      printf("ok\n");

    printf("  %d * %f = %f -> ", INT_FACTOR, FLOAT_VALUE, float_value); 
    fflush(stdout);
    if (float_value - FLOAT_VALUE * (float)INT_FACTOR > DELTA)
      printf("error transmitting FLOAT value\n");
    else
      printf("ok\n");

    printf("  %d * %f = %f -> ", INT_FACTOR, DOUBLE_VALUE, double_value); 
    fflush(stdout);
    if (double_value - DOUBLE_VALUE * (double)INT_FACTOR > DELTA)
      printf("error transmitting DOUBLE value\n");
    else
      printf("ok\n");

    printf("  upper('%c') = '%c' -> ", CHAR_VALUE, char_value);
    fflush(stdout);
    if (char_value != CHAR_VALUE - ('a' - 'A'))
      printf("error transmitting CHAR value\n");
    else
      printf("ok\n");
  }
  
  /* - - - - - - - - - - - - - - - - - - - - -  arrays in the same message */
  {
    int int_array [ARRAY_SIZE];
    short short_array [ARRAY_SIZE];
    unsigned short unsigned_short_array [ARRAY_SIZE];
    long long_array [ARRAY_SIZE];
    unsigned long unsigned_long_array [ARRAY_SIZE];
    float float_array [ARRAY_SIZE];
    double double_array [ARRAY_SIZE];
    char char_array [ARRAY_SIZE];
    int c;
    int ok;
    
    for (c = 0; c < ARRAY_SIZE; c++)
    {
      int_array [c] = c;
      short_array [c] = c;
      unsigned_short_array [c] = c;
      long_array [c] = c;
      unsigned_long_array [c] = c;
      float_array [c] = (float)c;
      double_array [c] = (double)c;
      char_array [c] = c % (CHAR_MAX / 2);
    }
    
    printf("arrays:\n");
    printf("  sending array size ... "); fflush(stdout);
    send_channel(to_server,
      INT, ARRAY_SIZE,
      END);
    printf("ok\n");
    
    printf("  sending arrays ... "); fflush(stdout);
    send_channel(to_server,
      INT_ARRAY, ARRAY_SIZE, int_array,
      SHORT_ARRAY, ARRAY_SIZE, short_array,
      UNSIGNED_SHORT_ARRAY, ARRAY_SIZE, unsigned_short_array,
      LONG_ARRAY, ARRAY_SIZE, long_array,
      UNSIGNED_LONG_ARRAY, ARRAY_SIZE, unsigned_long_array,
      FLOAT_ARRAY, ARRAY_SIZE, float_array,
      DOUBLE_ARRAY, ARRAY_SIZE, double_array,
      CHAR_ARRAY, ARRAY_SIZE, char_array,
      INT, INT_FACTOR,
      END);
    printf("ok\n");

    printf("  receiving ... "); fflush(stdout);
    receive_channel(from_server,
      INT_ARRAY, ARRAY_SIZE, int_array,
      SHORT_ARRAY, ARRAY_SIZE, short_array,
      UNSIGNED_SHORT_ARRAY, ARRAY_SIZE, unsigned_short_array,
      LONG_ARRAY, ARRAY_SIZE, long_array,
      UNSIGNED_LONG_ARRAY, ARRAY_SIZE, unsigned_long_array,
      FLOAT_ARRAY, ARRAY_SIZE, float_array,
      DOUBLE_ARRAY, ARRAY_SIZE, double_array,
      CHAR_ARRAY, ARRAY_SIZE, char_array,
      END);
    printf("ok\n");

    printf("  INT_ARRAY -> "); fflush(stdout);
    for (ok = 1, c = 0; c < ARRAY_SIZE && ok; c++)
      ok = int_array[c] == c * INT_FACTOR;
    if (ok)
      printf("ok\n");
    else
      printf("error during transmission\n");
      
    printf("  SHORT_ARRAY -> "); fflush(stdout);
    for (ok = 1, c = 0; c < ARRAY_SIZE && ok; c++)
      ok = short_array[c] == c * INT_FACTOR;
    if (ok)
      printf("ok\n");
    else
      printf("error during transmission\n");

    printf("  UNSIGNED_SHORT_ARRAY -> "); fflush(stdout);
    for (ok = 1, c = 0; c < ARRAY_SIZE && ok; c++)
      ok = unsigned_short_array[c] == c * INT_FACTOR;
    if (ok)
      printf("ok\n");
    else
      printf("error during transmission\n");
      
    printf("  LONG_ARRAY -> "); fflush(stdout);
    for (ok = 1, c = 0; c < ARRAY_SIZE && ok; c++)
      ok = long_array[c] == c * INT_FACTOR;
    if (ok)
      printf("ok\n");
    else
      printf("error during transmission\n");
      
    printf("  UNSIGNED_LONG_ARRAY -> "); fflush(stdout);
    for (ok = 1, c = 0; c < ARRAY_SIZE && ok; c++)
      ok = unsigned_long_array[c] == c * INT_FACTOR;
    if (ok)
      printf("ok\n");
    else
      printf("error during transmission\n");
      
    printf("  FLOAT_ARRAY -> "); fflush(stdout);
    for (ok = 1, c = 0; c < ARRAY_SIZE && ok; c++)
      ok = float_array[c] - c * (float)INT_FACTOR < DELTA;
    if (ok)
      printf("ok\n");
    else
      printf("error during transmission\n");
      
    printf("  DOUBLE_ARRAY -> "); fflush(stdout);
    for (ok = 1, c = 0; c < ARRAY_SIZE && ok; c++)
      ok = double_array[c] - c * (double)INT_FACTOR < DELTA;
    if (ok)
      printf("ok\n");
    else
      printf("error during transmission\n");
      
    printf("  CHAR_ARRAY -> "); fflush(stdout);
    for (ok = 1, c = 0; c < ARRAY_SIZE && ok; c++)
      ok = char_array[c] == (c % (CHAR_MAX / 2)) * (char) INT_FACTOR;
    if (ok)
      printf("ok\n");
    else
      printf("error during transmission\n");
  }
  
  disconnect_from_server(session_id);    
  exit(0);
}

/*-------------------------------------------------------------------------*/
/* process_arguments:                                                      */
/* Extract optional server host name and architecture from arguments.      */ 
/*-------------------------------------------------------------------------*/
void process_arguments (
  int argc,
  char *argv[],
  architecture *arch,
  char **server_host
)
{
  int c;
  
  *server_host = NULL;
  *arch = UNIX;
  
  for (c = 1; c < argc; c++)
  {
    switch(argv[c][0])
    {
      case '-':
        *server_host = argv[c]+1; 
	break;
      case '+':
        if (!strcmp(argv[c]+1, "unix"))
	  *arch = UNIX;
	else if (!strcmp(argv[c]+1, "pvm"))
	  *arch = PVM;
	else if (!strcmp(argv[c]+1, "pactc"))
	  *arch = PACTC;
	else 
	{
	  printf("%s: unknown \"%s\" architecture\n", argv[0], argv[c]+1);
	  exit(1);
	}
	break;
      default: 
        break;
    }
  }
}
