/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Master task source code.                                                */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 10.09.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: master.c                                                      */
/* Tested on :                                                             */
/*   PPC 0.9.0                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <limits.h>
#include <stdlib.h>
#include <ppc/csp.h>
#include <ppc/cslib.h>

/*-------------------------------------------------------------------------*/
/* ppc_main:                                                               */
/* Master task entry point.                                                */
/*-------------------------------------------------------------------------*/
void ppc_main (
  channel from_host,
  channel to_host
)
{
  channel to_client;
  channel from_client;

  connect_to_client(
    &to_client, 
    &from_client);
    
  /* - - - - - - - - - - - - - - - - - -  atomic types in the same message */
  {
    int int_value;
    short short_value;
    unsigned short unsigned_short_value;
    long long_value;
    unsigned long unsigned_long_value;
    float float_value;
    double double_value;
    char char_value;
    int int_factor;
    
    receive_channel(from_client,
      INT, &int_value,
      SHORT, &short_value,
      UNSIGNED_SHORT, &unsigned_short_value,
      LONG, &long_value,
      UNSIGNED_LONG, &unsigned_long_value,
      FLOAT, &float_value,
      DOUBLE, &double_value,
      CHAR, &char_value,
      INT, &int_factor,
      END);
      
    send_channel(to_client,
      INT, int_factor * int_value,
      SHORT, int_factor * short_value,
      UNSIGNED_SHORT, int_factor * unsigned_short_value,
      LONG, int_factor * long_value,
      UNSIGNED_LONG, int_factor * unsigned_long_value,
      FLOAT, (float)int_factor * float_value,
      DOUBLE, (double)int_factor * double_value,
      CHAR, char_value - ('a' - 'A'),
      END);
  }
  
  /* - - - - - - - - - - - - - - - - - - - - -  arrays in the same message */
  {
    int *int_array;
    short *short_array;
    unsigned short *unsigned_short_array;
    long *long_array;
    unsigned long *unsigned_long_array;
    float *float_array;
    double *double_array;
    char *char_array;
    int array_size;
    int c;
    int int_factor;
    
    receive_channel(from_client,
      INT, &array_size,
      END);
      
    int_array = malloc(array_size * sizeof(int));
    short_array = malloc(array_size * sizeof(short));
    unsigned_short_array = malloc(array_size * sizeof(unsigned short));
    long_array = malloc(array_size * sizeof(long));
    unsigned_long_array = malloc(array_size * sizeof(unsigned long));
    float_array = malloc(array_size * sizeof(float));
    double_array = malloc(array_size * sizeof(double));
    char_array = malloc(array_size * sizeof(char));
    
    receive_channel(from_client,
      INT_ARRAY, array_size, int_array,
      SHORT_ARRAY, array_size, short_array,
      UNSIGNED_SHORT_ARRAY, array_size, unsigned_short_array,
      LONG_ARRAY, array_size, long_array,
      UNSIGNED_LONG_ARRAY, array_size, unsigned_long_array,
      FLOAT_ARRAY, array_size, float_array,
      DOUBLE_ARRAY, array_size, double_array,
      CHAR_ARRAY, array_size, char_array,
      INT, &int_factor,
      END);

    for (c = 0; c < array_size; c++)
    {
      int_array [c] *= int_factor;
      short_array [c] *= int_factor;
      unsigned_short_array [c] *= int_factor;
      long_array [c] *= int_factor;
      unsigned_long_array [c] *= int_factor;
      float_array [c] *= (float)int_factor;
      double_array [c] *= (double)int_factor;
      char_array [c] = char_array[c] * (char) int_factor;
    }

    send_channel(to_client,
      INT_ARRAY, array_size, int_array,
      SHORT_ARRAY, array_size, short_array,
      UNSIGNED_SHORT_ARRAY, array_size, unsigned_short_array,
      LONG_ARRAY, array_size, long_array,
      UNSIGNED_LONG_ARRAY, array_size, unsigned_long_array,
      FLOAT_ARRAY, array_size, float_array,
      DOUBLE_ARRAY, array_size, double_array,
      CHAR_ARRAY, array_size, char_array,
      END);
  }

  disconnect_from_client();
}
