/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* Demonstrates the multi-session capability of CS-Lib where a single      */
/* client program can be connected to more than one server application.    */
/* Client source.                                                          */
/* Usage: multi_client <host1> <host2> ... <hostn>                         */
/* To simplify the usage of the client application, it is assumed that the */
/* given host do not require some password.                                */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 21.06.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: multi_client.c                                                */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <ppc/cslib.h>

#define MAX_SESSIONS  10
#define HOST_NAME_LEN 15

void main (int argc, char *argv[])
{
  channel to_server [MAX_SESSIONS];
  channel from_server [MAX_SESSIONS];
  int session_id [MAX_SESSIONS];
  int nb_sessions;
  int act_session;
  int result;
  
  if (argc == 1)                                    
  {                                                         /* print usage */
    printf("Usage: %s <host1> <host2> ... <hostn>\n", argv[0]);
    exit(1);
  }
  
  nb_sessions = argc - 1;
  
  for (act_session = 0; act_session < nb_sessions; act_session++)
  {
    session_id[act_session] = connect_to_server(          /* start session */
      &to_server[act_session],
      &from_server[act_session],
      "multi_server",
      argv[act_session+1],
      UNKNOWN,
      NULL);
    
    if (session_id[act_session] == -1)
    {                                   /* an error occured during startup */
      printf("Cannot connect to server %s-%s: %s\n", 
        last_server_host_name(),
        arch_string(last_server_arch()),
        ppc_strerror(ppc_errno));
	
      exit(1);
    }
  
    printf("connected to %s, %s\n",             /* print connection status */
      server_host_name(session_id[act_session]),
      arch_string(server_arch(session_id[act_session])));
  }
  
  printf("all servers connected to client\n");
  
  for (act_session = 0; act_session < nb_sessions; act_session++)
  {  
    /* Send jobs to servers. Here, we simply send to the server an integer */
    /* and we wait for the results it should send back to us.              */
    
    printf("server[%d]: sending... ", act_session);
    fflush(stdout);
    
    send_channel(to_server[act_session],
      INT, act_session,
      END);
      
    printf("sent\n");
  }

  for (act_session = 0; act_session < nb_sessions; act_session++)
  {  
    printf("server[%d]: receiving... ", act_session);
    fflush(stdout);
    
    receive_channel(from_server[act_session],
      INT, &result,
      END);
      
    printf("received %d\n", result);
  }

  for (act_session = 0; act_session < nb_sessions; act_session++)
  {
    printf("disconnecting from server[%d]... ", act_session);
    fflush(stdout);
    disconnect_from_server(session_id[act_session]);
    printf("ok\n");
  }
  exit(0);
}
