/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Nfact: calculates the number of factors of numbers and returns the one  */
/* which have the more factors.                                            */
/* NEXTSTEP client Controller class.                                       */
/* One object of this class is instanciated at startup. It controls the    */
/* interface objects.                                                      */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 15.07.1993  Droux       Created                                         */
/* 10.09.1994  Droux       Server renamed to nfact                         */
/*-------------------------------------------------------------------------*/
/* filename: Controller.m                                                  */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import "Controller.h"
#import <ppc/NXPPCKit.h>

#define SLEEPING_TEXT "The server is now sleeping. \nSend it a calculation \
request to wake it up."
#define WORKING_TEXT "The server is now processing your request. \nPlease \
wait for the result..."

@implementation Controller

/*-------------------------------------------------------------------------*/
/* getRange:                                                               */
/* Initialize the instance variables lowerLimit and upperLimit with from   */
/* the values stored in the lowerLimitCell and upperLimitCell.             */
/* Returned YES if the values are correct, NO otherwise.                   */
/*-------------------------------------------------------------------------*/
- (BOOL) getRange
{
  if ((sscanf([lowerLimitCell stringValue], "%d", &lowerLimit) != 1) ||
  	    (sscanf([upperLimitCell stringValue], "%d", &upperLimit) != 1))
  {
    NXRunAlertPanel("Range Invalid",
      "Specified numbers are not valid integers.",
      NULL, NULL, NULL);
    return NO;
  }
  
  if ((lowerLimit < 0) || (upperLimit < 0))
  {
    NXRunAlertPanel("Range Invalid",
      "The lower limit or the upper limit must be positive numbers.",
      NULL, NULL, NULL);
    return NO;
  }

  if (lowerLimit > upperLimit)
  {
    NXRunAlertPanel("Range Invalid",
      "The upper limit must be greater than the lower limit.",
      NULL, NULL, NULL);
    return NO;
  }
  
  return YES;
}

/*-------------------------------------------------------------------------*/
/* calculate:                                                              */
/* Message sent when the user clicks on the START button. Start a          */
/* calculation.                                                            */
/*-------------------------------------------------------------------------*/
- calculate: sender
{
  if (![self getRange])
    return nil;
  
  [server sendChannel:
    INT, lowerLimit,
    INT, upperLimit,
    END];
    
  [[statusTextField cell] setStringValue: WORKING_TEXT];
    
  [quitMenuItem setEnabled: NO];
  [startButton setEnabled: NO];

  return self;
}

/*-------------------------------------------------------------------------*/
/* channelHandler:                                                         */
/* Installed at startup as the channel handler for the active CS session.  */
/* This method is called by NXPPCKit when a message is comming from        */
/* the server.                                                             */
/*-------------------------------------------------------------------------*/
- channelHandler: theServer
{
  [theServer receiveChannel:
    INT, &winner,
    INT, &count,
    END];

  NXRunAlertPanel("Calculation Results",
    "Winner is %d with %d divisors",
    NULL, NULL, NULL,
    winner,
    count);

  [[statusTextField cell] setStringValue: SLEEPING_TEXT];

  [quitMenuItem setEnabled: YES];
  [startButton setEnabled: YES];
  
  return self;
}

@end

@implementation Controller(ApplicationDelegate)

/*-------------------------------------------------------------------------*/
/* appDidInit:                                                             */
/* Called just after application initialization.                           */
/*-------------------------------------------------------------------------*/
- appDidInit: sender
{
  server = [[PPCServer alloc] init];
  
  if ([server connectToServer: "nfact"
    loginWindow: loginWindow
    useLoginWindow: YES
    serverHost: NULL
    serverArch: UNKNOWN
    password: NULL
    errorPanel: YES] == nil)
  {                        /* error connecting to server, quit application */
    server = [server free];
    [NXApp terminate: self];
  }
  
  [server installChannelHandler: self
    handler: @selector(channelHandler:)];

  NXRunAlertPanel("Connection Successful",
    "You are now connected to %s, %s server.",
    NULL, NULL, NULL,
    [server serverHost],
    [server archString: [server serverArch]]);
    
  [[statusTextField cell] setStringValue: SLEEPING_TEXT];
  [controlWindow makeKeyAndOrderFront: sender];
  
  return self;
}

/*-------------------------------------------------------------------------*/
/* appWillTerminate:                                                       */
/* Called just before the application terminates.                          */
/*-------------------------------------------------------------------------*/
- appWillTerminate: sender
{
  if (server != nil)
  {                                                 /* server is connected */
    [server removeChannelHandler];
    [server sendChannel:               /* send terminate message to server */
      INT, 0,
      INT, 0,
      END];
    [server disconnectFromServer];
    server = [server free];
  }

  return self;
}

@end
