/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Master task.                                                            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 27.11.1993  Droux       Created                                         */
/* 01.07.1994  Droux       CS-Lib version                                  */
/* 15.07.1994  Droux       Master process requests until terminate msg     */
/*-------------------------------------------------------------------------*/
/* filename: master.c                                                      */
/* Tested on :                                                             */
/*   PPC 0.9.0                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <ppc/csp.h>
#include <ppc/cslib.h>

/*-------------------------------------------------------------------------*/
/* doit:                                                                   */
/* Divide the work to be done, send requests to the slaves and wait for    */
/* the result.                                                             */
/*-------------------------------------------------------------------------*/
void doit(
  channel to_slaves,                        /* requests to the first slave */
  channel from_slaves,                      /* result from the first slave */
  int nb_slaves,                                 /* total number of slaves */
  int lower,                                                /* lower limit */
  int upper,                                                /* upper limit */
  int *winner,                                    /* set to winner on exit */
  int *count         /* set to the number of factors of the winner on exit */
)
{
  int numbers_per_slave;         /* # of numbers to analyse per slave task */
  int act_slave;                                           /* actual slave */
  int act_upper;
  int act_lower;

                                      /* distribute the work to the slaves */
  numbers_per_slave = (upper-lower)/nb_slaves;
  for (act_slave = 0; act_slave < nb_slaves; act_slave++)
  {
    act_lower = lower+(numbers_per_slave*act_slave);
    act_upper = act_lower+numbers_per_slave;
    if (act_slave == nb_slaves)
      act_upper = upper;

    send_channel(to_slaves, 
      INT, act_lower,
      INT, act_upper,
      END);
  }
  
  if ((lower != 0) || (upper != 0))
    receive_channel(from_slaves,
      INT, winner,
      INT, count,
      END);
}

/*-------------------------------------------------------------------------*/
/* ppc_main:                                                               */
/* Master task entry point.                                                */
/*-------------------------------------------------------------------------*/
void ppc_main (
  channel to_host,
  channel from_host,
  channel to_slaves,
  channel from_slaves,
  int nb_slaves
)
{
  channel to_client;
  channel from_client;
  int lower;                                                /* lower limit */
  int upper;                                                /* upper limit */
  int winner;                         /* number with the most # of factors */
  int count;                                  /* and its number of factors */
  
  connect_to_client(
    &to_client, 
    &from_client);

  do
  {
    receive_channel(from_client, 
      INT, &lower, 
      INT, &upper,
      END);
      
    doit(to_slaves, from_slaves, nb_slaves, 
      lower, upper, &winner, &count);

    if ((lower != 0) || (upper != 0))
    {
      send_channel(to_client,
	INT, winner,
	INT, count,
	END);
    }
  } while ((lower != 0) || (upper != 0));

  disconnect_from_client();
}
