/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Slave task.                                                             */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 24.11.1993  Droux       Created                                         */
/* 15.07.1994  Droux       Slave process requests until terminate msg      */
/*-------------------------------------------------------------------------*/
/* filename: slave.c                                                       */
/* Tested on :                                                             */
/*   PPC 0.9.0                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <ppc/csp.h>

/*-------------------------------------------------------------------------*/
/* calculate:                                                              */
/* Calculates the winner and its count of numbers between the given range. */
/*-------------------------------------------------------------------------*/
void calculate (
  int lower,                                                /* lower limit */
  int upper,                                                /* upper limit */
  int *winner,                                /* winner number set on exit */
  int *count                                              /* and its count */
)
{
  int act_number;                         /* actual number being processed */
  int act_count;                 /* number of factors of the actual number */
  
  act_number = lower;
  *count = 0;
  while(act_number <= upper)
  {
    act_count = nfact(act_number);
    if (act_count > *count)
    {
      *count = act_count;
      *winner = act_number;
    }
    act_number++;
  }
}

/*-------------------------------------------------------------------------*/
/* ppc_main:                                                               */
/* Slave task entry point.                                                 */
/*-------------------------------------------------------------------------*/
void ppc_main (
  channel from_north,
  channel to_north,
  channel to_south,
  channel from_south,
  int nb_slaves,
  int my_number
)
{
  int lower;
  int upper;
  int winner;
  int count;
  int act_slave;
  
  do
  {
    receive_channel(from_north,                            /* get my request */
      INT, &lower,
      INT, &upper, 
      END);
      
    for (act_slave = 0; act_slave < (nb_slaves-(my_number+1)); act_slave++)
    {                          /* forward remaining requests to other slaves */
      int tmp_lower;
      int tmp_upper;
      
      receive_channel(from_north,
	INT, &tmp_lower,
	INT, &tmp_upper,
	END);
      
      send_channel(to_south,
	INT, tmp_lower,
	INT, tmp_upper,
	END);  
    }
    
    if ((lower != 0) || (upper != 0))
    {
      calculate(lower, upper, &winner, &count);    /* determine the winner */
	
      if (my_number < nb_slaves-1)            
      {        /* get result from preceding slave and compare it with mine */
	int pred_winner;
	int pred_count;
	
	receive_channel(from_south,
	  INT, &pred_winner,
	  INT, &pred_count,
	  END);
	
	if(pred_count > count)
	{
	  winner = pred_winner;
	  count = pred_count;
	}
      }
      
      send_channel(to_north,
	INT, winner,
	INT, count,
	END);
    } 
  } while ((lower != 0) || (upper != 0));
}
