/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* Simple Client-Server application.                                       */
/* Client source.                                                          */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 08.02.1994  Droux       Created                                         */
/* 30.08.1994  Droux       Server renamed from simple_server to simple     */
/* 14.09.1994  Droux       Arguments processing                            */
/*-------------------------------------------------------------------------*/
/* filename: simple_client.c                                               */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT & Intel                                            */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <ppc/cslib.h>

#define NB_ITEMS 100

void process_arguments (int, char **, architecture *, char **);

/*-------------------------------------------------------------------------*/
/* main:                                                                   */
/* Client entry-point.                                                     */
/*-------------------------------------------------------------------------*/
void main (int argc, char *argv[])
{
  channel to_server;
  channel from_server;
  int session_id;
  short items [NB_ITEMS];
  short result;
  int c;
  char *server_host;
  architecture arch;
  
  process_arguments(argc, argv, &arch, &server_host);
    
  session_id = connect_to_server(           /* start and connect to server */
    &to_server,
    &from_server,
    "simple",
    server_host,
    arch,
    "");
    
  if (session_id == -1)
  {                                     /* an error occured during startup */
    printf("Cannot connect to server %s-%s: %s\n", 
      last_server_host_name(),
      arch_string(last_server_arch()),
      ppc_strerror(ppc_errno));
    exit(1);
  }
  
  printf("connected to %s, %s\n",               /* print connection status */
    server_host_name(session_id),
    arch_string(server_arch(session_id)));
   
  for (c = 0; c < NB_ITEMS; c++)
    items[c] = c;
    
  send_channel(to_server, 
    UNSIGNED_SHORT, NB_ITEMS, 
    END);
  
  send_channel(to_server,
    SHORT_ARRAY, NB_ITEMS, items,
    END);
  
  receive_channel(from_server,
    SHORT, &result,
    END);
    
  printf("result is %hd\n", result);

  disconnect_from_server(session_id);    
  exit(0);
}

/*-------------------------------------------------------------------------*/
/* process_arguments:                                                      */
/* Extract optional server host name and architecture from arguments.      */ 
/*-------------------------------------------------------------------------*/
void process_arguments (
  int argc,
  char *argv[],
  architecture *arch,
  char **server_host
)
{
  int c;
  
  *server_host = NULL;
  *arch = UNIX;
  
  for (c = 1; c < argc; c++)
  {
    switch(argv[c][0])
    {
      case '-':
        *server_host = argv[c]+1; 
	break;
      case '+':
        if (!strcmp(argv[c]+1, "unix"))
	  *arch = UNIX;
	else if (!strcmp(argv[c]+1, "pvm"))
	  *arch = PVM;
	else if (!strcmp(argv[c]+1, "pactc"))
	  *arch = PACTC;
	else 
	{
	  printf("%s: unknown \"%s\" architecture\n", argv[0], argv[c]+1);
	  exit(1);
	}
	break;
      default: 
        break;
    }
  }
}
