/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Master task.                                                            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 16.01.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: master.c                                                      */
/* Tested on :                                                             */
/*   PPC 0.9.0                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <ppc/csp.h>

#define NB_THREADS  10

/*-------------------------------------------------------------------------*/
/* master:                                                                 */
/* Indefinitely listen to the given channels and print the received slave  */
/* identifier.                                                             */
/*-------------------------------------------------------------------------*/
void master (
  channel c [],
  int nb_slaves
)
{
  int winner;
  int x;
  int n;
  
  for (;;)
  { 
    alt_begin();
      alt_for (n = 0; n < nb_slaves; n++)
        alt_channel(1, c[n]);
      alt_timeout(SECONDS/2);
    alt_end(&winner);
    
    if (winner == TIMEOUT)
      printf("    T");
    else
    {
      receive_channel(winner, INT, &x, END);
      printf("%5d", x); 
    }
    fflush(stdout);
  }
}

/*-------------------------------------------------------------------------*/
/* slave:                                                                  */
/* Indefinitely send a message to the master.                              */
/*-------------------------------------------------------------------------*/
void slave (
  channel c,
  int nb_slaves,
  int n
)
{
  int delay;
  
  srand(n);
  rand();
  
  for (;;)
  {
    delay = (int)(((double)rand() / (double)RAND_MAX) * nb_slaves);
    sleep(delay);
    send_channel(c, INT, n+nb_slaves, END);
  }
}

/*-------------------------------------------------------------------------*/
/* ppc_main:                                                               */
/* Master task entry point.                                                */
/*-------------------------------------------------------------------------*/
void ppc_main (
  channel to_host,
  channel from_host,
  channel c [],
  int nb_slaves
)
{
  int n;
  channel channels [1000];
  channel d [NB_THREADS];
  
  for (n = 0; n < NB_THREADS; n++)
   d[n] = new_channel();
   
  for (n = 0; n < nb_slaves; n++)
    channels[n] = c[n];
  for (n = 0; n < NB_THREADS; n++)
    channels[n+nb_slaves] = d[n];
    
  par_begin();
    par_for (n = 0; n < NB_THREADS; n++)
      par(slave(d[n], nb_slaves, n+1));
    par(master(channels, NB_THREADS+nb_slaves));
  par_end();
}
