/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Master task implementation.                                             */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 07.12.1993  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: master.c                                                      */
/* Tested on :                                                             */
/*   PPC 0.9.0                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <ppc/csp.h>

#define NB_THREADS  10

/*-------------------------------------------------------------------------*/
/* m:                                                                      */
/* This function is incarnated as a thread which listens from the          */
/* specified channels and prints the obtained results.                     */
/*-------------------------------------------------------------------------*/
void m (channel c[])
{
  int winner;
  int x;
  int n;
  
  for (;;)
  { 
    alt_begin();
      alt_for (n = 0; n < NB_THREADS; n++)
        alt_channel(1, c[n]);
      alt_timeout(SECONDS/2);
    alt_end(&winner);
    
    if (winner == TIMEOUT)
      printf("    T");
    else
    {
      receive_channel(winner, INT, &x, END);
      printf("%5d", x); 
    }
    fflush(stdout);
  }
}

/*-------------------------------------------------------------------------*/
/* s:                                                                      */
/* This functions is incarnated as NB_THREADS threads of the master task.  */
/* It sends messages to the task m.                                        */
/*-------------------------------------------------------------------------*/
void s (channel c, int n)
{
  int delay;
  
  srand(n);
  rand();
  
  for (;;)
  {
    delay = (int)(((double)rand() / (double)RAND_MAX) * NB_THREADS);
    sleep(delay);
    send_channel(c, 
      INT, n, 
      END);
  }
}

/*-------------------------------------------------------------------------*/
/* ppc_main:                                                               */
/* Master task entry point.                                                */
/*-------------------------------------------------------------------------*/
void ppc_main (
  channel from_host,
  channel to_host
)
{
  int n;
  channel c [NB_THREADS];

  for (n = 0; n < NB_THREADS; n++)
   c[n] = new_channel();

  par_begin();
    par_for (n = 0; n < NB_THREADS; n++)
      par(s(c[n], n+1));
    par(m(c));
  par_end();
}
