/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* NFACT master task.                                                      */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 27.11.1993  Droux       Created                                         */
/* 18.08.1994  Droux       Topology changed from linear chain to farm      */
/*-------------------------------------------------------------------------*/
/* filename: master.c                                                      */
/* Tested on :                                                             */
/*   PPC 0.9.0                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <ppc/csp.h>

/*-------------------------------------------------------------------------*/
/* doit:                                                                   */
/* Divide the work to be done, send requests to the slaves and wait for    */
/* the result.                                                             */
/*-------------------------------------------------------------------------*/
void doit(
  channel to_slaves[],                               /* requests to slaves */
  channel from_slaves[],                        /* results from the slaves */
  int nb_slaves,                                 /* total number of slaves */
  int lower,                                                /* lower limit */
  int upper,                                                /* upper limit */
  int *winner,                                    /* set to winner on exit */
  int *count         /* set to the number of factors of the winner on exit */
)
{
  int numbers_per_slave;         /* # of numbers to analyse per slave task */
  int act_slave;                                           /* actual slave */
  int act_upper;
  int act_lower;
  int act_winner;
  int act_count;
 
                                      /* distribute the work to the slaves */
  numbers_per_slave = (upper-lower)/nb_slaves;
  for (act_slave = 0; act_slave < nb_slaves; act_slave++)
  {
    act_lower = lower+(numbers_per_slave*act_slave);
    act_upper = act_lower+numbers_per_slave;
    if (act_slave == nb_slaves)
      act_upper = upper;

    send_channel(to_slaves[act_slave], 
      INT, act_lower,
      INT, act_upper,
      END);
  }
  
  *count = 0;
  for (act_slave = 0; act_slave < nb_slaves; act_slave++)
  {
    receive_channel(from_slaves[act_slave],
      INT, &act_winner,
      INT, &act_count,
      END);
    
    if (act_count > *count)
    {
      *count = act_count;
      *winner = act_winner;
    }
  }
}

/*-------------------------------------------------------------------------*/
/* ppc_main:                                                               */
/* Master task entry point.                                                */
/*-------------------------------------------------------------------------*/
void ppc_main (
  channel to_host,
  channel from_host,
  channel to_slaves [],
  channel from_slaves [],
  int nb_slaves
)
{
  char tmp_str[100];
  int lower;                                                /* lower limit */
  int upper;                                                /* upper limit */
  int winner;                         /* number with the most # of factors */
  int count;                                  /* and its number of factors */
  
  printf("lower limit : "); fflush(stdout);
  gets(tmp_str); sscanf(tmp_str, "%d", &lower);
  printf("upper limit : "); fflush(stdout);
  gets(tmp_str); sscanf(tmp_str, "%d", &upper);

  doit(to_slaves, from_slaves, nb_slaves, lower, upper, &winner, &count);
  printf("the winner is %d with %d divisors\n", winner, count);
}
