/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* NFACT Slave task.                                                       */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 24.11.1993  Droux       Created                                         */
/* 18.08.1994  Droux       Topology changed from linear chain to farm.     */
/*-------------------------------------------------------------------------*/
/* filename: slave.c                                                       */
/* Tested on :                                                             */
/*   PPC 0.9.0                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <ppc/csp.h>

/*-------------------------------------------------------------------------*/
/* calculate:                                                              */
/* Calculates the winner and its count of numbers between the given range. */
/*-------------------------------------------------------------------------*/
void calculate (
  int lower,                                                /* lower limit */
  int upper,                                                /* upper limit */
  int *winner,                                /* winner number set on exit */
  int *count                                              /* and its count */
)
{
  int act_number;                         /* actual number being processed */
  int act_count;                 /* number of factors of the actual number */
  
  act_number = lower;
  *count = 0;
  while(act_number <= upper)
  {
    act_count = nfact(act_number);
    if (act_count > *count)
    {
      *count = act_count;
      *winner = act_number;
    }
    act_number++;
  }
}

/*-------------------------------------------------------------------------*/
/* ppc_main:                                                               */
/* Slave task entry point.                                                 */
/*-------------------------------------------------------------------------*/
void ppc_main (
  channel from_master,
  channel to_master
)
{
  int lower;
  int upper;
  int winner;
  int count;
  int act_slave;
  
  receive_channel(from_master,                           /* get my request */
    INT, &lower,
    INT, &upper, 
    END);
    
  calculate(lower, upper, &winner, &count);        /* determine the winner */
    
  send_channel(to_master,
    INT, winner,
    INT, count,
    END);
}
