/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Virtual machine configuration module interface.                         */
/* This file must be included by the file which implements the virtual     */
/* machine configuration routine virtual_machine().                        */
/* These functions are implemented by the file config.c.                   */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 17.11.1993  Droux       Created                                         */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: config.h                                                      */
/* Tested on :                                                             */
/*   NEXTSTEP 2.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, Intel Computers                                         */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef config_h
#define config_h

#include <stdio.h>
#include <ppc/csp/types.h>

#ifdef SUN4
#include <stddef.h>
#else
#include <stdlib.h>
#endif

#ifdef conf_pactc
#define virtual_machine(x,y) virtual_machine(x,y) { \
  printf("starting\r");
#endif

/*-------------------------------------------------------------------------*/
/* New types implemented by this module.                                   */
/*-------------------------------------------------------------------------*/
typedef int processor;
typedef int task;
typedef int channel;
typedef int implementation;

/*-------------------------------------------------------------------------*/
/* Definition of the maximum number of tasks, processors, and channels.    */
/*-------------------------------------------------------------------------*/
#define MAX_PROCESSORS           256
#define MAX_TASKS                2*MAX_PROCESSORS
#define MAX_CHANNELS             4*MAX_TASKS
#define MAX_CHANNELS_PER_TASK    100
#define MAX_TASKS_PER_PROCESSOR  10
#define MAX_PARAMETERS_PER_TASK  20

/*-------------------------------------------------------------------------*/
/* The following constant must be used to terminate a list of modules      */
/* during the definition of a task implementation.                         */
/*-------------------------------------------------------------------------*/
#define END_OF_MODULES NULL

/*-------------------------------------------------------------------------*/
/* new_processor:                                                          */
/* Creates a new processor.                                                */
/*-------------------------------------------------------------------------*/
processor new_processor (void);

/*-------------------------------------------------------------------------*/
/* new_channel:                                                            */
/* Creates a new channel.                                                  */
/*-------------------------------------------------------------------------*/
channel new_channel (void);

/*-------------------------------------------------------------------------*/
/* new_implementation:                                                     */
/* Declare a new task implementation.                                      */
/* An implementation is a collection of modules which contain the source   */
/* code of the task. This function takes as parameter a serie of names     */
/* which corresponds to the name of the source files without extension.    */
/* This list must be terminated by the NULL pointer.                       */
/* For example, one can declare a task implementation as follows:          */
/*  implementation master_i = new_implementation("master", "loadb", NULL); */
/*-------------------------------------------------------------------------*/
#ifndef config_impl
implementation new_implementation (char *module_name, ...);
#endif

/*-------------------------------------------------------------------------*/
/* new_task:                                                               */
/* Declare a new task.                                                     */
/* This function returns the id of a new task. It must be used to define   */
/* the processor on which the task will be executed, the implementation    */
/* which implements it, and the parameters that must be passed to the      */
/* task at startup.                                                        */
/* The general form of this function is:                                   */
/*   t = new_task(                                                         */
/*     <processor>,                                                        */
/*     <implementation>,                                                   */
/*     <parameters>                                                        */
/*   );                                                                    */
/* The parameters are a list of tuples of the form <type>, <value> if the  */
/* value is atomic or <type>, <nitems>, <value> if the value is an array   */
/* where type is a constant defined in this file describing the type of the*/
/* parameter. The list of tuples must be terminated by the constant END.   */
/* Allowed types identifiers are defined in <ppc/types.h>.                 */
/*-------------------------------------------------------------------------*/
#ifndef config_impl
task new_task (
  processor p,
  implementation i,
  ...
);
#endif

/*-------------------------------------------------------------------------*/
/* Include the hardware configuration files if necesary.                   */
/*-------------------------------------------------------------------------*/

#ifdef conf_unix
#include <ppc/config/confunix.h>
#endif

#ifdef conf_pvm
#include <ppc/config/confpvm.h>
#endif

#ifdef conf_pactc
#include <ppc/config/confpact.h>
#endif

#endif
