/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* PACT-C Hardware configuration interface.                                */
/* The functions defined in this file must be called from the function     */
/* hardware_config() implemented by the user.                              */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 18.11.1993  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: confpact.h                                                    */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*   PACTC v92.1                                                           */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef confpact_h
#define confpact_h

typedef enum
{
  CHAN_INPUT,
  CHAN_OUTPUT
} direction_type;

typedef enum
{
  CONNECTED,
  NOT_CONNECTED
} connection_status_type;

#ifndef chanpact_h

#include <ppc/config.h>

/*-------------------------------------------------------------------------*/
/* New types and definitions specific to the transputer PACT-C environment.*/
/*-------------------------------------------------------------------------*/

typedef int transputer;                  /* transputer node on the network */

typedef enum                                       /* types of transputers */
{
  T212, M212, T222, T225, T400, T414, T425, T800, T801, T805
} tran_types;

#define MAX_NB_TRANS 100      /* maximum number of transputer in a network */

/* It is possible to place more than one virtual processor on a single     */
/* transputer. MAX_PROCS_PER_TRANS defines the maximum number of virtual   */
/* processors that can be placed on one transputer.                        */
#define MAX_PROCS_PER_TRANS 5

#define HOST (transputer)-200          /* the host is a special transputer */
#define HOST_LINK        -1                                   /* host link */

#define K 1024                                           /* memory factors */
#define M K*K                    /* for example 16*M for 16 Mega bytes RAM */

/*-------------------------------------------------------------------------*/
/* new_transputer:                                                         */
/* Creates a new transputer and returns its id.                            */
/*-------------------------------------------------------------------------*/
transputer new_transputer (
  tran_types type,             /* transputer types (T222, T400, T805, ...) */
  unsigned long memory                       /* transputer memory in bytes */
);

/*-------------------------------------------------------------------------*/
/* link_transputers:                                                       */
/* Declares the transputers physical connections.                          */
/*-------------------------------------------------------------------------*/
void link_transputers (
  transputer tran1, int link1, 
  transputer tran2, int link2
);

/*-------------------------------------------------------------------------*/
/* map_transputer:                                                         */
/* Maps a virtual processor on a given transputer.                         */
/*-------------------------------------------------------------------------*/
void place_processor_on_transputer (
  processor proc, 
  transputer tran
);

/*-------------------------------------------------------------------------*/
/* is_connected:                                                           */
/* Determine if a channel is connected or not. The channel is specified    */
/* as the parameter of a task.                                             */
/* This function is called only by a PACT-C main stub before starting      */
/* the PPC task entry point.                                               */
/*-------------------------------------------------------------------------*/
connection_status_type is_connected (
  int task_id,                                          /* task identifier */
  int param_id                                        /* task parameter id */
);

/*-------------------------------------------------------------------------*/
/* is_connected_in_array:                                                  */
/* Determine if a channel is connected or not. The channel is specified    */
/* as the element of a parameter array of a specified task.                */
/* This function is called only by a PACT-C main stub before starting      */
/* the PPC task entry point.                                               */
/*-------------------------------------------------------------------------*/
connection_status_type is_connected_in_array (
  int task_id,                                          /* task identifier */
  int param_id,                                 /* task array parameter id */
  int array_idx             /* index of the element in the array parameter */
);

#endif
#endif
