/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Definitions local to the configurer implementation modules and not      */
/* exported to the user program.                                           */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 17.11.1993  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: confpriv.h                                                    */
/* Tested on :                                                             */
/*   NEXTSTEP 2.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, Intel Computers                                         */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef confpriv_h
#define confpriv_h

#include <ppc/config.h>

/*-------------------------------------------------------------------------*/
/* Structures needed to store the processors, tasks, and channels          */
/* properties.                                                             */ 
/*-------------------------------------------------------------------------*/

#define MAX_MODULE_NAME_LEN    20
#define MAX_OBJECTS_PER_MODULE 20
#define MAX_TASKS_PER_MODULE   20
#define MAX_MODULES            20

typedef int module;                         /* task modules implementation */

typedef struct                                            /* modules_descr */
{                                   /* implementation modules informations */
  int free;                                      /* 1 if free, 0 otherwise */
  char name [MAX_MODULE_NAME_LEN];               /* module collection name */
  int nb_names;                                 /* number of modules_names */
  char modules_names [MAX_OBJECTS_PER_MODULE][MAX_MODULE_NAME_LEN]; 
  int nb_tasks;                 /* number of tasks these modules implement */
  task tasks [MAX_TASKS_PER_MODULE]; /* tasks implemented by these modules */
} modules_descr;

typedef struct                                          /* array parameter */
{
  int nitems;
  void *address;
} parameter_ptr;

typedef union                                           /* parameter value */
{                                              /* possible parameter value */
  channel channel_val;
  int int_val;
  short short_val;
  unsigned short unsigned_short_val;
  long long_val;
  unsigned long unsigned_long_val;
  float float_val;
  double double_val;
  char char_val;
} parameter_value;

typedef union                                                 /* parameter */
{
  parameter_value value;
  parameter_ptr ptr;
} parameter_descr;

typedef struct                                                /* parameter */
{                                            /* task parameter description */
  int type;                  /* parameter type: OUTPUT_CHANNEL, SHORT, ... */
  parameter_descr param;                                /* parameter value */
} parameter;

typedef struct                                            /* channel_descr */
{                                                  /* channel informations */
  int free;                                 /* 1 if free slot, 0 otherwise */
  task from;                                      /* id of the source task */
  task to;                                        /* id of the target task */
} channel_descr;

typedef struct                                               /* task_descr */
{                                                     /* task informations */
  int free;                                 /* 1 if free slot, 0 otherwise */
  channel channels [MAX_CHANNELS_PER_TASK];        /* channels of the task */
  processor on_processor;/* id of the processor on which the task will run */
  int nb_parameters;                               /* number of parameters */
  parameter parameters [MAX_PARAMETERS_PER_TASK];       /* task parameters */
  module implemented;                  /* module which implements the task */
} task_descr;

typedef struct                                          /* processor_descr */
{                                                /* processor informations */
  int free;                                 /* 1 if free slot, 0 otherwise */
  task tasks [MAX_TASKS_PER_PROCESSOR];  /* tasks running on the processor */
} processor_descr;

/*-------------------------------------------------------------------------*/
/* The variables above are defined in config.c.                            */
/*-------------------------------------------------------------------------*/
#ifndef config_impl
extern processor_descr processors [MAX_PROCESSORS];
extern task_descr tasks [MAX_TASKS];
extern channel_descr channels [MAX_CHANNELS];
extern modules_descr modules [MAX_MODULES];
#endif

/*-------------------------------------------------------------------------*/
/* Host channels are special channels that are not initialized by the      */
/* user. In order to detect them during the configuration, we declare      */
/* them here.                                                              */
/*-------------------------------------------------------------------------*/
#define FROM_HOST_CHANNEL_ID -100
#define TO_HOST_CHANNEL_ID   -200

/*-------------------------------------------------------------------------*/
/* virtual_machine:                                                        */
/* Virtual machine configuration. This function must be implemented by the */
/* PPC programmer.                                                         */
/*-------------------------------------------------------------------------*/
void virtual_machine(channel from_host, channel to_host);

/*-------------------------------------------------------------------------*/
/* free_config_data:                                                       */
/* Release the eventual data allocated during the virtual machine          */
/* configuration. This function is called even by the configurer main      */
/* function or from the starter at startup time.                           */
/* This function is implemented in the config.c source file.               */
/*-------------------------------------------------------------------------*/
void free_config_data(void);

#endif
