/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* PVM Hardware configuration interface.                                   */
/* The functions defined in this file must be called from the function     */
/* hardware_config() implemented by the user.                              */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 07.01.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: confpvm.h                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef confpvm_h
#define confpvm_h

#include <ppc/config.h>

#define MAX_HOST_NAME_LEN 50

#ifndef confpvm_serv
                          /* for each virtual processor, its physical host */
extern char hosts [MAX_PROCESSORS][MAX_HOST_NAME_LEN];     
#endif

/*-------------------------------------------------------------------------*/
/* place_processor_on_host:                                                */
/* Place a virtual processor on a physical host given by its internet      */
/* name. If name is NULL, then the processor is placed on any host         */
/* automatically by the PVM system.                                        */
/* Called by the PVM hardware configuration function.                      */
/*-------------------------------------------------------------------------*/
void place_processor_on_host(processor proc, const char *host_name);

/*-------------------------------------------------------------------------*/
/* start_pvm_tasks:                                                        */
/* Called by the main PVM task which is responsible to start all other     */
/* tasks and to give them all information about the parallel program such  */
/* as the tasks's pids. Only one task of the parallel program must call    */
/* this function. This task is the one which takes the to_host and         */
/* from_host channels as parameters because it must be able to             */
/* communicate with the user by stdout and stdin.                          */
/* This functions uses the virtual machine configuration informations and  */
/* thus must be called after the function virtual_machine().               */
/* It returns the tid array containing the task ids of the started tasks   */
/* as well as the caller's tid.                                            */
/* my_task_nb and name must be set to the correct value when calling.      */
/* name corresponds to the name of the PPC program.                        */
/*-------------------------------------------------------------------------*/
void start_pvm_tasks (int my_task_nb, char *name, int tids [], int my_tid);

/*-------------------------------------------------------------------------*/
/* check_main_task:                                                        */
/* Checks if one of the tasks implemented by the given module is the main  */
/* task. The main task is the task which communicates with the user and    */
/* which starts the other tasks.                                           */
/* Returns the task number if it is the case, -1 if it isn't.              */
/* This function is used during the files generation as well as during the */
/* execution of the tasks to determine its number.                         */
/*-------------------------------------------------------------------------*/
int check_main_task (int impl);

#endif
