/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* CS-Lib API. These functions are implemented by the modules cliconn and  */
/* clichan for client applications.                                        */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 07.02.1994  Droux       Created                                         */
/* 22.06.1994  Droux       Added cliasync.h inclusion                      */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: cslib.h                                                       */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef cslib_h
#define cslib_h

/* the following constant correspond to the length of the message sent     */
/* by the server to the client when establishing a TCP socket connection.  */
#define PORT_NAME_LEN 20

#ifdef _CSCLIENT
                                       /* included by a client application */
#include <ppc/csp/types.h>
#include <ppc/cslib/clichan.h>          
#include <ppc/cslib/error.h> 
#include <ppc/cslib/cliasync.h>   

#ifndef NULL
#define NULL (char *)0
#endif

#define MAX_NB_SESSIONS 10  /* maximum # client-server sessions per client */

typedef enum                      /* possible host architecture specifiers */
{
  UNIX,
  PVM,
  PACTC,
  UNKNOWN
} architecture;

#include <ppc/cslib/hlist.h>    /* requires the definition of architecture */

/*-------------------------------------------------------------------------*/
/* connect_to_server:                                                      */
/* Start and connect to the server specified by prog_name, host_name,      */
/* arch, and evt. password and returns the two channels to_server and      */
/* from_server.                                                            */
/* Returns the session identifier if the server has been correctly         */
/* started, or -1 if an error occured.                                     */
/*-------------------------------------------------------------------------*/
int connect_to_server (
  channel *to_server,                             /* channel to the server */
  channel *from_server,                         /* channel from the server */
  char *prog_name,                                  /* server program name */
  char *host_name,                                     /* server host name */
  architecture arch,                                /* server architecture */
  char *password                                   /* facultative password */
);

/*-------------------------------------------------------------------------*/
/* disconnect_from_server:                                                 */
/* Disconnect from the given server.                                       */
/* This function returns only when the corresponding server is terminated. */
/*-------------------------------------------------------------------------*/
void disconnect_from_server(
  int session_id
);

/*-------------------------------------------------------------------------*/
/* server_host_name:                                                       */
/* Returns a pointer to a string containing the name of the server         */
/* corresponding to the given session identifier.                          */
/* Returns NULL if session_id does not correspond to a valid session.      */
/*-------------------------------------------------------------------------*/
char *server_host_name(
  int session_id
);

/*-------------------------------------------------------------------------*/
/* server_arch:                                                            */
/* Returns the architecture of the server corresponding to the given       */
/* session identifier.                                                     */
/* Returns UNKNOWN if session_id does not correspond to a valid session.   */
/*-------------------------------------------------------------------------*/
architecture server_arch(
  int session_id
);

/*-------------------------------------------------------------------------*/
/* last_server_host_name:                                                  */
/* Returns a pointer to a string containing the name of the last server    */
/* for which a connection has been tried.                                  */
/* Can be used for error management.                                       */
/*-------------------------------------------------------------------------*/
char *last_server_host_name(void);

/*-------------------------------------------------------------------------*/
/* last_server_arch:                                                       */
/* Returns the architecture of the last server for which a connection has  */
/* been tried or established.                                              */
/*-------------------------------------------------------------------------*/
architecture last_server_arch(void);

#else
                                              /* included by a PPC program */
#include <ppc/csp/chan.h>

/*-------------------------------------------------------------------------*/
/* connect_to_client:                                                      */
/* This function must be the first function called when a PPC program is   */
/* used as a CS-Lib server. It returns two channels that must be used      */
/* to send and receive messages to and from the client.                    */
/* Returns -1 if the connection failed, 0 otherwise.                       */
/*-------------------------------------------------------------------------*/
int connect_to_client (
  channel *to_client,
  channel *from_client
);

/*-------------------------------------------------------------------------*/
/* disconnect_from_client:                                                 */
/* Must be called before terminating the program when the PPC program is   */
/* used as a server application.                                           */
/*-------------------------------------------------------------------------*/
void disconnect_from_client (void);

#endif 

#endif
