/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* Client channels definitions and API. The client channels are provided   */
/* to allows non-PPC custom application to communicate with a PPC          */
/* program using PPC-like channels.                                        */
/* The functions send_channel and receive_channel are identical to the     */
/* corresponding PPC channels functions.                                   */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 08.02.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: clichan.h                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef clichan_h
#define clichan_h

typedef int channel;                                 /* channel identifier */

/*-------------------------------------------------------------------------*/
/* send_channel:                                                           */
/* Send the given data on the given channel.                               */
/* The parameters list is a list of tuples. A tuple can be:                */
/*    1. atomic value:    <type_spec>, <value>                             */
/*    2. array:           <type_spec>, <nitems>, <value address>           */
/*    3. termination:     END                                              */
/* For example:                                                            */
/*    int a;                                                               */
/*    float b [100];                                                       */
/*    channel c;                                                           */
/*    send_channel(c, INT, a, FLOAT_ARRAY, 100, b, END);                   */
/*-------------------------------------------------------------------------*/
#ifndef clichan_impl
void send_channel (channel id, ...);
#endif

/*-------------------------------------------------------------------------*/
/* receive_channel:                                                        */
/* Get the given data from the specified channel.                          */
/* The parameters list is a list of tuples. A tuple can be:                */
/*    1. atomic value:    <type_spec>, <value address>                     */
/*    2. array:           <type_spec>, <nitems>, <value address>           */
/*    3. termination:     END                                              */
/* For example:                                                            */
/*    int a;                                                               */
/*    float b [100];                                                       */
/*    channel c;                                                           */
/*    send_channel(c, INT, &a, FLOAT_ARRAY, 100, b, END);                  */
/*-------------------------------------------------------------------------*/
#ifndef clichan_impl
void receive_channel (channel id, ...);
#endif

#endif
