/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Error management.                                                       */
/* The PPC error management is similar to the UNIX one. If a routine       */
/* failed, then it returns a value of -1 and it sets the global variable   */
/* ppc_errno to a code which corresponds to the error. It is possible      */
/* to obtain the message description corresponding to the actual error     */
/* by calling the function ppc_strerror(), for example:                    */
/*   fprintf(strerror, "ERROR: %s\n", ppc_strerror(ppc_errno));            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 09.02.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: error.h                                                       */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef error_h
#define error_h

/*-------------------------------------------------------------------------*/
/* Exported global variables.                                              */
/*-------------------------------------------------------------------------*/
extern int ppc_errno;                    /* can be set by any PPC function */
extern char *ppc_errlist[];                 /* errors textual descriptions */
                                                    
/*-------------------------------------------------------------------------*/
/* Exported error codes.                                                   */
/*-------------------------------------------------------------------------*/
#define EPPC_HLIST_UNKARCH		0           
#define EPPC_HLIST_LOEQ			1
#define EPPC_HLIST_LOVAL		2
#define EPPC_HLIST_EPEQ			3
#define EPPC_HLIST_EPVAL		4
#define EPPC_HLIST_UNKOPT		5
#define EPPC_HLIST_ITMEXC		6
#define EPPC_HLIST_PARTDEF		7
#define EPPC_HLIST_ARCHNFND		8
#define EPPC_HLIST_HOSTNFND		9
#define EPPC_HLIST_HOSTARCHNFND		10
#define EPPC_HLIST_BADIDNA		11
#define EPPC_HLIST_BADIDAR		12
#define EPPC_HFIND_NODEF		13        
#define EPPC_CONN_OOFSLOTS		14        
#define EPPC_CONN_PIPECRE		15
#define EPPC_CONN_FORK			16
#define EPPC_HLIST_BADIDPW		17
#define EPPC_HLIST_BADIDPR		18
#define EPPC_CONN_EXEC			19
#define EPPC_CONN_RSHF			20
#define EPPC_CONN_BADP			21
#define EPPC_CONN_PATH			22
#define EPPC_CONN_PROG			23
#define EPPC_CONN_COMP			24
#define EPPC_CONN_HOST			25
#define EPPC_CONN_PERM			26
#define EPPC_HLIST_BADIDLI		27
#define EPPC_CONN_SOCK			28
#define EPPC_CHAN_OUTOF			29
#define EPPC_CONN_SCANF			30
#define EPPC_CONN_PARE			31
#define EPPC_CONN_UNHO			32
#define EPPC_ASYNC_BADSE		33
#define EPPC_ASYNC_SETOWN		34
#define EPPC_ASYNC_FASYNC		35
#define EPPC_ASYNC_SIG			36
#define EPPC_ASYNC_TOOMANY		37
#define EPPC_ASYNC_GETFL		38
#define EPPC_CONN_COMPAT		39
#define EPPC_CONN_PVMN			40
#define EPPC_HLIST_NOHLIST		41
#define EPPC_HLIST_NOHOSTDEF		42
#define EPPC_LPANEL_CANCELED		43
#define EPPC_LPANEL_MULTIHOST		44

/*-------------------------------------------------------------------------*/
/* ppc_strerror:                                                           */
/* Returns a pointer to a textual description of the error corresponding   */
/* to the given error code.                                                */
/*-------------------------------------------------------------------------*/
char *ppc_strerror (int code);

#endif
