/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* Hosts file list management. Every manipulation of the hosts list must   */
/* be done via this module.                                                */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 08.02.1994  Droux       Created                                         */
/* 09.07.1994  Droux       Added MAX_HOST_NAME_LEN                         */
/* 11.07.1994  Droux       Added MAX_LO_LEN and MAX_EP_LEN                 */
/* 11.07.1994  Droux       Added hlist_nb_hosts()                          */
/* 11.07.1994  Droux       MAX_HOSTS moved from cslib.c to cslib.h         */
/* 11.07.1994  Droux       nb_arch in .c replaced by NB_ARCH in .h         */
/*-------------------------------------------------------------------------*/
/* filename: hlist.h                                                       */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef hlist_h
#define hlist_h

#include <ppc/cslib.h>

#define MAX_HOSTS 30                            /* maximum number of hosts */
#define MAX_HOST_NAME_LEN 100               /* maximum length of host name */
#define MAX_LO_LEN 30                    /* maximum alternative login name */
#define MAX_EP_LEN 100                      /* maximum executable path len */
#define NB_ARCH 3                      /* number of possible architectures */

/*-------------------------------------------------------------------------*/
/* arch_string:                                                            */
/* Return a pointer to the string corresponding to the given architecture. */
/* For example, arch_string(UNIX) will return a pointer to a string        */
/* containing "UNIX".                                                      */
/*-------------------------------------------------------------------------*/
char *arch_string(architecture arch);

/*-------------------------------------------------------------------------*/
/* read_hosts_list:                                                        */
/*                                                                         */
/* Read the hosts list file. This function is automatically called during  */
/* the first host interrogation.                                           */
/*                                                                         */
/* Return a negative value for unsuccessful call and stores an error code  */
/* in errno, otherwise returns zero. Possible errors are:                  */
/*                                                                         */
/*   [EPPC_HLIST_UNKARCH]	unknown architecture specifier in hosts    */
/*				list.                                      */
/*   [EPPC_HLIST_LOEQ] 		missing '=' for \"lo\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_LOVAL]		missing value for \"lo\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_EPEQ]		missing '=' for \"ep\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_EPVAL]		missing value for \"ep\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_UNKOPT]	unknown option in hosts list.              */
/*   [EPPC_HLIST_ITMEXC]	too many items in hosts list line.         */
/*   [EPPC_HLIST_PARTDEF]	host only partially defined in hosts list. */
/*                                                                         */
/*-------------------------------------------------------------------------*/
int hlist_read ();

/*-------------------------------------------------------------------------*/
/* hlist_find_host_name:                                                   */
/*                                                                         */
/* Finds a host of the given architecture and returns its id.              */
/*                                                                         */
/* Return a negative value for unsuccessful call and stores an error code  */
/* in errno. Possible errors are:                                          */
/*                                                                         */
/*   [EPPC_HLIST_UNKARCH]	unknown architecture specifier in hosts    */
/*				list.                                      */
/*   [EPPC_HLIST_LOEQ]		missing '=' for \"lo\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_LOVAL]		missing value for \"lo\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_EPEQ]		missing '=' for \"ep\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_EPVAL]		missing value for \"ep\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_UNKOPT]	unknown option in hosts list.              */
/*   [EPPC_HLIST_ITMEXC]	too many items in hosts list line.         */
/*   [EPPC_HLIST_PARTDEF]	host only partially defined in hosts list. */
/*   [EPPC_HLIST_ARCHNFND]	specified architecture not found in hosts  */
/*                              list.                                      */
/*                                                                         */
/*-------------------------------------------------------------------------*/
int hlist_find_host_name (architecture arch);

/*-------------------------------------------------------------------------*/
/* hlist_find_host_arch:                                                   */
/*                                                                         */
/* Finds a host of the given name and returns its id.                      */
/*                                                                         */
/* Return a negative value for unsuccessful call and stores an error code  */
/* in errno. Possible errors are:                                          */
/*                                                                         */
/*   [EPPC_HLIST_UNKARCH]	unknown architecture specifier in hosts    */
/*				list.                                      */
/*   [EPPC_HLIST_LOEQ] 		missing '=' for \"lo\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_LOVAL]		missing value for \"lo\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_EPEQ]		missing '=' for \"ep\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_EPVAL]		missing value for \"ep\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_UNKOPT]	unknown option in hosts list.              */
/*   [EPPC_HLIST_ITMEXC]	too many items in hosts list line.         */
/*   [EPPC_HLIST_PARTDEF]	host only partially defined in hosts list. */
/*   [EPPC_HLIST_HOSTNFND]	specified host name not found in hosts     */
/*                              list.                                      */
/*                                                                         */
/*-------------------------------------------------------------------------*/
int hlist_find_host_arch (char *host_name);

/*-------------------------------------------------------------------------*/
/* hlist_find_host_name_arch:                                              */
/*                                                                         */
/* Finds a host of the given name AND architecture and returns its id.     */
/*                                                                         */
/* Return a negative value for unsuccessful call and stores an error code  */
/* in errno. Possible errors are:                                          */
/*                                                                         */
/*   [EPPC_HLIST_UNKARCH]	unknown architecture specifier in hosts    */
/*				list.                                      */
/*   [EPPC_HLIST_LOEQ] 		missing '=' for \"lo\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_LOVAL]		missing value for \"lo\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_EPEQ]		missing '=' for \"ep\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_EPVAL]		missing value for \"ep\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_UNKOPT]	unknown option in hosts list.              */
/*   [EPPC_HLIST_ITMEXC]	too many items in hosts list line.         */
/*   [EPPC_HLIST_PARTDEF]	host only partially defined in hosts list. */
/*   [EPPC_HLIST_HOSTARCHNFND]	specified host name with given             */
/*                              architecture not found in hosts list.      */
/*                                                                         */
/*-------------------------------------------------------------------------*/
int hlist_find_host_name_arch (char *host_name, architecture arch);

/*-------------------------------------------------------------------------*/
/* hlist_get_host_name:                                                    */
/*                                                                         */
/* Copy in the string str the host name corresponding to the given         */
/* host id.                                                                */
/*                                                                         */
/* Return a negative value for unsuccessful call and stores an error code  */
/* in errno, otherwise returns zero. Possible errors are:                  */
/*                                                                         */
/*   [EPPC_HLIST_UNKARCH]	unknown architecture specifier in hosts    */
/*				list.                                      */
/*   [EPPC_HLIST_LOEQ] 		missing '=' for \"lo\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_LOVAL]		missing value for \"lo\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_EPEQ]		missing '=' for \"ep\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_EPVAL]		missing value for \"ep\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_UNKOPT]	unknown option in hosts list.              */
/*   [EPPC_HLIST_ITMEXC]	too many items in hosts list line.         */
/*   [EPPC_HLIST_PARTDEF]	host only partially defined in hosts list. */
/*   [EPPC_HLIST_BADIDNA]	bad host id given when inquiring host      */
/*				name, internal error.                      */
/*                                                                         */
/*-------------------------------------------------------------------------*/
int hlist_get_host_name (
  int id, 
  char str[]
);

/*-------------------------------------------------------------------------*/
/* hlist_get_host_name_ptr:                                                */
/* Returns a pointer to the host name corresponding to the given host id,  */
/* NULL if an error occured. Similar to hlist_get_host_name with only a    */
/* different interface.                                                    */
/*-------------------------------------------------------------------------*/
char *hlist_get_host_name_ptr (
  int id
);

/*-------------------------------------------------------------------------*/
/* hlist_get_host_arch:                                                    */
/*                                                                         */
/* Copy in arch the architecture corresponding to the given host.          */
/*                                                                         */
/* Return a negative value for unsuccessful call and stores an error code  */
/* in errno, otherwise returns zero. Possible errors are:                  */
/*                                                                         */
/*   [EPPC_HLIST_UNKARCH]	unknown architecture specifier in hosts    */
/*				list.                                      */
/*   [EPPC_HLIST_LOEQ] 		missing '=' for \"lo\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_LOVAL]		missing value for \"lo\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_EPEQ]		missing '=' for \"ep\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_EPVAL]		missing value for \"ep\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_UNKOPT]	unknown option in hosts list.              */
/*   [EPPC_HLIST_ITMEXC]	too many items in hosts list line.         */
/*   [EPPC_HLIST_PARTDEF]	host only partially defined in hosts list. */
/*   [EPPC_HLIST_BADIDAR]	bad host id given when inquiring host      */
/*				architecture, internal error.              */
/*                                                                         */
/*-------------------------------------------------------------------------*/
int hlist_get_host_arch (int id, architecture *arch);

/*-------------------------------------------------------------------------*/
/* hlist_pw_required:                                                      */
/*                                                                         */
/* Returns 1 if the given host requires a password when starting a server, */
/* 0 if it does not, and -1 if an error occured.                           */
/*                                                                         */
/* Possible errors are:                                                    */
/*   [EPPC_HLIST_UNKARCH]	unknown architecture specifier in hosts    */
/*				list.                                      */
/*   [EPPC_HLIST_LOEQ] 		missing '=' for \"lo\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_LOVAL]		missing value for \"lo\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_EPEQ]		missing '=' for \"ep\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_EPVAL]		missing value for \"ep\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_UNKOPT]	unknown option in hosts list.              */
/*   [EPPC_HLIST_ITMEXC]	too many items in hosts list line.         */
/*   [EPPC_HLIST_PARTDEF]	host only partially defined in hosts list. */
/*   [EPPC_HLIST_BADIDPW]	bad host id given when inquiring host      */
/*				password necessity, internal error.        */
/*-------------------------------------------------------------------------*/
int hlist_pw_required (
  int id
);

/*-------------------------------------------------------------------------*/
/* hlist_exec_path_required:                                               */
/*                                                                         */
/* Returns 1 if a custom executable file has been defined for the given    */
/* host, 0 if not, and -1 if an error occured.                             */
/*                                                                         */
/* Possible errors are:                                                    */
/*                                                                         */
/*   [EPPC_HLIST_UNKARCH]	unknown architecture specifier in hosts    */
/*				list.                                      */
/*   [EPPC_HLIST_LOEQ] 		missing '=' for \"lo\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_LOVAL]		missing value for \"lo\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_EPEQ]		missing '=' for \"ep\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_EPVAL]		missing value for \"ep\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_UNKOPT]	unknown option in hosts list.              */
/*   [EPPC_HLIST_ITMEXC]	too many items in hosts list line.         */
/*   [EPPC_HLIST_PARTDEF]	host only partially defined in hosts list. */
/*   [EPPC_HLIST_BADIDPR]      bad host id given when inquiring for       */
/*				custom exec path presence.                 */
/*-------------------------------------------------------------------------*/
int hlist_exec_path_required (
  int id
);

/*-------------------------------------------------------------------------*/
/* hlist_exec_path:                                                        */
/*                                                                         */
/* Returns a pointer to a string containing the custom executable          */
/* path for the given host. Returns NULL if not defined or if an error     */
/* occured.                                                                */
/*-------------------------------------------------------------------------*/
char *hlist_exec_path (
  int id
);

/*-------------------------------------------------------------------------*/
/* hlist_exec_lo_defined:                                                  */
/*                                                                         */
/* Returns 1 if a custom login name has been defined for the given         */
/* host, 0 if not, and -1 if an error occured.                             */
/*                                                                         */
/* Possible errors are:                                                    */
/*                                                                         */
/*   [EPPC_HLIST_UNKARCH]	unknown architecture specifier in hosts    */
/*				list.                                      */
/*   [EPPC_HLIST_LOEQ] 		missing '=' for \"lo\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_LOVAL]		missing value for \"lo\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_EPEQ]		missing '=' for \"ep\" option in hosts     */
/*				list.                                      */
/*   [EPPC_HLIST_EPVAL]		missing value for \"ep\" option in hosts   */
/*				list.                                      */
/*   [EPPC_HLIST_UNKOPT]	unknown option in hosts list.              */
/*   [EPPC_HLIST_ITMEXC]	too many items in hosts list line.         */
/*   [EPPC_HLIST_PARTDEF]	host only partially defined in hosts list. */
/*   [EPPC_HLIST_BADIDLI]      bad host id given when inquiring for       */
/*				custom exec path presence.                 */
/*-------------------------------------------------------------------------*/
int hlist_exec_lo_defined (
  int id
);

/*-------------------------------------------------------------------------*/
/* hlist_custom_name:                                                      */
/*                                                                         */
/* Returns a pointer to a string containing the custom user name           */
/* for the given host. Returns NULL if not defined or an error             */
/* occured.                                                                */
/*-------------------------------------------------------------------------*/
char *hlist_custom_name (
  int id
);

/*-------------------------------------------------------------------------*/
/* hlist_nb_hosts:                                                         */
/* Returns the number of hosts declared in the hosts list.                 */
/*-------------------------------------------------------------------------*/
int hlist_nb_hosts (void);

#endif
