/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* User channel interface.                                                 */
/* Channels definitions common for every implementation.                   */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 25.11.1993  Droux       Created                                         */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: chan.h                                                        */
/* Tested on :                                                             */
/*   NEXTSTEP 2.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, Intel Computers                                         */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef chan_h
#define chan_h

#include <ppc/csp/types.h>

#define END -1                   /* end of parameters for send and receive */

/*-------------------------------------------------------------------------*/
/* New types.                                                              */
/*-------------------------------------------------------------------------*/
typedef int channel;                                 /* channel identifier */

/*-------------------------------------------------------------------------*/
/* chan_init:                                                              */
/* Initializes the channels implementation. This function is never called  */
/* directly by the programmer.                                             */
/*-------------------------------------------------------------------------*/
void chan_init (void);

/*-------------------------------------------------------------------------*/
/* par_channel:                                                            */
/* Creates a new channel to communicate between two parallel threads.      */
/*-------------------------------------------------------------------------*/
channel par_channel (void);
#define new_channel par_channel

/*-------------------------------------------------------------------------*/
/* chan_close:                                                             */
/* Close the given channel.                                                */
/*-------------------------------------------------------------------------*/
void chan_close (channel id);

/*-------------------------------------------------------------------------*/
/* send_channel:                                                           */
/* Send the given data on the given channel.                               */
/* The parameters list is a list of tuples. A tuple can be:                */
/*    1. atomic value:    <type_spec>, <value>                             */
/*    2. array:           <type_spec>, <nitems>, <value address>           */
/*    3. termination:     END                                              */
/* For example:                                                            */
/*    int a;                                                               */
/*    float b [100];                                                       */
/*    channel c;                                                           */
/*    send_channel(c, INT, a, FLOAT_ARRAY, 100, b, END);                   */
/*-------------------------------------------------------------------------*/
#ifndef chan_impl
void send_channel (channel id, ...);
#endif

/*-------------------------------------------------------------------------*/
/* receive_channel:                                                        */
/* Get the given data from the specified channel.                          */
/* The parameters list is a list of tuples. A tuple can be:                */
/*    1. atomic value:    <type_spec>, <value address>                     */
/*    2. array:           <type_spec>, <nitems>, <value address>           */
/*    3. termination:     END                                              */
/* For example:                                                            */
/*    int a;                                                               */
/*    float b [100];                                                       */
/*    channel c;                                                           */
/*    send_channel(c, INT, &a, FLOAT_ARRAY, 100, b, END);                  */
/*-------------------------------------------------------------------------*/
#ifndef chan_impl
void receive_channel (channel id, ...);
#endif

/*-------------------------------------------------------------------------*/
/* chan_write:                                                   TEMPORARY */
/* Write the given buffer of given length on the specified channel.        */
/* TEMPORARY FUNCTION FOR TEST PURPOSE.                                    */
/*-------------------------------------------------------------------------*/
int chan_write (channel id, void *vbuf, int len);

/*-------------------------------------------------------------------------*/
/* chan_read:                                                    TEMPORARY */
/* Read the given buffer of given length from the specified channel.       */
/* TEMPORARY FUNCTION FOR TEST PURPOSE.                                    */
/*-------------------------------------------------------------------------*/
int chan_read (channel id, void *vbuf, int len);

#endif
