/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* PACT-C channels implementation.                                         */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 26.01.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: chanpact.h                                                    */
/* Tested on :                                                             */
/*   PACT-C 93.1, T800, T805                                               */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef chanpact_h
#define chanpact_h

#include <ppc/config/confpact.h>

/*-------------------------------------------------------------------------*/
/* open_pactc_channel:                                                     */
/* Open a new PACT-C channel and initialize it with the value given        */
/* as parameter. Returns the channel identifier.                           */
/* This function is called during the startup phase to map a PACT-C        */
/* channel on a PPC channel.                                               */
/*-------------------------------------------------------------------------*/
channel open_pactc_channel (
  void |* data_channel,                                    /* data channel */
  char |* sync_channel,                          /* sychronization channel */
  direction_type direction,                     /* input or output channel */
  connection_status_type connected                     /* connected or not */
);

/*-------------------------------------------------------------------------*/
/* open_pactc_channels_array:                                              */
/* Opens an array of new PACT-C channel and initialize it with the values  */
/* given as parameter. The channels ids are stored in the array of         */
/* channels given as parameter. Returns this array.                        */
/*-------------------------------------------------------------------------*/
channel *open_pactc_channels_array (
  void |* data_channels[],   /* PACT-C data channels, as given at start-up */
  char |* sync_channels[],            /* idem but synchronization channels */
  channel ids[],                        /* where to store the PPC channels */
  int nitems,                                        /* number of channels */
  direction_type direction,                     /* input or output channel */
  connection_status_type connected[]                   /* connected or not */
);

/*-------------------------------------------------------------------------*/
/* get_sync_pactc_chan:                                                    */
/* Return the synchronization PACT-C channel corresponding to the          */
/* specified PPC channel.                                                  */
/* This function is used by the alternate construct.                       */
/*-------------------------------------------------------------------------*/
char |*get_sync_pactc_chan (channel id);

#endif
