/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel Programming Language.                            */
/* Lower-Half PVM channels interface.                                      */
/* The functions found in this files are implemented in chanpvmlh.c.       */
/* These functions are used by the PVM PPC implementations exclusively.    */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 07.01.1994  Droux       Created                                         */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: chanpvmlh.h                                                   */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef chanpvmlh_h
#define chanpvmlh_h

#include <ppc/csp/chan.h>
#include <ppc/csp/chanpriv.h>

/*-------------------------------------------------------------------------*/
/* pvm_chan_init:                                                          */
/* Initializes the lower-half PVM channel implementation.                  */
/* This function is never directly called by the programmer but when the   */
/* parallel program is started.                                            */
/*-------------------------------------------------------------------------*/
void pvm_chan_init (void);

/*-------------------------------------------------------------------------*/
/* lh_new_pvm_channel:                                                     */
/* Declares a new lower-half PVM channel.                                  */
/*-------------------------------------------------------------------------*/
channel lh_new_pvm_channel (int tid, int tag);

/*-------------------------------------------------------------------------*/
/* pvm_chan_write:                                                         */
/* Write the given data items to the specified channel.                    */
/* <nitems> corresponds to the number of items of type <type> to be        */
/* transmitted.                                                            */
/*-------------------------------------------------------------------------*/
int pvm_chan_write (channel id, void *vbuf, int nitems, int type);

/*-------------------------------------------------------------------------*/
/* pvm_chan_read:                                                          */
/* Read the given data items from the specified channel.                   */
/* <nitems> corresponds to the number of items of type <type> to be        */
/* received.                                                               */
/*-------------------------------------------------------------------------*/
int pvm_chan_read (channel id, void *vbuf, int nitems, int type);

/*-------------------------------------------------------------------------*/
/* pvm_start_write:                                                        */
/* Starts a channel transmission of data. Must be called before any call   */
/* to pvm_chan_write for each message transmission.                        */
/*-------------------------------------------------------------------------*/
void pvm_start_write (void);

/*-------------------------------------------------------------------------*/
/* pvm_end_write:                                                          */
/* Terminates a channel transmission of data. Must be called after all     */
/* calls to pvm_chan_write for each message transmission.                  */
/*-------------------------------------------------------------------------*/
void pvm_end_write (channel id);

/*-------------------------------------------------------------------------*/
/* pvm_start_read:                                                         */
/* Starts a channel reception of data. Must be called before any call      */
/* to pvm_chan_read for each message reception.                            */
/*-------------------------------------------------------------------------*/
void pvm_start_read (channel id);

/*-------------------------------------------------------------------------*/
/* pvm_tid_and_tag:                                                        */
/* Returns the tid and tag corresponding to the given lower-half channel.  */
/* This function is used by the PVM alternate construct.                   */
/*-------------------------------------------------------------------------*/
void pvm_tid_and_tag (channel id, int *tid, int *tag);

/*-------------------------------------------------------------------------*/
/* pvm_disable_recv:                                                       */
/* Calling this function will disable the next call to pvm_start_read on   */
/* the specified channel. This function is used by the PVM alternate       */
/* construct in which the equivalent of pvm_start_read is called when data */
/* is available on the channel.                                            */
/* Warning: if two calls to pvm_disable_recv are made before calling       */
/* pvm_start_read, the effect of the first call is not taken in account.   */
/*-------------------------------------------------------------------------*/
void pvm_disable_recv (channel id);

#endif
