/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Upper-Half PVM channels specific functions interface.                   */
/* The functions found in this files are implemented in chanpvmuh.c.       */
/* These functions are used by the PVM PPC implementations exclusively.    */
/* They are used implemented by the PVM upper-half but are not accessible  */
/* from the user and are thus defined in a separate header file.           */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 12.01.1994  Droux       Created                                         */
/* 28.06.1994  Droux       Added CSLIB_CHAN type of channel for CS-Lib     */
/* 28.06.1994  Droux       Added new_cslib_channel()                       */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: chanpvmuh.h                                                   */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef chanpvmuh_h
#define chanpvmuh_h

#include <ppc/csp/chan.h>

typedef enum      /* the two lower-half channel types that are used by ... */
{                                /* ... the upper-half PVM implementation. */
  PVM_CHAN,                               /* channel between two PVM tasks */
  UNIX_CHAN,                       /* between two threads of the same task */
  CSLIB_CHAN                 /* between the PPC PVM program and the client */
} lh_types;

/*-------------------------------------------------------------------------*/
/* new_pvm_channel:                                                        */
/* Declares a new PVM channel.                                             */
/*-------------------------------------------------------------------------*/
channel new_pvm_channel (int tid, int tag);

/*-------------------------------------------------------------------------*/
/* lh_chan_type:                                                           */
/* Returns the lower-half channel type corresponding to the given channel. */
/*-------------------------------------------------------------------------*/
lh_types lh_chan_type (channel id);

/*-------------------------------------------------------------------------*/
/* lh_chan_id:                                                             */
/* Returns the lower-half channel id corresponding to the given channel.   */
/*-------------------------------------------------------------------------*/
channel lh_chan_id (channel id);

/*-------------------------------------------------------------------------*/
/* new_cslib_channel:                                                      */
/* Return the two channels from and to the client application and          */
/* initialize them with the given values.                                  */
/* This function is called by the unixconn module.                         */
/*-------------------------------------------------------------------------*/
void new_cslib_channel(
  int from_client,
  int to_client,
  channel *from_client_chan,
  channel *to_client_chan
);

#endif
