/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Lower-Half UNIX channels interface.                                     */
/* The functions found in this files are implemented in chanunixlh.c.      */
/* This functions are used even by the UNIX or by the PVM PPC              */
/* implementations.                                                        */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 25.11.1993  Droux       Created                                         */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: chanunixlh.h                                                  */
/* Tested on :                                                             */
/*   NEXTSTEP 2.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, Intel Computers                                         */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef chanunixlh_h
#define chanunixlh_h

#include <ppc/csp/chan.h>
#include <ppc/csp/chanpriv.h>

#define MAX_UNIX_CHANNELS 1000

/*-------------------------------------------------------------------------*/
/* unix_par_channel:                                                       */
/* Creates a new channel to communicate between two parallel threads.      */
/* UNIX version.                                                           */
/*-------------------------------------------------------------------------*/
channel unix_par_channel (void);

/*-------------------------------------------------------------------------*/
/* unix_chan_close:                                                        */
/* Close the given channel.                                                */
/* UNIX version.                                                           */
/*-------------------------------------------------------------------------*/
void unix_chan_close (channel id);

/*-------------------------------------------------------------------------*/
/* unix_chan_write:                                                        */
/* Write the given data items to the specified channel.                    */
/* <nitems> corresponds to the number of items of type <type> to be        */
/* transmitted.                                                            */
/*-------------------------------------------------------------------------*/
int unix_chan_write (channel id, void *vbuf, int nitems, int type);

/*-------------------------------------------------------------------------*/
/* unix_chan_read:                                                         */
/* Read the given data items from the specified channel.                   */
/* <nitems> corresponds to the number of items of type <type> to be        */
/* received.                                                               */
/*-------------------------------------------------------------------------*/
int unix_chan_read (channel id, void *vbuf, int nitems, int type);

/*-------------------------------------------------------------------------*/
/* unix_start_write:                                                       */
/* Starts a channel transmission of data. Must be called before any call   */
/* to unix_chan_write for each message transmission.                       */
/* Does nothing under UNIX. Implemented for compatibility with the PVM     */
/* lower-half module.                                                      */
/*-------------------------------------------------------------------------*/
void unix_start_write (void);

/*-------------------------------------------------------------------------*/
/* unix_end_write:                                                         */
/* Terminates a channel transmission of data. Must be called after all     */
/* calls to unix_chan_write for each message transmission.                 */
/* Does nothing under UNIX. Implemented for compatibility with the PVM     */
/* lower-half module.                                                      */
/*-------------------------------------------------------------------------*/
void unix_end_write (channel id);

/*-------------------------------------------------------------------------*/
/* unix_start_read:                                                        */
/* Starts a channel reception of data. Must be called before any call      */
/* to unix_chan_read for each message reception.                           */
/* Does nothing under UNIX. Implemented for compatibility with the PVM     */
/* lower-half module.                                                      */
/*-------------------------------------------------------------------------*/
void unix_start_read (channel id);

/*-------------------------------------------------------------------------*/
/* unix_read_fd:                                                           */
/* Returns the read file descriptor of the given channel. This function    */
/* is particularly usefull in the alt construct.                           */
/*-------------------------------------------------------------------------*/
int unix_read_fd (channel c);

#endif
