/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Parallel construction PACT-C interface.                                 */
/* The PACT-C implementation of the PPC parallel construct is very simple  */
/* and can be implemented just by the definitions above.                   */
/* There is no corresponding .c file!                                      */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 26.01.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: parpact.h                                                     */
/* Tested on :                                                             */
/*   PACT-C v92.1, T805                                                    */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef parunix_h
#define parunix_h

#include <par.h>
#undef par
#define pactc_par __par                          /* tested only with v92.1 */

/*-------------------------------------------------------------------------*/
/* par_begin:                                                              */
/* Head of the parallel construction which must be used by the programmer. */
/* A user par_begin(); is extended in the following sequences:             */
/*   internal_par_begin();                                                 */
/*   {                                                                     */
/*     int _DUMB_LOCAL_VARIABLE;                                           */
/* This expansion allows local variables to be declared inside the         */
/* parallel construct.                                                     */
/*-------------------------------------------------------------------------*/
#define par_begin() pactc_par {

/*-------------------------------------------------------------------------*/
/* par:                                                                    */
/* Starts a new parallel thread.                                           */
/* Under UNIX, threads are implemented as processes.                       */
/*-------------------------------------------------------------------------*/
#define par(func) func

/*-------------------------------------------------------------------------*/
/* par_end:                                                                */
/* Wait for all threads to terminate (join) which must be used by the      */
/* programmer.                                                             */
/* A user par_end(); is expended to the following sequence:                */
/*   }                                                                     */
/*   internal_par_end();                                                   */
/*-------------------------------------------------------------------------*/
#define par_end() }

/*-------------------------------------------------------------------------*/
/* par_for:                                                                */
/* Start an arbitrary number of threads in parallel. This function can be  */
/* used as follows:                                                        */
/*   par_for(n = 0; n < N; n++)                                            */
/*     par(f(n));                                                          */
/*-------------------------------------------------------------------------*/
#define par_for pactc_par

#endif
