/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Parallel construction UNIX interface.                                   */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 26.11.1993  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: parunix.h                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 2.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, Intel Computers                                         */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef parunix_h
#define parunix_h

#include <stdio.h>

/*-------------------------------------------------------------------------*/
/* internal_par_begin:                                                     */
/* Head of the parallel construction.                                      */
/* This function is automatically called at the beginning of a par         */
/* construction.                                                           */
/*-------------------------------------------------------------------------*/
void internal_par_begin();

/*-------------------------------------------------------------------------*/
/* par_begin:                                                              */
/* Head of the parallel construction which must be used by the programmer. */
/*-------------------------------------------------------------------------*/
#define par_begin internal_par_begin

/*-------------------------------------------------------------------------*/
/* par:                                                                    */
/* Starts a new parallel thread.                                           */
/* Under UNIX, threads are implemented as processes.                       */
/*-------------------------------------------------------------------------*/
#define par(func)                                                           \
  {                                                                         \
    int child_pid;                         /* pid of the created process */ \
    if ((child_pid = fork()) == -1)                                         \
    {                                                                       \
      fprintf(stderr, "par: cannot fork new process\n");                    \
      exit(1);                                                              \
    }                                                                       \
    if (!child_pid)                                                         \
    {                                                   /* child process */ \
      func;                                                                 \
      exit(0);                                                              \
    }                                                                       \
  } 

/*-------------------------------------------------------------------------*/
/* internal_par_end:                                                       */
/* Wait for all threads to terminate (join).                               */
/*-------------------------------------------------------------------------*/
void internal_par_end();

/*-------------------------------------------------------------------------*/
/* par_end:                                                                */
/* Wait for all threads to terminate (join) which must be used by the      */
/* programmer.                                                             */
/*-------------------------------------------------------------------------*/
#define par_end internal_par_end

/*-------------------------------------------------------------------------*/
/* par_for:                                                                */
/* Start an arbitrary number of threads in parallel. This function can be  */
/* used as follows:                                                        */
/*   par_for(n = 0; n < N; n++)                                            */
/*     par(f(n));                                                          */
/*-------------------------------------------------------------------------*/
#define par_for for

#endif
