/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* PPC types. The constants defined in this file are used by the           */
/* configurer and for the CSP channels communication.                      */
/* They are used to identify the type of a parameter when calling a        */
/* C function that has a variable number of parameters.                    */ 
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 26.11.1993  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: types.h                                                       */
/* Tested on :                                                             */
/*   NEXTSTEP 2.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, Intel Computers                                         */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef types_h
#define types_h

/*-------------------------------------------------------------------------*/
/* The definitions below are used to identify the type of a data to be     */
/* transmitted on a channel as well as to specify the interface of a task. */
/*-------------------------------------------------------------------------*/
typedef enum
{
  /* to use by both the task_interface() and send_channel() */
  INT,                                                            /* C int */
  SHORT,                                       /* C signed short (16 bits) */
  UNSIGNED_SHORT,                             /* C unsigned short (16 bits */
  LONG,                                         /* C signed long (32 bits) */
  UNSIGNED_LONG,                              /* C unsigned long (32 bits) */
  FLOAT,                                              /* C float (32 bits) */
  DOUBLE,                                            /* C double (32 bits) */
  CHAR,                                                 /* C char (8 bits) */

  INT_ARRAY,
  SHORT_ARRAY,           
  UNSIGNED_SHORT_ARRAY,  
  LONG_ARRAY,            
  UNSIGNED_LONG_ARRAY,   
  FLOAT_ARRAY,           
  DOUBLE_ARRAY,          
  CHAR_ARRAY,            

  /* to use only by task_interface() */
  OUTPUT_CHANNEL,                                        /* output channel */
  INPUT_CHANNEL,                                          /* input channel */
  OUTPUT_CHANNEL_ARRAY,                           
  INPUT_CHANNEL_ARRAY
} types_names;

#define END -1                             /* to terminate a list of items */
#define STRING CHAR_ARRAY          /* STRING and CHAR_ARRAY are synonymous */

/*-------------------------------------------------------------------------*/
/* The definitions below correspond to the sizes of the types defined      */
/* above.                                                                  */
/*-------------------------------------------------------------------------*/
extern int type_size [];

#endif
