/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Virtual machine configuration main file.                                */
/* This file must be linked with the user virtual machine description      */
/* (containing the function virtual_machine()), and with the user hardware */
/* complement for the various supported architectures.                     */
/* When executed without arguments the configuration program will generate */
/* the makefiles and main stubs for the parallel program.                  */
/* When called with the argument "v" (as verbose), it will not generate    */
/* anything but just print a complete description of the virtual machine.  */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 17.11.1993  Droux       Created                                         */
/* 21.01.1994  Droux       Ported to PACT-C                                */
/* 01.07.1994  Droux       Allows printing of the machine description      */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: confmain.c                                                    */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel                                                 */
/*   Linux 1.0.8, Intel                                                    */
/*   PACT-C v92.1, T805                                                    */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <string.h>
#include <ppc/config.h>
#include <ppc/config/confpriv.h>
#include "confhard.h"
#include "confrep.h"

typedef enum          /* possible actions of this program depending on the */
{                                     /* arguments passed during execution */
  CONFIGURE,                        /* configure, generate necessary files */
  REPORT                                                   /* print report */
} possible_actions;

/*-------------------------------------------------------------------------*/
/* program_name:                                                     LOCAL */
/* Extract the PPC program name from the executable name under which the   */
/* configurer has been called.                                             */
/*-------------------------------------------------------------------------*/
int program_name (char *argv0, char *name)
{
  char full_name [100];
  char *point_ptr;
  char *last_slash;
  char *act_slash;
  char *cp_ptr;
  char *fl_ptr;

  last_slash = argv0+1;                 /* remove the path name from argv0 */
  do 
  {          
    act_slash = strchr(last_slash, '/');
    if (act_slash != NULL)
      last_slash = act_slash+1;
  } while (act_slash != NULL);
  
  for (cp_ptr = last_slash, fl_ptr = &full_name[0]; 
       *cp_ptr != '\0'; cp_ptr++, fl_ptr++)
    *fl_ptr = *cp_ptr;
  *fl_ptr = '\0';
                                   /* remove the .con configurer extension */
  if ((point_ptr = strchr(full_name, '.')) == NULL)
    return -1;
  *point_ptr = '\0';
  
  strcpy(name, full_name);
  return 0;
}

/*-------------------------------------------------------------------------*/
/* main:                                                                   */
/* Configurer entry point.                                                 */
/*-------------------------------------------------------------------------*/
void main (int argc, char *argv[])
{
  channel from_host;
  channel to_host;
  char name [100];
  possible_actions action;

  if (program_name(argv[0], name) < 0)             /* get PPC program name */
  {
    printf("%s: bad configurer file name\n", argv[0]);
    exit(1);
  }

  switch(argc)                         /* determine action to be performed */
  {
    case 1:
      action = CONFIGURE;
      break;
    case 2:
      action = REPORT;
      break;
    default:
      printf("%s: invalid number of arguments\n", argv[0]);
      break;
  }
                                                  /* analyse configuration */
  printf("starting configuration analysis of \"%s\"\n", name);
  from_host = FROM_HOST_CHANNEL_ID;
  to_host = TO_HOST_CHANNEL_ID;
  virtual_machine(from_host, to_host);

  switch(action)                                /* perform selected action */
  {
    case CONFIGURE:
      generate_files(name);
      break;
    case REPORT:
      printf("\nvirtual machine configuration:\n");
      print_recap();
      print_hw_report();
      printf("\n");
      break;
  }

  free_config_data();       /* release data allocated by virtual_machine() */
  exit(0);
}
