/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Virtual machine configuration report implementation.                    */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 17.11.1993  Droux       Created                                         */
/* 21.01.1994  Droux       Ported to PACT-C                                */
/* 01.07.1994  Droux       Output cleanup (now officially supported)       */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: confrep.c                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel                                                 */
/*   Linux 1.0.8, Intel                                                    */
/*   PACT-C v92.1, T805                                                    */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdio.h>
#include <ppc/config.h>
#include <ppc/config/confpriv.h>

/*-------------------------------------------------------------------------*/
/* print_recap:                                                            */
/* Print the virtual machine configuration. For test purpose.              */
/*-------------------------------------------------------------------------*/
void print_recap (void)
{
  int n;
  int nb_processors;
  int nb_tasks;
  int nb_channels;
  int act_proc;
  int act_proc_idx;
  
  nb_processors = 0;                         /* count number of processors */
  for (n = 0; n < MAX_PROCESSORS; n++)
    if (!processors[n].free)
      nb_processors++;
  printf("  %d virtual processors\n", nb_processors);

  nb_tasks = 0;                                   /* count number of tasks */
  for (n = 0; n < MAX_PROCESSORS; n++)
    if (!tasks[n].free)
      nb_tasks++;
  printf("  %d tasks\n", nb_tasks);
  
  nb_channels = 0;                             /* count number of channels */
  for (n = 0; n < MAX_CHANNELS; n++)
    if (!channels[n].free)
      nb_channels++;
  printf("  %d channels\n", nb_channels);
  
  act_proc = 0;
  for (act_proc_idx = 0; act_proc_idx < MAX_PROCESSORS; act_proc_idx++)
  {
    if(!processors[act_proc_idx].free)
    {
      int act_task;
      int act_task_idx;
      
      printf("  processor %d :\n", act_proc);
      for (act_task_idx = 0; act_task_idx < MAX_TASKS_PER_PROCESSOR;
        act_task_idx++)
      {
        act_task = processors[act_proc_idx].tasks[act_task_idx];
        if (act_task != -1)
	{
	  int act_param_type;
	  int act_param_idx;
	  int nb_parameters;
	  int act_chan;
	  int other_task;
	  int act_module;
	  
          printf("    task %d ", act_task);
	  act_module = tasks[act_task].implemented;
	  printf("(\"%s\")\n", modules[act_module].name);
	  nb_parameters = tasks[act_task].nb_parameters;
          for (act_param_idx = 0; act_param_idx < nb_parameters;
	    act_param_idx++)
	  {
	    act_param_type = tasks[act_task].parameters[act_param_idx].type;
	    if ((act_param_type == INPUT_CHANNEL) ||
	        (act_param_type == OUTPUT_CHANNEL))
	    {
	      act_chan = 
	        tasks[act_task].parameters[act_param_idx].param.value.channel_val;
	      printf("      channel ");
              if (act_chan == FROM_HOST_CHANNEL_ID)
                printf("  FH   from   host");
              else if (act_chan == TO_HOST_CHANNEL_ID)
                printf("  TH   to     host");
              else
	      {
                printf("%4d   ", act_chan);
	        if (act_param_type == INPUT_CHANNEL)
		{
		  other_task = channels[act_chan].from;
                  if (other_task == -1)
	            printf("(not connected)");
		  else 
		  {
		    printf("from   task");
		    printf("%4d", other_task);
		  }
		}
		else
		{
		  other_task = channels[act_chan].to;
                  if (other_task == -1)
	            printf("(not connected)");
		  else 
		  {
		    printf("to     task");
		    printf("%4d", other_task);
		  }
		}
	      }
	      printf("\n");
	    }
	  }
	}
      }
      
      act_proc++;
    }
  }
  {                                          /* print implementation info */
    int m;
    int act_module;
    for (act_module = 0; act_module < MAX_MODULES; act_module++)
      if (!modules[act_module].free)
      {
        printf("  \"%s\" is implemented by modules ", modules[act_module].name);
	for (m = 0; m < modules[act_module].nb_names; m++)
	  printf("\"%s\" ", modules[act_module].modules_names[m]);
	printf("\n");
      }
  }
}

