/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* LoginWindow class interface.                                            */
/* Subclass of Window, this class implements the PPC login window which    */
/* allows a user to select the host on which a server application must be  */
/* started.                                                                */
/* One instance of this class is created when the LoginPalette is          */
/* imported in a NEXTSTEP application. This instance must be then passed   */
/* as parameter to the connect method of the PPCServer class.              */
/* This class is never used directly by the programmer, but indirectly by  */
/* the PPCServer class.                                                    */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 09.07.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: LoginWindow.h                                                 */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import <appkit/appkit.h>
#import <ppc/cslib.h>

@interface LoginWindow:Window
{                                                               /* outlets */
  id serverFormCell;          /* server host name in "Selected Host" group */
  id archFormCell;         /* server architecture in "Selected Host" group */
  id loginFormCell;            /* user login name in "Selected Host" group */ 
  id passwdTextField;           /* password value in "Selected Host" group */
  id pathFormCell;             /* executable path in "Selected Host" group */
  id hostsBrowser;                                   /* host names browser */
  id archBrowser;                            /* host architectures browser */
  id selLoginFormCell;            /* user login name in "Hosts List" group */
  id selPathFormCell;             /* executable path in "Hosts List" group */
  id selPasswdSwitch;             /* password switch in "Hosts List" group */
  id setHostButton;                                   /* "Set Host" button */
  id loginButton;                                        /* "Login" button */
  id setAsDefaultButton;                        /* "Set As Default" button */
  id removeDefaultButton;                       /* "Remove Default" button */
  id restoreDefaultButton;                     /* "Restore Default" button */
                                                /* misc instance variables */
  id hosts [MAX_HOSTS];                                 /* PPCHost objects */
  int nbHosts;                         /* number of defined items in hosts */
  char userName [MAX_LO_LEN];           /* user login name on current host */
                 /* instance variable containing the host to use as server */
  BOOL canceled;               /* YES if login procedure has been canceled */
  char selHostName [MAX_HOST_NAME_LEN];              /* selected host name */
  architecture selHostArch;                  /* selected host architecture */
  char selPw [100];                             /* password entered if any */
  id selectedHost;      /* host actually selected in "Selected Host" group */
  int selectedArch;     /* arch actually selected in "Selected Host" group */
                                          /* default host and architecture */
  id defaultHost;                                   /* default server host */
  int defaultArchIdx;                        /* default architecture index */
}

/*-------------------------------------------------------------------------*/
/* init:                                                                   */
/* Misc instance initializations.                                          */
/*-------------------------------------------------------------------------*/
- init;

/*-------------------------------------------------------------------------*/
/* free:                                                                   */
/* Misc instance deallocation.                                             */
/*-------------------------------------------------------------------------*/
- free;

/*-------------------------------------------------------------------------*/
/* keyDown:                                                                */
/* keyDown event handler called when the user press a key. This method     */
/* perform user interaction corresponding to the pressed key if defined.   */
/*-------------------------------------------------------------------------*/
- keyDown: (NXEvent *)event;

/*-------------------------------------------------------------------------*/
/* initHosts:                                                              */
/* Read the hosts file and initialize the content of the login window.     */
/* This method must be called before showing the window.                   */
/*-------------------------------------------------------------------------*/
- initHosts;

/*-------------------------------------------------------------------------*/
/* loginClicked:                                                           */
/* Message sent when the user clicks on the "Login" button. Will cause     */
/* the window to read its actual login contents and break the modal loop.  */
/*-------------------------------------------------------------------------*/
- loginClicked: sender;

/*-------------------------------------------------------------------------*/
/* cancelClicked:                                                          */
/* Message sent when the user clicks on the "Cancel" button. Will cause    */
/* the window to break the modal loop.                                     */
/*-------------------------------------------------------------------------*/
- cancelClicked: sender;

/*-------------------------------------------------------------------------*/
/* passwdSwitchClicked:                                                    */
/* Message sent when the user clicks on the "Password Required" switch of  */
/* the hosts list.                                                         */
/*-------------------------------------------------------------------------*/
- passwdSwitchClicked: sender;

/*-------------------------------------------------------------------------*/
/* setHost:                                                                */
/* Message sent when the user clicks on the "Set Host" button.             */
/*-------------------------------------------------------------------------*/
- setHost: sender;

/*-------------------------------------------------------------------------*/
/* setAsDefault:                                                           */
/* Message sent when the user clicks on the "Set as Default" button.       */
/*-------------------------------------------------------------------------*/
- setAsDefault: sender;

/*-------------------------------------------------------------------------*/
/* removeDefault:                                                          */
/* Message sent when the user clicks on the "Remove Default" button.       */
/*-------------------------------------------------------------------------*/
- removeDefault: sender;

/*-------------------------------------------------------------------------*/
/* restoreDefault:                                                         */
/* Message sent when the user clicks on the "Restore Default" button.      */
/*-------------------------------------------------------------------------*/
- restoreDefault: sender;

/*-------------------------------------------------------------------------*/
/* hostsBrowserClick:                                                      */
/* Message sent when the user clicks on a cell of the hosts browser.       */
/*-------------------------------------------------------------------------*/
- hostsBrowserClick: sender;

/*-------------------------------------------------------------------------*/
/* archBrowserClick:                                                       */
/* Message sent when the user clicks on a cell of the architectures        */
/* browser.                                                                */
/*-------------------------------------------------------------------------*/
- archBrowserClick: sender;

/*-------------------------------------------------------------------------*/
/* browser:                                                                */
/* NXBrowser delegate method sent when one of the browsers needs the       */
/* contents of the cells.                                                  */
/*-------------------------------------------------------------------------*/
- (int) browser: sender
  fillMatrix: matrix
  inColumn: (int)column;

/*-------------------------------------------------------------------------*/
/* canceled:                                                               */
/* Return YES if the user has clicked on the "Cancel" button.              */
/*-------------------------------------------------------------------------*/
- (BOOL)canceled;

/*-------------------------------------------------------------------------*/
/* selHostName:                                                            */
/* Return the selected host name. Only valid after the user has clicked    */
/* on "Login".                                                             */
/*-------------------------------------------------------------------------*/
- (char *) selHostName;

/*-------------------------------------------------------------------------*/
/* selHostArch:                                                            */
/* Return the selected host architecture. Only valid after the user has    */
/* clicked on "Login".                                                     */
/*-------------------------------------------------------------------------*/
- (architecture) selHostArch;

/*-------------------------------------------------------------------------*/
/* selPw:                                                                  */
/* Return the selected password. Only valid after the user has clicked on  */
/* "Login".                                                                */
/*-------------------------------------------------------------------------*/
- (char *) selPw;

@end
