/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* PPCArch class implementation.                                           */
/* A PPCArch object corresponds to a specific architecture of a server     */
/* host. It contains all properties specific to this architecture.         */
/* Each host has a set of architecture objects.                            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 11.07.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: PPCArch.m                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import "PPCArch.h"

@implementation PPCArch

/*-------------------------------------------------------------------------*/
/* init:                                                                   */
/* Class initializations.                                                  */
/*-------------------------------------------------------------------------*/
- init
{
  [super init];
  arch = UNKNOWN;
  return self;
}

/*-------------------------------------------------------------------------*/
/* setArch:                                                                */
/* Set the architecture properties.                                       */
/*-------------------------------------------------------------------------*/
- setArch: (architecture) archP                            /* architecture */
  loRequired: (BOOL) loRequiredP          /* YES if alternative login name */
  lo: (char *) loP                               /* alternative login name */
  pwRequired: (BOOL) pwRequiredP               /* YES if password required */
  epRequired: (BOOL) epRequiredP         /* YES if executable path defined */
  ep: (char *) epP;                                     /* executable path */
{
  arch = archP;
  loRequired = loRequiredP;
  if (loRequired)
    lo = loP;
  else
    lo = NULL;
  pwRequired = pwRequiredP;
  epRequired = epRequiredP;
  if (epRequired)
    ep = epP;
  else
    ep = NULL;
  return self;
}

/*-------------------------------------------------------------------------*/
/* arch:                                                                   */
/* Return the object architecture.                                         */
/*-------------------------------------------------------------------------*/
- (architecture) arch
{
  return arch;
}

/*-------------------------------------------------------------------------*/
/* loRequired:                                                             */
/* Return YES if an alternative login name is defined.                     */
/*-------------------------------------------------------------------------*/
- (BOOL) loRequired
{
  return loRequired;
}

/*-------------------------------------------------------------------------*/
/* lo:                                                                     */
/* Return the alternative login name if it is defined or NULL otherwise.   */
/*-------------------------------------------------------------------------*/
- (char *) lo
{
  return lo;
}

/*-------------------------------------------------------------------------*/
/* pwRequired:                                                             */
/* Return YES if a password is required.                                   */
/*-------------------------------------------------------------------------*/
- (BOOL) pwRequired
{
  return pwRequired;
}

/*-------------------------------------------------------------------------*/
/* epRequired:                                                             */
/* Return YES if an executable path is defined.                            */
/*-------------------------------------------------------------------------*/
- (BOOL) epRequired
{
  return epRequired;
}

/*-------------------------------------------------------------------------*/
/* ep:                                                                     */
/* Return the executable path if defined or NULL otherwise.                */
/*-------------------------------------------------------------------------*/
- (char *) ep
{
  return ep;
}

/*-------------------------------------------------------------------------*/
/* getArchString:                                                          */
/* Return a string corresponding to the object architecture.               */
/*-------------------------------------------------------------------------*/
- (char *) getArchString
{
  return arch_string(arch);
}

@end
