/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* PPCHost class interface.                                                */
/* A PPCHost object correspond to a server host identified by its          */
/* internet address. One PPCHost can support more than one architecture.   */
/* (This is a difference compared to hlist host where a new host is        */
/* created for each different architecture.)                               */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 11.07.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: PPCHost.h                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#import <appkit/appkit.h>
#import <ppc/cslib.h>

@interface PPCHost:Object
{
  char *hostName;                             /* internet name of the host */
  id architectures [NB_ARCH];       /* architectures supported by the host */
  int nbArch;                         /* number of supported architectures */
}

/*-------------------------------------------------------------------------*/
/* init:                                                                   */
/* Misc instance initializations.                                          */
/*-------------------------------------------------------------------------*/
- init;

/*-------------------------------------------------------------------------*/
/* free:                                                                   */
/* Misc instance deallocation.                                             */
/*-------------------------------------------------------------------------*/
- free;

/*-------------------------------------------------------------------------*/
/* initHost:                                                               */
/* Init the host object with the given internet name.                      */
/*-------------------------------------------------------------------------*/
- initHost: (char *) hostNameP;

/*-------------------------------------------------------------------------*/
/* addArch:                                                                */
/* Add the given architecture as supported by the host.                    */
/*-------------------------------------------------------------------------*/
- addArch: (architecture) arch                             /* architecture */
  loRequired: (BOOL) loRequiredP          /* YES if alternative login name */
  lo: (char *) loP                               /* alternative login name */
  pwRequired: (BOOL) pwRequiredP               /* YES if password required */
  epRequired: (BOOL) epRequiredP         /* YES if executable path defined */
  ep: (char *) epP;                                     /* executable path */

/*-------------------------------------------------------------------------*/
/* getHostName:                                                            */
/* Return the host internet name.                                          */
/*-------------------------------------------------------------------------*/
- (char *) getHostName;

/*-------------------------------------------------------------------------*/
/* nbArch:                                                                 */
/* Return the number of architectures supported by the host.               */
/*-------------------------------------------------------------------------*/
- (int) nbArch;

/*-------------------------------------------------------------------------*/
/* arch:                                                                   */
/* Return the architecture of the specified supported architecture.        */
/*-------------------------------------------------------------------------*/
- (architecture) arch: (int) idx;

/*-------------------------------------------------------------------------*/
/* getArchString:                                                          */
/* Return the name of the specified supported architecture.                */
/*-------------------------------------------------------------------------*/
- (char *) getArchString: (int) idx;

/*-------------------------------------------------------------------------*/
/* archIdxFromString:                                                      */
/* Return the architecture identifier corresponding to the given           */
/* architecture name or -1 if not supported.                               */
/*-------------------------------------------------------------------------*/
- (int) archIdxFromString: (const char *) archName;

/*-------------------------------------------------------------------------*/
/* loRequired:                                                             */
/* Return YES if the specified architecture requires an alternative        */
/* login name.                                                             */
/*-------------------------------------------------------------------------*/
- (BOOL) loRequired: (int) idx;

/*-------------------------------------------------------------------------*/
/* lo:                                                                     */
/* Return the alternative login name corresponding to the specified        */
/* architecture.                                                           */
/*-------------------------------------------------------------------------*/
- (char *) lo: (int) idx;

/*-------------------------------------------------------------------------*/
/* pwRequired:                                                             */
/* Return YES if the specified architecture requires a password.           */
/*-------------------------------------------------------------------------*/
- (BOOL) pwRequired: (int) idx;

/*-------------------------------------------------------------------------*/
/* epRequired:                                                             */
/* Return YES if the specified architecture requires an optional           */
/* executable path.                                                        */
/*-------------------------------------------------------------------------*/
- (BOOL) epRequired: (int) idx;

/*-------------------------------------------------------------------------*/
/* ep:                                                                     */
/* Return the executable path corresponding to the specified architecture. */
/*-------------------------------------------------------------------------*/
- (char *) ep: (int) idx;

@end
