/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* The functions implemented by this module are used by the client and     */
/* the server applications to verify that both entities are communicating  */
/* using a compatible version of CS-Lib.                                   */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 25.06.1994  Droux       Created                                         */
/* 13.07.1994  Droux       CSLIB_VERSION_LEN moved from .c to .h           */
/* 13.07.1994  Droux       COMPATIBILITY_LENGTH moved from .c to .h        */
/*-------------------------------------------------------------------------*/
/* filename: csversion.c                                                   */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include "csversion.h"
#include "ion.h"

/* We define above the version of the protocol implemented by this version */
/* of PPC. For example, the PPC version 1.03 could have a different        */
/* protocol version than PPC 0.07 and in this case, we have to check if    */
/* the client and the server are still able to communicate together.       */
#define CSLIB_VERSION "0001"                    /* PPC versions 0.07, 0.08 */
#define COMPATIBLE_VERSION   "1"   /* returned to the server if compatible */
#define INCOMPATIBLE_VERSION "0"

/*-------------------------------------------------------------------------*/
/* server_check_version:                                                   */
/* Server version of the test function.                                    */
/* Returns 1 if compatible, -1 otherwise.                                  */
/*-------------------------------------------------------------------------*/
int server_check_version (
  int fc,                                            /* socket from client */
  int tc                                               /* socket to client */
)
{
  char result [COMPATIBILITY_LENGTH+1];
  
  writen(tc, CSLIB_VERSION, CSLIB_VERSION_LEN);
  
  readn(fc, result, COMPATIBILITY_LENGTH);
  result[COMPATIBILITY_LENGTH] = '\0';
  
  if (!strcmp(result, COMPATIBLE_VERSION))
  {                                                 /* compatible versions */
    return 1;
  }
  
  return -1;
}

/*-------------------------------------------------------------------------*/
/* client_check_version:                                                   */
/* Client version of the test function.                                    */
/* Returns 1 if compatible, -1 otherwise.                                  */
/*-------------------------------------------------------------------------*/
int client_check_version (
  int fs,                                            /* socket from server */
  int ts                                               /* socket to server */
)
{
  char ver [CSLIB_VERSION_LEN+1];
  
  readn(fs, ver, CSLIB_VERSION_LEN);
  ver[CSLIB_VERSION_LEN] = '\0';

  if (strcmp(ver, CSLIB_VERSION))
  {                                               /* incompatible versions */
    writen(ts, INCOMPATIBLE_VERSION, COMPATIBILITY_LENGTH);
    return -1;
  }

  writen(ts, COMPATIBLE_VERSION, COMPATIBILITY_LENGTH);
  return 1;
}
