/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Error management.                                                       */
/* The PPC error management is similar to the UNIX one. If a routine       */
/* failed, then it returns a value of -1 and it sets the global variable   */
/* ppc_errno to a code which corresponds to the error. It is possible      */
/* to obtain the message description corresponding to the actual error     */
/* by calling the function ppc_strerror(), for example:                    */
/*   fprintf(strerror, "ERROR: %s\n", ppc_strerror(ppc_errno));            */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 09.02.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: error.c                                                       */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
/* Exported global variables.                                              */
/*-------------------------------------------------------------------------*/
int ppc_errno = 0;
char *ppc_errlist [] =
{
  /* EPPC_HLIST_UNKARCH */
  "Unknown architecture specifier in hosts list.",
  /* EPPC_HLIST_LOEQ */
  "Missing '=' for \"lo\" option in hosts list.",
  /* EPPC_HLIST_LOVAL */
  "Missing value for \"lo\" option in hosts list.",
  /* EPPC_HLIST_EPEQ */
  "Missing '=' for \"ep\" option in hosts list.",
  /* EPPC_HLIST_EPVAL */
  "Missing value for \"ep\" option in hosts list.",
  /* EPPC_HLIST_UNKOPT */
  "Unknown option in hosts list.",
  /* EPPC_HLIST_ITMEXC */
  "Too many items in hosts list line.",
  /* EPPC_HLIST_PARTDEF */
  "Host only partially defined in hosts list.",
  /* EPPC_HLIST_ARCHNFND */
  "Specified architecture not found in hosts list.",
  /* EPPC_HLIST_HOSTNFND */
  "Specified host name not found in hosts list.",
  /* EPPC_HLIST_HOSTARCHNFND */
  "Specified host name with given architecture not found in hosts list.",
  /* EPPC_HLIST_BADIDNA */
  "Bad host id given when inquiring host name, internal error.",
  /* EPPC_HLIST_BADIDAR */
  "Bad host id given when inquiring host architecture, internal error.",
  /* EPPC_HFIND_NODEF */
  "At least host name or architecture must be specified to find host.",
  /* EPPC_CONN_OOFSLOTS */
  "Too many client-server sessions activated at the same time.",
  /* EPPC_CONN_PIPECRE */
  "Error creating pipes for connection with remote server program.",
  /* EPPC_CONN_FORK */
  "Bad host id given when inquiring host password necessity.",
  /* EPPC_HLIST_BADIDPW */
  "Bad host id given when inquiring host password necessity.",
  /* EPPC_HLIST_BADIDPR */
  "Bad host id given when inquiring for custom exec path presence.",
  /* EPPC_CONN_EXEC */
  "Cannot execute the \"rsh\" program. Must be in /usr/bin or /usr/ucb.",
  /* EPPC_CONN_RSHF */
  "Rsh program not found. Must be placed in /usr/bin or /usr/ucb.",
  /* EPPC_CONN_BADP */
  "Incorrect password.",
  /* EPPC_CONN_PATH */
  "Executable directory does not exists on server host.",
  /* EPPC_CONN_PROG */
  "Specified program not found on server host.",
  /* EPPC_CONN_COMP */
  "Specified program not compiled for the given architecture.",
  /* EPPC_CONN_HOST */
  "Specified program not compiled for the given host.",
  /* EPPC_CONN_PERM */
  "Permission denied to execute a program on the server host.",
  /* EPPC_HLIST_BADIDLI */
  "Bad host id given when inquiring for custom user name presence.",
  /* EPPC_CONN_SOCK */
  "Error creating TCP socket connection with server host. Internal error.",
  /* EPPC_CHAN_OUTOF */
  "Too many channels opened at the same time.",
  /* EPPC_CONN_SCANF */
  "Error connecting to server: unexpected message received.",
  /* EPPC_CONN_PARE */
  "Password required for selected host.",
  /* EPPC_CONN_UNHO */
  "Specified host is unknown for the computer you are using.",
  /* EPPC_ASYNC_BADSE */
  "Invalid session identifier.",
  /* EPPC_ASYNC_SETOWN */
  "Cannot set owner of the socket with fcntl().",
  /* EPPC_ASYNC_FASYNC */
  "Cannot enable asynchronous I/O on channel with fcntl().",
  /* EPPC_ASYNC_SIG */
  "Cannot install I/O signal catcher.",
  /* EPPC_ASYNC_TOOMANY */
  "Too many channel handlers installed.",
  /* EPPC_ASYNC_GETFL */
  "Error getting socket flags with fcntl().",
  /* EPPC_CONN_COMPAT */
  "Client and server versions of the CS-Lib protocol are incompatible.",
  /* EPPC_CONN_PVMN */
  "PVM is not running on the server host.",
  /* EPPC_HLIST_NOHLIST */
  "Hosts list not found. Must be saved as ~/.ppchosts or ~/ppchosts.",
  /* EPPC_HLIST_NOHOSTDEF */
  "No host defined in hosts list.",
  /* EPPC_LPANEL_CANCELED */
  "Login process canceled by user.",
  /* EPPC_LPANEL_MULTIHOST */
  "The same server host is declared twice with the same architecture."
};

/*-------------------------------------------------------------------------*/
/* ppc_strerror:                                                           */
/* Returns a pointer to a textual description of the error corresponding   */
/* to the given error code.                                                */
/*-------------------------------------------------------------------------*/
char *ppc_strerror (int code)
{
  return ppc_errlist[code];
}
