/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* IO functions.                                                           */
/* The two functions used in this module must be used instead of read()    */
/* and write() when it must be guaranteed that the given number of bytes   */
/* to be respecively read or written are effectively transfered.           */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 16.02.1994  Droux       Created                                         */
/* 13.07.1994  Droux       Added libc.h inclusion under NEXTSTEP           */
/*-------------------------------------------------------------------------*/
/* filename: ion.c                                                         */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#if defined(NEXT) || defined(NEXTINTEL)
  #include <libc.h>
#endif
#include <stdio.h>
#include "ion.h"

/*-------------------------------------------------------------------------*/
/* readn:                                                                  */
/* Reads exactely n bytes from the specified socket desciptor.             */
/* Returns the number of byes read.                                        */
/*-------------------------------------------------------------------------*/
int readn (int fd, char *buffer, int size)
{
  /* Stream sockets exhibit a behavior with the read() and write() system  */
  /* calls that differs from normal file I/O. A read on a socket might     */
  /* input or output fewer bytes than requested, but this is not an error  */
  /* condition. The reason is that buffer limits might be reached for the  */
  /* socket in the kernel and all that is required is for the caller to    */
  /* invoke the read() or write() system call again, to input or output the*/
  /* remaining bytes.                                                      */
  /* (UNIX Network programming, W. Richard Stevens, p. 279 ff)             */
  
  int nleft;                                 /* remaining bytes to be read */
  int nread;              /* read bytes during the last read() system call */
  
  for (nleft = size; nleft > 0;)
  {                     /* while the all requested datas has not been read */
    nread = read(fd, buffer, nleft);
    if (nread < 0)
    {
      fprintf(stderr, "readn: error reading from channel\n");
      exit(1);
    }
    nleft -= nread;
    buffer += nread;
  }
  return (size - nleft);                                 
}

/*-------------------------------------------------------------------------*/
/* writen:                                                                 */
/* Writes exactely n bytes on the specified socket desciptor.              */
/* Returns the number of byes written.                                     */
/*-------------------------------------------------------------------------*/
int writen (int fd, char *buffer, int size)
{
  int nleft;                              /* remaining bytes to be written */
  int nwritten;       /* written bytes during the last write() system call */
  
  for (nleft = size; nleft > 0;)
  {                  /* while the all requested datas has not been written */
    nwritten = write(fd, buffer, nleft);
    if (nwritten < 0)
    {
      fprintf(stderr, "writen: error writing to channel\n");
      exit(1);
    }
    nleft -= nwritten;
    buffer += nwritten;
  }
  return (size - nleft);
}
