/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* Private clichan functions interfaces.                                   */ 
/* The functions declared here are implemented by the module clichan.c.    */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 16.02.1994  Droux       Created                                         */
/* 22.06.1994  Droux       Added channel_socket()                          */
/* 08.07.1994  Droux       New structure of source directory               */
/* 12.07.1994  Droux       clichan splitted into three parts.              */
/*-------------------------------------------------------------------------*/
/* filename: pclichan.h                                                    */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef pclichan_h
#define pclichan_h

#include <stdio.h>
#include <rpc/rpc.h>
#include <ppc/cslib/clichan.h>

#define MAX_CS_CHANS (2*MAX_NB_SESSIONS)  /* max number of client channels */

typedef struct         /* contains the characteristics of a client channel */
{
  int free;                                           /* 1 if slot is free */
  XDR xdrs;              /* client-server connection is implemented by XDR */
  int socket;         /* TCP socket on which data must be send or received */
  FILE *stream;                           /* stream associated with socket */
} cli_chan_descr;

extern cli_chan_descr cli_chans [MAX_CS_CHANS];         /* client channels */

/*-------------------------------------------------------------------------*/
/* new_client_channel:                                                     */
/* Create two new channels and initialize them according to the given      */
/* sockets. The new channels ids are returned in from_server and           */
/* to_server. Returns -1 if an error occured, 0 otherwise.                 */
/*-------------------------------------------------------------------------*/
int new_client_channel(
  int fs_socket, 
  int ts_socket, 
  channel *from_server, 
  channel *to_server
);

/*-------------------------------------------------------------------------*/
/* close_client_channels:                                                  */
/* Close the given client channels.                                        */
/*-------------------------------------------------------------------------*/
void close_client_channels(
  channel from_server,
  channel to_server
);

/*-------------------------------------------------------------------------*/
/* channel_socket:                                                         */
/* Return the socket associated with the given channel.                    */
/*-------------------------------------------------------------------------*/
int channel_socket (
  channel chan
);

#endif
