;
;   @(#)b004asm.asm 1.3 92/12/20    (c) PACT
;
;   PSERVER - PACT host server
;
;   Copyright (c) 1991, 1992 PACT
;   All Rights Reserved.
;
;   MS-DOS i8086 assembler ReadLink and WriteLink routines
;


        DOSSEG
        .MODEL  SMALL


        .DATA

extrn       _C012_idr:word
extrn       _C012_odr:word
extrn       _C012_isr:word
extrn       _C012_osr:word


        .CODE

        ASSUME CS:_TEXT, DS:DGROUP

timeout     dw  0
timeout_fl  db  0
last_count  dw  0

clock       equ 06Ch
biosseg     equ 040h

_linkid     equ 4
_buffer     equ 6
_count      equ 8
_timeout    equ 10

;
; ReadLink(int linkid, char *buffer, int count, int timeout)
;
; Read data from link specified, store in buffer.  Read until count bytes
; read, or no data read in timeout/10 seconds.
;

PUBLIC      _ReadLink

_ReadLink   proc
        push    bp
        mov bp, sp
        push    ds
        push    si
        push    di

; Convert timeout (10ths of secs) to count by multiplying with 1.82
        push    ds
        pop es          ; es := ds
        mov ax, [bp+_timeout]
        mov bx, 182         ; convert timeout
        mul bx
        mov bx, 100
        div bx
        mov cs:[timeout], ax
        mov cs:[timeout_fl], 0  ; no timeout

; Save count
        mov cx, [bp+_count]
        mov cs:[last_count], cx
        mov di, [bp+_buffer]

; Setup for read.  Determine timeout clock time.
non_dma_read:   mov dx, _C012_isr       ; dx := statusreg
        mov bx, _C012_idr       ; bx := datareg
        mov ax, biosseg
        mov ds, ax          ; ds := bios dataseg
        mov ax, ds:[clock]      ; get clock
        add ax, cs:[timeout]
        dec ax
        mov si, ax          ; si := clock + timeout - 1

; Main loop.  Check read status reg.  Load and loop if data.
rd1:        in  al, dx          ; get status
        test    al, 1           ; data ready ?
        jz  rot_ready       ; if so
        xchg    dx, bx          ; dx := data reg
        in  al, dx          ; al := data
        stosb               ; store
        xchg    bx, dx          ; dx := status reg
        loop    rd1         ; next
        jmp rd2         ; exit

; No data.  Check for timeout and loop if not.
rot_ready:                  ; check timeout
        cmp si, ds:[clock]      ; timeout ?
        jne rd1         ; if not, poll status reg

; Compare data count with previous.  If nothing read in timeout slot,
; go return with timeout status.
        cmp cx, cs:[last_count] ; compare count with previous
        je  r_get_out       ; same - timeout

; Data read since last timeout.  Reset count and timer and loop.
        mov cs:[last_count], cx ; record new count
        mov ax, ds:[clock]
        add ax, cs:[timeout]    ; and reset timer
        dec ax
        mov si, ax
        jmp rd1

; Timeout occurred.  Clean up and get out.
r_get_out:  mov cs:[timeout_fl], 2  ; timeout occurred
        pop di          ; restore regs
        pop si
        pop ds
        mov ax, [bp+_count]     ; get count
        sub ax, cx          ; minus # bytes read
        pop bp          ; ax := bytes left to read
        ret

; Normal return.
rd2:        mov ax, [bp+_count]
        pop di
        pop si
        pop ds
        pop bp          ; ax := bytes read!
        ret
_ReadLink   endp



;
; Writelink(int linkid, char *buffer, int count, int timeout)
;
; Write data to link.  Similar to ReadLink.
;

PUBLIC      _WriteLink

_WriteLink  proc
        push    bp
        mov bp, sp
        push    es
        push    si
        push    di

        push    ds
        pop es
        mov ax, [bp+_timeout]
        mov bx, 182
        mul bx
        mov bx, 100
        div bx
        mov cs:[timeout], ax

        mov cx, [bp+_count]
        mov cs:[last_count], cx
        mov si, [bp+_buffer]

non_dma_write:  mov dx, _C012_osr
        mov bx, _C012_odr
        mov ax, biosseg
        mov es, ax
        mov ax, es:[clock]
        add ax, cs:[timeout]
        dec ax
        mov cs:[timeout_fl], 0
        mov di, ax

wr1:        in  al, dx
        test    al, 1
        jz  not_ready
        xchg    dx, bx
        lodsb
        out dx, al
        xchg    bx, dx
        loop    wr1
        jmp wr2

not_ready:
        cmp di, es:[clock]
        jne wr1

        cmp cx, cs:[last_count]
        je  wr_get_out

        mov cs:[last_count], cx
        mov ax, es:[clock]
        add ax, cs:[timeout]
        dec ax
        mov di, ax
        jmp wr1

wr_get_out: mov cs:[timeout_fl], 2
        mov ax, [bp+_count]
        sub ax, cx
        jmp wrterm
wr2:
        mov ax, [bp+_count]
wrterm:     pop di
        pop si
        pop es
        pop bp
        ret
_WriteLink  endp



        END

