/*
 *
 *  @(#)b004dos.c   1.8 92/12/20    (c) PACT
 *
 *  PSERVER - PACT host server
 *
 *  Copyright (c) 1991, 1992 PACT
 *  All Rights Reserved
 *
 *  PC/B004 interface module.
 *
 */

#include    <dos.h>                     /* DOS interface */
#include    "pconf.h"                   /* global defs */
#include    "linkio.h"                  /* link interface definition */


/*
 *
 *      The fact that a link is currently active link is maintained.
 *
 */

#define FREELINK    (-1)
static  link_t  activelink = FREELINK;


/*
 *
 *      Base port addresses.  Default is 0x150, as per Inmos B004 specs.
 *
 */

static  int C012_base = 0x150;
#define C012_DEFAULT_BASE   0x150


/*
 *
 *      Define the layout of the various port address used to communicate
 *      with the board.
 *
 */

#if HOSTID==HOST_NECPC      /* its really a B010  */
#define C012_DEFAULT_BASE       0xd0
#define C012_IDR_OFFSET         0
#define C012_ODR_OFFSET         2
#define C012_ISR_OFFSET         4
#define C012_OSR_OFFSET         6
#define C012_RESET_OFFSET       8
#define C012_ERROR_OFFSET       8
#define C012_ANALYSE_OFFSET     10
#else
#define C012_DEFAULT_BASE       0x150
#define C012_IDR_OFFSET         0
#define C012_ODR_OFFSET         1
#define C012_ISR_OFFSET         2
#define C012_OSR_OFFSET         3
#define C012_RESET_OFFSET       16
#define C012_ERROR_OFFSET       16
#define C012_ANALYSE_OFFSET     17
#endif

int     C012_idr,                   /* some of these are used by b004asm.asm */
         C012_odr,
        C012_isr,
        C012_osr,
        C012_reset,
        C012_error,
        C012_analyse;


/*
 *
 *      link_t  OpenLink (char *link)
 *
 *  Open link.  If link is NULL or empty, the default link is used.
 *  Otherwise, link should be a hex number indicating the base address
 *  of the port to use.  Returns negative number if an error occurs,
 *  and a positive linkid if the link was successfully opened.
 *
 */

#if __STDC__
link_t  OpenLink (char *link)
#else
link_t  OpenLink (link)
char    *link;
#endif
{
    /* If a link is already active, we can't open another.              */
    if (activelink != FREELINK)
        return ER_LINK_CANT;

    /* Determine link port base address from link name.                 */
    if (! link || link[0] == '\0')
        C012_base = C012_DEFAULT_BASE;
    else if (sscanf (link,  "#%x", &C012_base) != 1 &&
             sscanf (link, "0x%x", &C012_base) != 1 &&
             sscanf (link,   "%x", &C012_base) != 1)
        return ER_LINK_BAD;

    /* Set various port addresses.                                      */
    C012_idr     = C012_base + C012_IDR_OFFSET;
    C012_odr     = C012_base + C012_ODR_OFFSET;
    C012_isr     = C012_base + C012_ISR_OFFSET;
    C012_osr     = C012_base + C012_OSR_OFFSET;
    C012_reset   = C012_base + C012_RESET_OFFSET;
    C012_error   = C012_base + C012_ERROR_OFFSET;
    C012_analyse = C012_base + C012_ANALYSE_OFFSET;

    /* Flag link as active and return `link descriptor'.                */
    activelink = 1;
    return activelink;
}


/*
 *
 *      int CloseLink (link_t linkid)
 *
 *  Close link.  Must be open to be closed.  If any error occurs (e.g. link
 *  is not open), a negative number is returned.  Otherwise, 1 is returned.
 *
 */

#if __STDC__
int     CloseLink (link_t linkid)
#else
int     CloseLink (linkid)
link_t   linkid;
#endif
{
    /* If given link id is the one open, flag it as closed.  If not,    */
    /* return ER_LINK_BAD to indicate error.                            */
    if (linkid != activelink)
        return ER_LINK_BAD;
    activelink = FREELINK;

    /* Return 1 to indicate success.                                    */
    return 1;
}


#if ! defined (B4ASM)

/*
 *
 *      int ReadLink (link_t linkid, char *buf, unsigned len, int timout)
 *
 *  Read len bytes from specified link into buffer.  Returns number of
 *  bytes read (before timeout), or a negative number if an error occurs
 *  (wrong link or 0-size transfer).  The timout parameter is currently
 *  ignored - a crude timeout mechanism with a max loop count is provided.
 *
 */

#if __STDC__
int     ReadLink (link_t linkid, char *buf, unsigned len, int timout)
#else
int     ReadLink (linkid, buf, len, timout)
link_t      linkid;
char        *buf;
unsigned    len;
int         timout;
#endif
{
    register    int n, l;

    /* If linkid is not the active link, fail.                          */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Zero-size transfers can't be done.                               */
    if (len < 1)
        return ER_LINK_CANT;

    /* Read as many bytes as specified into buffer.                     */
    for (n=0; n<len; n++) {

        /* Wait some max number of loops (crude timeout) for status     */
        /* to become ready.                                             */
        for (l=0; ! (inp (C012_isr) & 1) && l<10000; l++)
            kbhit();

        if (inp (C012_isr) & 1) {
            *buf++ = inp (C012_idr);
#ifdef  B004_DEBUG
            printf ("<%#02x", *(buf-1));
#endif
        }

        /* Timed out without status OK, so give up and return number    */
        /* of bytes succesfully read.                                   */
        else
            return n;
     }

    /* All done, return number of bytes read.                           */
    return n;
}


/*
 *
 *      int WriteLink (link_t linkid, char *buf, unsigned len, int timout)
 *
 *  Write len bytes from specified link into buffer.  Returns number of
 *  bytes written (before timeout), or a negative number if an error occurs
 *  (wrong link or 0-size transfer).  The timout parameter is currently
 *  ignored - a crude timeout mechanism with a max loop count is provided.
 *
 */

#if __STDC__
int     WriteLink (link_t linkid, char *buf, unsigned len, int timout)
#else
int     WriteLink (linkid, buf, len, timout)
link_t      linkid;
char        *buf;
unsigned    len;
int         timout;
#endif
{
    register    int n, l;

    /* If linkid is not the active link, fail.                          */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Zero-size transfers can't be done.                               */
    if (len < 1)
        return ER_LINK_CANT;

    /* Write as many bytes as specified from buffer.                    */
    for (n=0; n<len; n++) {

        /* Wait some max number of loops (crude timeout) for status     */
        /* to become ready.                                             */
        for (l=0; ! (inp (C012_osr) & 1) && l<10000; l++)
            kbhit();

        if (inp (C012_osr) & 1) {
            outp (C012_odr, *buf);
            buf++;
#ifdef  B004_DEBUG
            printf (">%#02x", *(buf-1) & 0xFF);
#endif
        }

        /* Timed out without status OK, so give up and return number    */
        /* of bytes succesfully read.                                   */
        else
            return n;
    }

    /* All done, return number of bytes read.                           */
    return n;
}

#endif  /* B4ASM */


/*
 *
 *      int ResetLink (link_t linkid)
 *
 *  Reset transputer.  Returns 1 if successful, and a negative number to
 *  indicate an error.
 *
 */

#if __STDC__
int     ResetLink (link_t linkid)
#else
int     ResetLink (linkid)
link_t  linkid;
#endif
{
    int i;

    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Wait a while with a crude loop.                                  */
#   define  WAIT()          for (i=0; i<10000; i++) ;

    /* With analyse low, pull reset low, high and low again.            */
    outp (C012_analyse, 0);
    WAIT();
    outp (C012_reset, 0);
    WAIT();
    outp (C012_reset, 1);
    WAIT();
    outp (C012_reset, 0);
    WAIT();

#undef  WAIT

    /* Return 1 to indicate success.                                    */
    return 1;
}


/*
 *
 *      int AnalyseLink (link_t linkid)
 *
 *  Analyse transputer.  Returns 1 if successful, and a negative number to
 *  indicate an error.
 *
 */

#if __STDC__
int     AnalyseLink (link_t linkid)
#else
int     AnalyseLink (linkid)
link_t  linkid;
#endif
{
    int i;

    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Wait a while with a crude loop.                                  */
#    define  WAIT()          for (i=0; i<10000; i++);

    /* With analyse high, pull reset high and low again, then pull      */
    /* analyse low again.                                               */
    outp (C012_analyse, 0);
    WAIT();
    outp (C012_analyse, 1);
    WAIT();
    outp (C012_reset, 1);
    WAIT();
    outp (C012_reset, 0);
    WAIT();
    outp (C012_analyse, 0);
    WAIT();

#undef  WAIT

    /* Return 1 to indicate success.                                    */
    return 1;
}


/*
 *
 *      int TestError (link_t linkid)
 *
 *  Returns state of ErrorFlag:  1 if set, 0 if not.
 *
 */

#if __STDC__
int     TestError (link_t linkid)
#else
int     TestError (linkid)
link_t  linkid;
#endif
{
    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Read ErrorFlag bit and return 1 if set (we read !ErrorFlag).     */
    return (inp (C012_error) & 1) ? 0 : 1;
}


/*
 *
 *      int TestRead (link_t linkid)
 *
 *  Returns 1 if transputer is ready to receive data, 0 if not.
 *
 */

#if __STDC__
int     TestRead (link_t linkid)
#else
int     TestRead (linkid)
link_t  linkid;
#endif
{
    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Read read status and return 1 if ready.                          */
     return (inp (C012_isr) & 1);
}


/*
 *
 *      int TestWrite (link_t linkid)
 *
 *  Returns 1 if transputer is ready to receive data, 0 if not.
 *
 */

#if __STDC__
int     TestWrite (link_t linkid)
#else
int     TestWrite (linkid)
link_t  linkid;
#endif
{
    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Read write status and return 1 if ready.                         */
    return (inp (C012_osr) & 1);
}

