/*
 *
 *  @(#)config.h    1.5 92/12/20    (c) PACT
 *
 *  PSERVER - PACT host server
 *
 *  Copyright (c) 1991, 1992 PACT
 *  All Rights Reserved
 *
 *  Configuration description.
 *
 */

#ifndef _config_included
#define _config_included


/*
 *
 *      SERVERID
 *
 *  Id of server is passed to transputer upon request, so it can determine
 *  what version of the server is being used.  Unfortunately, not all
 *  server's with the same id have the same features...
 *  The pserver sends a special server id to the transputer runtime, unless
 *  in iserver emulation mode, which tells the transputer runtime it can
 *  use the PACT extensions.
 *
 */

#define SERVERID    14


/*
 *
 *  The following host ids are defined:
 *
 *      1   PC                  5   IBM 370
 *      2   NEC-PC              6   Sun 4
 *      3   VAX                 7   Sun-386i
 *      4   Sun-3               8   Apollo
 *
 *  The following host environments are defined:
 *
 *      1   DOS                 4   SunOS
 *      2   Helios              5   CMS
 *      3   VMS                 167 USV/386
 *
 *  The following interface boards are defined:
 *
 *      1   B004                7   QT0
 *      2   B008                8   B015
 *      3   B010                9   IBM_OAT
 *      4   B011                10  B016
 *      5   B014                11  UDP
 *      6   DRX-11             (12  Parsytec)
 *
 */

#define HOST_PC         1
#define HOST_NECPC      2
#define HOST_VAX        3
#define HOST_SUN3       4
#define HOST_IBM370     5
#define HOST_SUN4       6
#define HOST_SUN386I    7
#define  HOST_APOLLO     8

#define OS_DOS          1
#define OS_HELIOS       2
#define OS_VMS          3
#define OS_SUNOS        4
#define OS_CMS          5
#define OS_USV386       167     /* PACT extension */

#define BOARD_B004      1
#define BOARD_B008      2
#define BOARD_B010      3
#define BOARD_B011      4
#define BOARD_B014      5
#define BOARD_DRX11     6
#define BOARD_QT0       7
#define BOARD_B015      8
#define BOARD_IBM_OAT   9
#define BOARD_B016      10
#define BOARD_UDP       11
#define BOARD_PARSY_SC  167     /* PACT extension */


/*
 *
 *  Try and determine configuration from compiler's predefined symbols.
 *
 */

#if sun             /* Sun3, with QT0 */
#   define  HOSTID          HOST_SUN3
#   define  OSID            OS_SUNOS
#   ifndef  BOARDID
#       define  BOARDID     BOARD_QT0
#   endif

#elif   sparc       /* (Sun4) SparcStation */
#   define  HOSTID          HOST_SUN4
#   define  OSID            OS_SUNOS
#   ifndef  BOARDID
#       define  BOARDID     0
#   endif

#elif   i386        /* AT/386, with B004 */
#   define  HOSTID          HOST_PC
#   define  OSID            OS_USV386
#   ifndef  BOARDID
#       define  BOARDID     BOARD_B004
#   endif

#elif   __TURBOC__  /* PC, with B004 */
#   define  HOSTID          HOST_PC
#   define  OSID            OS_DOS
#   ifndef  BOARDID
#       define  BOARDID     BOARD_B004
#   endif

#elif   helios      /* Helios, by default on Sun4 with Parsytec SC */
#   ifndef  HOSTID
#       define  HOSTID      HOST_SUN4
#   endif
#   define  OSID            OS_HELIOS
#   ifndef  BOARDID
#       define  BOARDID     BOARD_PARSY_SC
#   endif

#else                /* unknown */
#   define  HOSTID          0
#   define  OSID            0
#   ifndef  BOARDID
#       define  BOARDID     0
#   endif
#endif


/*
 *
 *  Some more configuration things.
 *
 */

#if __STDC__
#   define  HAS_STRERROR            /* has strerror() function */
#endif

#if __STDC__ || OSID==OS_DOS
#   define  HAS_RENAME              /* has rename() function */
#   define  HAS_REMOVE              /* has remove() function */
#endif

#if OSID==OS_DOS
#   define  INTSZ   16              /* size of int and long in bits */
#   define  LONGSZ  32
#else
#   define  INTSZ   32
#   define  LONGSZ  32
#endif

#if OSID==OS_HELIOS
#   define  FD_XOR  0x80000000      /* ensure fd is positive */
#endif


#endif
