/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* CS-Lib transputer implementation.                                       */
/* This module is linked with pserver when this program is used to         */
/* interface a client application and a transputer program.                */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 25.08.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: cslib.c                                                       */
/* Tested on :                                                             */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <stdlib.h>
#include <syslog.h>
#include <ppc/cslib.h>
#include "cslib.h"
#include "unixconn.h"

#define MAX_FILE_NAME_LEN 1000
#define STDIN_FD  0                               /* stdin file descriptor */
#define STDOUT_FD 1                              /* stdout file descriptor */

typedef enum {                                       /* pipe array indices */
  PIPE_IN,                                              /* input end-point */
  PIPE_OUT,                                            /* output end-point */
  PIPE_SIZE
} pipe_indices;

static channel to_client;            /* CS-Lib channels to and from client */
static channel from_client;
int to_client_fd;             /* TCP sockets corresponding to the channels */
int from_client_fd;

/*-------------------------------------------------------------------------*/
/* start_redirector:                                                       */
/* Start the transputer redirector. This little program allows a           */
/* transputer network to write messages to the console of the host.        */
/*-------------------------------------------------------------------------*/
void start_redirector (void)
{
  char name [MAX_FILE_NAME_LEN];
  char *argv[] = {name, NULL};
  char *ppc_home;
  int the_pipe [PIPE_SIZE];
  
  ppc_home = getenv("PPC_HOME");
  sprintf(name, "%s/lib/%s/tranredir", ppc_home, HOST_ARCH);
  pipe(the_pipe);
  
  if (!fork())
  {
    close(STDIN_FD);
    close(the_pipe[PIPE_OUT]);
    dup2(the_pipe[PIPE_IN], STDIN_FD);
    execv(name, argv);
    exit(1);
  }
  
  close(STDOUT_FD);
  dup2(the_pipe[PIPE_OUT], STDOUT_FD);
}

/*-------------------------------------------------------------------------*/
/* init_for_cs_lib:                                                        */
/* Initialize pserver to act as an interface between a UNIX application    */
/* running on a remote host and a PACT-C program running on a transputer   */
/* network.                                                                */
/* The following operations are performed:                                 */
/*   - connect to client                                                   */
/*   - start a redirection program                                         */
/*   - redirect stdout and stderr to the redirection program               */
/*-------------------------------------------------------------------------*/
void init_for_cs_lib (void)
{
  connect_to_client(&to_client, &from_client);
  get_sockets(&to_client_fd, &from_client_fd);
  start_redirector();
}
