/*
 *
 *  @(#)file.c  1.18    94/01/23    (c) PACT
 *
 *  PSERVER - PACT host server
 *
 *  Copyright (c) 1991-1993 PACT
 *  All Rights Reserved
 *
 *  File support.
 *
 */

#include    <errno.h>                   /* errno's */
#include    <fcntl.h>                   /* level-1 I/O */
#include    <limits.h>                  /* for OPEN_MAX */
#include    "pserver.h"                 /* general defs */
#if OSID==OS_DOS && __TURBOC__
#   define  HAVE_NO_STAT
#endif
#ifndef HAVE_NO_STAT
#include    <sys/stat.h>                /* for stat/fstat */
#endif
#if OSID==OS_SUNOS || OSID==OS_USV386
#   include <unistd.h>                  /* for access */
#endif
#include    "error.h"                   /* error manager */
#include    "file.h"                    /* own defs and decls */
#include    "host.h"                    /* some host-defined things */
#include    "server.h"                  /* server defs */
#ifdef CS_LIB
  #include "pactcserv.h"
#endif

#ifdef CS_LIB
  extern int to_client_fd;            
  extern int from_client_fd;
#endif

#if PSERVER_EXTENSION
/*
 *
 *      bool_t ISFILDES (fd)
 *
 *  Check if transputer fd is fildes (pserver) or stream (iserver).  Return
 *  TRUE if fildes, FALSE if stream.
 *
 */

#ifndef OPEN_MAX
#   define  OPEN_MAX    _POSIX_OPEN_MAX
#endif

#define ISFILDES(fd)        ((fd) < OPEN_MAX)

#endif  /* PSERVER_EXTENSION */


/*
 *
 *      FILE *STREAM (fd)
 *
 *  Convert fd as used on transputer to stream on host.  Special fd's 0,
 *  1 and 2 indicate standard input, output and error, resp.  Otherwise,
 *  the file descriptor as used on transputer is equal to the host's
 *  stream, except for a possibly toggled sign bit (see above).
 *
 */

#ifndef FD_XOR
#   define  FD_XOR  0
#endif

#define  STREAM(x)   ((x)==0 ? stdin :               \
                     (x)==1 ? stdout :              \
                     (x)==2 ? stderr :              \
                              (FILE *) ((x) ^ FD_XOR))


/*
 *
 *      int file_open ()
 *
 *  Open a file.  Text and binary files are supported, and various
 *  modes (read, write, update, create, append, etc.).
 *
 *  Args:   VEC     name                Rets:   INT32   stream
 *          BYTE    type
 *          BYTE    mode
 *
 */

SERVER_FN (file_open, 3, 1)
{
    FILE    *f;
    char    *name;
    int     type;

    /* Null-terminate name, check type and mode.                        */
    type = BARG(1);
    name = (char *) VDAT(VARG(0));
    name[VLEN(VARG(0))] = '\0';             /* overwrites type */
    if (type < SP_BIN || type > SP_TEXT) {
        error ("file_open(%s): illegal file type %d", name, type);
        return SP_ERROR;
    }
    if (BARG(2) < SP_FIRSTMODE || BARG(2) > SP_LASTMODE) {
        error ("file_open(%s): illegal file mode %d", name, BARG(1));
        return SP_ERROR;
    }

    /* Open file.                                                       */
    f = fopen (name, filemodes[type-SP_BIN][BARG(2)-1]);

    /* Xor with constant to ensure it's >=0.                            */
    if (f)
        f = (FILE *) (((tint_t) f) ^ FD_XOR);

    /* Make sure fd is >= 0.                                            */
    assert (((tint_t) f) >= 0);

    /* Store result:  stream.                                           */
    WRET(0) = (tint_t) f;

    /* Return success or error, depending on whether open worked.       */
    return f ? SP_SUCCESS : SP_ERROR;
}}


#if PSERVER_EXTENSION
/*
 *
 *      int file_open2 ()
 *
 *  Open a file the right way, i.e. with open(2).  Supports all oflags and
 *  allows mode to be set of new file.
 *
 *  PSERVER EXTENSION
 *
 *   Args:   VEC     name                Rets:   INT32   stream
 *          WORD    oflags
 *          WORD    mode
 *
 */

SERVER_FN (file_open2, 3, 1)
{
    int     fd;
    char    *name;
    int     spoflags;
    int     oflags;

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    /* Null-terminate name, check oflags and mode.                      */
    spoflags = WARG(1);
    name = (char *) VDAT(VARG(0));
    name[VLEN(VARG(0))] = '\0';             /* overwrites oflags */

    /* Convert spoflags to host's oflags.                               */
    oflags = 0;
    if (spoflags & SP_O_RDONLY) oflags |= O_RDONLY;
    if (spoflags & SP_O_WRONLY) oflags |= O_WRONLY;
    if (spoflags & SP_O_RDWR  ) oflags |= O_RDWR  ;
#ifdef  O_NDELAY
    if (spoflags & SP_O_NDELAY) oflags |= O_NDELAY;
#endif
    if (spoflags & SP_O_APPEND) oflags |= O_APPEND;
#ifdef  O_SYNC
    if (spoflags & SP_O_SYNC  ) oflags |= O_SYNC  ;
#endif
    if (spoflags & SP_O_CREAT ) oflags |= O_CREAT ;
    if (spoflags & SP_O_TRUNC ) oflags |= O_TRUNC ;
    if (spoflags & SP_O_EXCL  ) oflags |= O_EXCL  ;
#ifdef  O_BINARY
    if (spoflags & SP_O_BINARY) oflags |= O_BINARY;
#endif
#ifdef  O_TEXT
    if (spoflags & SP_O_TEXT)   oflags |= O_TEXT;
#endif

    /* XXX - do something with mode? */

    /* Open file.                                                       */
    /* XXX - make sure text/bin modes work under MS-DOS */
    fd = open (name, oflags, WARG(2));

    /* Store result:  stream.                                           */
    WRET(0) = (tint_t) fd;

    /* Return success or error, depending on whether open worked.       */
    return fd!=-1 ? SP_SUCCESS : SP_ERROR;
}}
#endif


/*
 *
 *      int file_close ()
 *
 *  Close a file.  Flush any unwritten data and discard any unread buffered
 *  input.
 *
 *  Args:   INT32   stream              Rets:   -
 *
 */

SERVER_FN (file_close, 1, 0)
{
    extern  bool_t  verbose;

    /* Can't close std*.                                                */
    if (WARG(0) == 0 || WARG(0) == 1 || WARG(0) == 2)
        return SP_ERROR;

#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0)))
        return close (WARG(0)) == -1 ? SP_ERROR : SP_SUCCESS;
    else
#endif

    /* Close stream and return status.                                  */
    return fclose (STREAM (WARG (0))) == EOF ? SP_ERROR : SP_SUCCESS;
}}


/*
 *
 *      int file_read ()
 *
 *  Read from stream.  As getblock, but result is always 'success'.
 *  This function is obsolete and has been replaced by getblock().
 *
 *  Args:   INT32   stream              Rets:   VEC     data
 *          INT16   count
 *
 */

SERVER_FN (file_read, 2, 1)
{
    FILE    *f = STREAM (WARG(0));
    int     count;

    /* Check size to prevent possible buffer overflow.                  */
    if (HARG(1) > MAXPACKSZ-VLENSZ-1-VLENSZ)
        HARG(1) = MAXPACKSZ-VLENSZ-1-VLENSZ;

    /* Read data from stream into buffer, store count read and return   */
    /* buffer.                                                          */
#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0))) {
        if ((count = read (WARG(0), VDAT(sbuffer), HARG(1))) != -1)
            VDAT(sbuffer)[count] = '\0';
    }
    else if (pserverext && isatty (fileno (f))) {
        if (fgets (VDAT(sbuffer), HARG(1)+1, f))
            count = strlen (VDAT(sbuffer));
        else
            count = 0;
    }
    else
#endif
        count = fread (VDAT(sbuffer), 1, HARG(1), f);
#ifdef  __TURBOC__      /* doesn't set EOF if binary stream */
    if (count == 0 && ! feof (f) && ! ferror (f))
        f->flags |= _F_EOF;
#endif
    SETVLEN (sbuffer, count);
    VRET(0) = sbuffer;

     /* Return success.                                                  */
    return SP_SUCCESS;
}}


/*
 *
 *      int file_write ()
 *
 *  Write to stream.  As putblock, but result is always 'success'.
 *  This function is obsolete and has been replaced by putblock().
 *
 *  Args:   INT32   stream              Rets:   INT16   written
 *          VEC     data
 *
 */

SERVER_FN (file_write, 2, 1)

    /* Write data to stream and return number of bytes written.         */
#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0)))
        HRET(0) = write (WARG(0), VDAT(VARG(1)), VLEN(VARG(1)));
    else {
#endif
        HRET(0) = fwrite (VDAT(VARG(1)), 1, VLEN(VARG(1)), STREAM(WARG(0)));

        /* Flush if stdout/stderr.                                      */
        if (WARG(0) == 1)
            (void) fflush (stdout);
        else if (WARG(0) == 2)
            (void) fflush (stderr);

#if PSERVER_EXTENSION
    }
#endif

    /* Return success.                                                  */
    return SP_SUCCESS;
}


/*
 *
 *      int file_getblock ()
 *
 *  Read block if binary data from stream.   Input stops when the specified
 *  number of bytes is read, or end of file is reached, or an error occurs.
 *  If count is less than one no input occurs.  The stream is left
 *  positioned immediately after the data read, and is undefined if a read
 *  error occurs.  The number of bytes actually read may be less than
 *  requested.  Result will be 0 if successful, and something else in
 *  case of errors, in which case eof() and ferror() should be used to
 *  check the status.
 *
 *  PSERVER extension:  If stream is connected to terminal (isatty) use
 *                      level 2 read() to ensure proper terminal behavior
 *                      (e.g. stop reading at new-line).  If stream is
 *                      connected to terminal, it is by definition not a
 *                      binary stream, so iserver transputer runtime
 *                      would not use getblock() (but gets() instead, see
 *                      below).
 *
 *  Args:   INT32   stream              Rets:   VEC     data
 *          INT16   count
 *
 */

SERVER_FN (file_getblock, 2, 1)
{
    FILE    *f = STREAM (WARG(0));
    int     count;

    /* Check size to prevent possible buffer overflow.                  */
    if (HARG(1) > MAXPACKSZ-VLENSZ-1-VLENSZ)
        HARG(1) = MAXPACKSZ-VLENSZ-1-VLENSZ;
	
    /* Read data from stream into buffer, store count read and return   */
    /* buffer.                                                          */

    #ifdef CS_LIB
    if (WARG(0) == CLIENT_TO_SERVER_FD)
    {
      count = readn(from_client_fd, VDAT(sbuffer), HARG(1));
    }
    else
    {
    #endif

    #if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0))) {
        if ((count = read (WARG(0), VDAT(sbuffer), HARG(1))) != -1)
            VDAT(sbuffer)[count] = '\0';
    }
    else if (pserverext && isatty (fileno (f))) {
        if (fgets (VDAT(sbuffer), HARG(1)+1, f))
            count = strlen (VDAT(sbuffer));
        else
            count = 0;
    }
    else
    #endif
        count = fread (VDAT(sbuffer), 1, HARG(1), f);
    #ifdef  __TURBOC__      /* doesn't set EOF if binary stream */
    if (count == 0 && ! feof (f) && ! ferror (f))
        f->flags |= _F_EOF;
    #endif

    #ifdef CS_LIB
    }
    #endif

    SETVLEN (sbuffer, count);
    VRET(0) = sbuffer;

    /* Return error if 0 bytes read.                                    */
    return count == 0 ? SP_ERROR : SP_SUCCESS;
}}


/*
 *
 *      int file_putblock ()
 *
 *  Write block of binary data to stream, which should be opened for output.
 *  No output is performed if count is less than zero.  The position of the
 *  stream is advanced by the number of bytes actually written, and is
 *  undefined if a write error occurs.  The number of bytes actually written
 *  is returned.  This may be less than requested.  Result will be 0
 *  if successfull, and something else in case of errors, in which case
 *  eof() and ferror() should be used to check the status.  If the stream
 *  is 1 (standard output), the write is automatically flushed.
 *
 *  Args:   INT32   stream              Rets:   INT16   written
 *          VEC     data
 *
 */

SERVER_FN (file_putblock, 2, 1)

    #ifdef CS_LIB
    if (WARG(0) == SERVER_TO_CLIENT_FD)
    {
      HRET(0) = writen(to_client_fd, VDAT(VARG(1)), VLEN(VARG(1)));
    }
    else
    {
    #endif

#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0)))
        HRET(0) = write (WARG(0), VDAT(VARG(1)), VLEN(VARG(1)));
    else {
#endif
        /* Write data to stream and return number of bytes written.     */
        HRET(0) = fwrite (VDAT(VARG(1)), 1, VLEN(VARG(1)), STREAM(WARG(0)));

        /* Flush if stdout/stderr.                                      */
        if (WARG(0) == 1)
            (void) fflush (stdout);
        else if (WARG(0) == 2)
            (void) fflush (stderr);

#if PSERVER_EXTENSION
    }
#endif

    #ifdef CS_LIB
    }
    #endif

    /* Return error if 0 bytes read.                                    */
    return HRET(0) == 0 ? SP_ERROR : SP_SUCCESS;
}


/*
 *
 *      int file_gets ()
 *
 *  Read a line from a stream, which must be open for input.  Characters
 *  are read until the end of file is reached, a new-line character is
 *  seen or the number of characters read is not less than count.  If the
 *  input is terminated because of a new-line, the new-line character is
 *  not included in the returned array.  If end of file is encountered and
 *  nothing has been read from the stream then gets fails.
 *
 *  Args:   INT32   stream              Rets:   VEC     string
 *          INT16   count
 *
 */

SERVER_FN (file_gets, 2, 1)
{
    int len;

    /* Check size to prevent possible buffer overflow.                  */
    if (HARG(1) > MAXPACKSZ-VLENSZ-1-VLENSZ) {
        error ("file_gets(): max string length too large");
        HARG(1) = MAXPACKSZ-VLENSZ-1-VLENSZ;
    }

    /* Read string.  Return error result if error.                      */
#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0))) {
        if ((len = read (WARG(0), VDAT(sbuffer), HARG(1))) == -1)
            return SP_ERROR;
        sbuffer[len] = '\0';
    }
    else
#endif
        if (fgets (VDAT(sbuffer), HARG(1), STREAM (WARG(0))) == NULL)
            return SP_ERROR;

    /* Strip trailing new-line, if any, and return success.             */
    len = strlen (VDAT(sbuffer));
    if (VDAT(sbuffer)[len-1] == '\n')
        VDAT(sbuffer)[--len] = '\0';
    SETVLEN (sbuffer, len);
    VRET(0) = sbuffer;
    return SP_SUCCESS;
}}


/*
 *
 *       int file_puts ()
 *
 *  Write line of text to stream, which must be opened for output.  The
 *  host-specified convention for new-line will be appended to the line
 *  and output to the file.  The maximum line length is host specified.
 *
 *  Args:   INT32   stream              Rets:   -
 *          VEC     string
 *
 */

SERVER_FN (file_puts, 2, 0)

    /* Output string and newline and return status.                     */
#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0)))
        return (write (WARG(0), VDAT(VARG(1)), VLEN(VARG(1))) == -1 ||
                write (WARG(0), "\n", sizeof "\n") == -1) ? SP_ERROR : SP_SUCCESS;
    else {
#endif
        VDAT(VARG(1))[VLEN(VARG(1))] = '\0';
        return (fputs (VDAT(VARG(1)), STREAM (WARG(0))) == EOF ||
                fputs ("\n", STREAM (WARG(0))) == EOF) ? SP_ERROR : SP_SUCCESS;
#if PSERVER_EXTENSION
    }
#endif
}


/*
 *
 *      int file_flush ()
 *
 *  Flushes the specified stream, which should be open for output.  Any
 *  internally buffered data is written to the destination device.  The
 *  stream remains open.
 *
 *  Args:   INT32   stream              Rets:   -
 *
 */

SERVER_FN (file_flush, 1, 0)

#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0)))
        return SP_SUCCESS;
    else
#endif
        /* Flush stream and return status.                              */
        return fflush (STREAM (WARG(0))) == EOF ? SP_ERROR : SP_SUCCESS;
}


/*
 *
 *      int file_seek ()
 *
 *  Set file position for stream.  A subsequent read or write will access
 *  data at the new position.  For a binary stream the new position will be
 *  pos bytes from whence (set, current or end).  For a text stream, pos
 *  must be zero or a value returned by ftell().  If the latter is used,
 *  whence must be 'set'.
 *
 *  Args:   INT32   stream              Rets:   -
 *          INT32   pos
 *          INT32   whence
 *
 */

SERVER_FN (file_seek, 3, 0)
{
    int whence;

    /* Check whence and convert to host-defined value.                  */
    switch (WARG(2)) {
    case SP_SEEK_SET:   whence = SEEK_SET;          break;
    case SP_SEEK_CUR:   whence = SEEK_CUR;          break;
    case SP_SEEK_END:   whence = SEEK_END;          break;
    default:    error ("file_seek(): illegal whence %d", WARG(2));
                return SP_ERROR;
    }

    /* Seek in stream and return status.                                */
#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0)))
        return lseek (WARG(0), WARG(1), whence) == -1 ? SP_ERROR : SP_SUCCESS;
    else
#endif
        return fseek (STREAM (WARG(0)), WARG(1), whence) == EOF ? SP_ERROR : SP_SUCCESS;
}}


/*
 *
 *      int file_tell ()
 *
 *  Return current position in stream.
 *
 *  Args:   INT32   stream              Rets:   INT32   pos
 *
 */

SERVER_FN (file_tell, 1, 1)

    /* Determine position in stream.                                    */
#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0)))
        WRET(0) = tell (WARG(0));
    else
#endif
        WRET(0) = ftell (STREAM (WARG(0)));

    /* Return error if ftell failed, success otherwise.                 */
    return WRET(0) == -1 ? SP_ERROR : SP_SUCCESS;
}


/*
 *
 *      int file_eof ()
 *
 *  Returns success if end of file indicator for stream is set.
 *
 *  PACT EXTENSION: If file was opened with open2 (fildes), always return
 *                  SUCCESS - it's up to the runtime to interpret the
 *                  -1 count returned by {GET|PUT}BLOCK as meaning error.
 *
 *  Args:   INT32   stream              Rets:   -
 *
 */

SERVER_FN (file_eof, 1, 0)

#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0)))
        return SP_SUCCESS;
    else
#endif
        /* Return success if eof.                                       */
        return feof (STREAM (WARG(0))) ? SP_SUCCESS : SP_ERROR;
}


/*
 *
 *      int file_ferror ()
 *
 *  Get error file status.  If set, return a host-defined error number
 *  and message.  PACT extension: if stream arg is -1, return errno
 *  and string.
 *
 *  Args:   INT32   stream              Rets:   INT32   errno
 *                                              VEC     string
 *
 */

SERVER_FN (file_ferror, 1, 2)

    /* Check stream error, and return errno and possibly string if set. */
    if (
#if PSERVER_EXTENSION
        (pserverext && WARG(0) == -1) ||
        ISFILDES(WARG(0)) ||
#endif
        ferror (STREAM (WARG(0)))
       ) {
        int maxstrlen = MAXPACKSZ-VLENSZ-1-4-VLENSZ;
        WRET(0) = errno;
#ifdef  HAS_STRERROR
        strncpy (VDAT(sbuffer), strerror (errno), maxstrlen-1);
        VDAT(sbuffer)[maxstrlen-1] = '\0';
#else
        VDAT(sbuffer)[0] = '\0';
#endif
        SETVLEN (sbuffer, strlen (VDAT(sbuffer)));
        VRET(1) = sbuffer;
        return SP_SUCCESS;
    }

    /* Return error result if no stream error.                          */
    else
        return SP_ERROR;
}


/*
 *
 *      int file_remove ()
 *
 *  Delete a named file.
 *
 *  Args:   VEC     name                Rets:   -
 *
 */

SERVER_FN (file_remove, 1, 0)

    /* Null terminate filename.                                         */
     VDAT(VARG(0))[VLEN(VARG(0))] = '\0';

    /* Remove and return status.                                        */
    return remove ((char *) VDAT(VARG(0))) ? SP_ERROR : SP_SUCCESS;
}


/*
 *
 *      int file_rename ()
 *
 *  Change name of old file to new file.
 *
 *  Args:   VEC     old                 Rets:   -
 *          VEC     new
 *
 */

SERVER_FN (file_rename, 2, 0)
{
    char    *old, *new;

    /* Get new and old names, and null-terminate.  Must be careful,     */
    /* because the string terminator for the old name overwrites the    */
    /* length of the new name.  So do new name first, then old name.    */
    new = (char *) VDAT(VARG(1));
    new[VLEN(VARG(1))] = '\0';
    old = (char *) VDAT(VARG(0));
    old[VLEN(VARG(0))] = '\0';

#if __TURBOC__
    /* Remove new file, if any, if old name exists.                     */
    if (rename (old, old) == 0)
        remove (new);
#endif

    /* Rename and return status.                                        */
    return rename (old, new) ? SP_ERROR : SP_SUCCESS;
}}


/*
 *
 *      int file_isatty ()
 *
 *  Return success if stream is connected to a terminal, error otherwise.
 *
 *  Args:   INT32   stream              Rets:   -
 *
 */

SERVER_FN (file_isatty, 1, 0)

#if PSERVER_EXTENSION
    if (pserverext && ISFILDES(WARG(0)))
        return isatty (WARG(0)) ? SP_SUCCESS : SP_ERROR;
    else
#endif
        return isatty (fileno (STREAM (WARG(0)))) ? SP_SUCCESS : SP_ERROR;
}


#if PSERVER_EXTENSION
/*
 *
 *      int file_access ()
 *
 *  Check if named file can be accessed in specified manner, and return
 *  success if so, and error if not.
 *
 *  Args:   VEC     path                Rets:   -
 *          INT32   amode
 *
 */

SERVER_FN (file_access, 2, 0)
{
    char    *path;
    int     amode, spamode;

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    /* Get amode and null terminate filename.                           */
    spamode = WARG(1);
    path = (char *) VDAT(VARG(0));
    path[VLEN(VARG(0))] = '\0';

    /* MS-DOS supports only F_OK and W_OK, values 0 and 2, resp.  I     */
    /* guess anything may be read and executed?                         */
#if OSID==OS_DOS
#   define  F_OK    0
#   define  W_OK    2

    /* XXX - if SP_R_OK or SP_X_OK, use stat() to figure out more? */
#endif

    amode = 0;
    if (spamode == SP_F_OK) amode = F_OK;
    else {
#ifdef  X_OK
        if (spamode & SP_X_OK)  amode |= X_OK;
#endif
        if (spamode & SP_W_OK)  amode |= W_OK;
#ifdef  R_OK
        if (spamode & SP_R_OK)  amode |= R_OK;
#endif
    }

    /* Return success if access allowed.                                */
    return access (path, amode) == -1 ? SP_ERROR : SP_SUCCESS;
}}


/*
 *
 *      int file_fcntl ()
 *
 *  Do things with stream.  See SVR3 or XPG3 docs for details.
 *
 *  Args:   BYTE    command             Rets:   INT32   value
 *          INT32   stream                      BYTE    locktype
 *          INT32   arg                         BYTE    lockwhence
 *          BYTE    locktype                    INT32   lockstart
 *          BYTE    lockwhence                  INT32   locklen
 *          INT32   lockstart                   INT32   lockpid
 *          INT32   locklen
 *          INT32   lockpid
 *
 */

SERVER_FN (file_fcntl,  8, 6)
{
    int             fildes;
    int             cmd;
    int             arg;
#if OSID!=OS_DOS
    struct flock    lock;
#endif
    int             res;

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    /* Fcntl is a UNIX command - not sure how much works on non-UNIX.   */
#if OSID==OS_SUNOS || OSID==OS_USV386
    /* Get info passed into the proper variables and structures.        */
    switch (BARG(0)) {
    case SP_F_DUPFD:    cmd = F_DUPFD;  break;
    case SP_F_GETFD:    cmd = F_GETFD;  break;
    case SP_F_SETFD:    cmd = F_SETFD;  break;
    case SP_F_GETFL:    cmd = F_GETFL;  break;
    case SP_F_SETFL:    cmd = F_SETFL;  break;
    case SP_F_GETLK:    cmd = F_GETLK;  break;
    case SP_F_SETLK:    cmd = F_SETLK;  break;
    case SP_F_SETLKW:   cmd = F_SETLKW; break;
    default:            error ("file_fcntl(): illegal command %d\n", BARG(0));
                        return SP_ERROR;
    }
    fildes = WARG(1);
    if (! ISFILDES(fildes))             /* too much hassle to support streams as well */
        return SP_ERROR;
    switch (cmd) {
    case F_DUPFD: case F_SETFD: case F_SETFL:
        arg = WARG(2);
        break;
    case F_GETLK: case F_SETLK: case F_SETLKW:
        switch (BARG(3)) {
        case SP_F_RDLCK:    lock.l_type = F_RDLCK;      break;
        case SP_F_WRLCK:    lock.l_type = F_WRLCK;      break;
        case SP_F_UNLCK:    lock.l_type = F_UNLCK;      break;
        default:            error ("file_fcntl(): illegal lock type %d\n", BARG(3));
                            return SP_ERROR;
        }
        switch (BARG(4)) {
        case SP_SEEK_SET:   lock.l_whence = SEEK_SET;   break;
        case SP_SEEK_CUR:   lock.l_whence = SEEK_CUR;   break;
        case SP_SEEK_END:   lock.l_whence = SEEK_END;   break;
        default:            error ("file_fcntl(): illegal whence %d", BARG(4));
                            return SP_ERROR;
        }
        lock.l_start = WARG(5);
        lock.l_len   = WARG(6);
        lock.l_pid   = WARG(7);
        arg = (int) &lock;
        break;
    }

    /* Do fcntl, return result, or SP_ERROR if failed.                  */
    WRET(0) = fcntl (fildes, cmd, arg);
    if (WRET(0) == -1)
        return SP_ERROR;

    /* Return lock structure.                                           */
    switch (cmd) {
    case F_GETLK: case F_SETLK: case F_SETLKW:
        switch (lock.l_type) {
        case F_RDLCK:   BRET(1) = SP_F_RDLCK;   break;
         case F_WRLCK:   BRET(1) = SP_F_WRLCK;   break;
        case F_UNLCK:   BRET(1) = SP_F_UNLCK;   break;
        default:        error ("file_fcntl(): fcntl() returned illegal lock type %d\n", lock.l_type);
                        return SP_ERROR;
        }
        switch (lock.l_whence) {
        case SEEK_SET:  BRET(2) = SP_SEEK_SET;  break;
        case SEEK_CUR:  BRET(2) = SP_SEEK_CUR;  break;
        case SEEK_END:  BRET(2) = SP_SEEK_END;  break;
        default:        error ("file_fcntl(): fcntl() returned illegal whence %d", lock.l_whence);
                        return SP_ERROR;
        }
        WRET(3) = lock.l_start;
        WRET(4) = lock.l_len;
        WRET(5) = lock.l_pid;
        break;
    default:
        BRET(1) = 0;
        BRET(2) = 0;
        WRET(3) = 0;
        WRET(4) = 0;
        WRET(5) = 0;
    }

    /* Return success if OK.                                            */
    return SP_SUCCESS;

    /* Not implemented on non-UNIX.                                     */
#else

    return SP_UNIMPLEMENTED;
#endif
}}


/*
 *
 *      int file_stat ()
 *
 *  Return stat info of named file or opened stream (if not -1).
 *
 *  Args:   VEC     path                Rets:   INT32   mode
 *          INT32   stream                      INT32   ino
 *                                              INT32   dev
 *                                              INT32   rdev
 *                                              INT32   nlink
 *                                              INT32   uid
 *                                              INT32   gid
 *                                              INT32   size
 *                                              INT32   atime
 *                                              INT32   mtime
 *                                              INT32   ctime
 *
 */

SERVER_FN (file_stat,   2, 11)
{
#ifndef HAVE_NO_STAT
    struct stat s;
#endif
    int         mode;

    if (! pserverext)
        return SP_UNIMPLEMENTED;

#ifndef HAVE_NO_STAT
     /* Fstat() if stream not -1.                                        */
    if (WARG(1) != -1) {
        if (fstat (ISFILDES(WARG(1)) ? WARG(1) : fileno(STREAM(WARG(1))), &s) == -1)
            return SP_ERROR;
    }

    /* Or fstat() if no path.                                           */
    else {
        char    *path = (char *) VDAT(VARG(0));
        path[VLEN(VARG(0))] = '\0';
        if (stat (path, &s) == -1)
            return SP_ERROR;
    }

    /* Convert mode from host bits to SP.                               */
    mode = 0;
    switch (s.st_mode & S_IFMT) {
    case S_IFIFO:   mode = SP_S_IFIFO;  break;
    case S_IFCHR:   mode = SP_S_IFCHR;  break;
    case S_IFDIR:   mode = SP_S_IFDIR;  break;
    case S_IFBLK:   mode = SP_S_IFBLK;  break;
    case S_IFREG:   mode = SP_S_IFREG;  break;
    default:        error ("file_stat(): [f]stat() returned illegal mode %x\n", s.st_mode);
                    return SP_ERROR;
    }
    if (s.st_mode & S_ISUID)    mode |= SP_S_ISUID;
    if (s.st_mode & S_ISGID)    mode |= SP_S_ISGID;
    if (s.st_mode & S_ISVTX)    mode |= SP_S_ISVTX;
#ifdef S_ENFMT
    if (s.st_mode & S_ENFMT)    mode |= SP_S_ENFMT;
#endif
    if (s.st_mode & S_IRUSR)    mode |= SP_S_IRUSR;
    if (s.st_mode & S_IWUSR)    mode |= SP_S_IWUSR;
    if (s.st_mode & S_IXUSR)    mode |= SP_S_IXUSR;
    if (s.st_mode & S_IRGRP)    mode |= SP_S_IRGRP;
    if (s.st_mode & S_IWGRP)    mode |= SP_S_IWGRP;
    if (s.st_mode & S_IXGRP)    mode |= SP_S_IXGRP;
    if (s.st_mode & S_IROTH)    mode |= SP_S_IROTH;
    if (s.st_mode & S_IWOTH)    mode |= SP_S_IWOTH;
    if (s.st_mode & S_IXOTH)    mode |= SP_S_IXOTH;

    /* Return all fields.                                               */
    WRET(0) = mode;
    WRET(1) = s.st_ino;
    WRET(2) = s.st_dev;
    WRET(3) = s.st_rdev;
    WRET(4) = s.st_nlink;
    WRET(5) = s.st_uid;
    WRET(6) = s.st_gid;
    WRET(7) = s.st_size;
    WRET(8) = s.st_atime;
    WRET(9) = s.st_mtime;
    WRET(10)= s.st_ctime;

    /* XXX - may need to fix time fields for non-DOS/UNIX */

    /* Return success.                                                  */
    return SP_SUCCESS;
#else

    /* Not implemented for all systems yet.                             */
    return SP_UNIMPLEMENTED;
#endif
}}


/*
 *
 *      int file_link ()
 *
 *  Link old file to new one and return success/error.  Always returns
 *  error on MS-DOS, where this doesn't work.
 *
 *  Args:   VEC     old                 Rets:   -
 *          VEC     new
 *
 */

SERVER_FN (file_link,   2, 0)
{
    char    *old, *new;

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    /* Get new and old names, and null-terminate.  Must be careful,     */
    /* because the string terminator for the old name overwrites the    */
    /* length of the new name.  So do new name first, then old name.    */
    new = (char *) VDAT(VARG(1));
    new[VLEN(VARG(1))] = '\0';
    old = (char *) VDAT(VARG(0));
    old[VLEN(VARG(0))] = '\0';

    /* On UNIX, do link() call and return status.                       */
#if OSID==OS_SUNOS || OSID==OS_USV386
    return link (old, new) == -1 ? SP_ERROR : SP_SUCCESS;

    /* No such thing on MS-DOS.                                         */
#else
    return SP_ERROR;
#endif
}}


/*
 *
 *      int file_unlink ()
 *
 *  Unlink file or directory.  Returns success if succeeded, error otherwise.
 *
 *  Args:   VEC     path                Rets:   -
 *
 */

SERVER_FN (file_unlink, 1, 0)
{
    char    *path = (char *) VDAT(VARG(0));

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    /* Null terminate filename.                                         */
    path[VLEN(VARG(0))] = '\0';

    /* On MS-DOS, call rmdir() if path is directory.                    */
#if OSID==OS_DOS && !defined(HAVE_NO_STAT)
    {
        struct stat s;
        if (stat (path, &s) == -1) {
            return SP_ERROR;
        if (s.st_mode & S_IFDIR)
            return rmdir (path) == -1 ? SP_ERROR : SP_SUCCESS;
     }
#endif

    /* Unlink and return status.                                        */
    return unlink (path) == -1 ? SP_ERROR : SP_SUCCESS;
}}
#endif

