/*
 *
 *  @(#)helios.c    1.2 93/03/03    (c) PACT
 *
 *  PSERVER - PACT host server
 *
 *  Copyright (c) 1991, 1992 PACT
 *  All Rights Reserved
 *
 *  Helios interface module.
 *
 */

#include    <stdio.h>                   /* standard I/O */
#include    <helios.h>                  /* Helios stuff */
#include    <config.h>                  /* Helios config info */
#include    <root.h>                    /* Helios root info */
#include    "linkio.h"                  /* link interface definition */
#include    "pconf.h"                   /* config info (for BOARDID) */


/*
 *
 *      The fact that a link is currently active link is recorded.
 *
 */

#define FREELINK    (-1)
static  link_t  activelink = FREELINK;


/*
 *
 *      Info about Inmos and Parsytec subsystem reset mechanisms.
 *
 */

#if BOARDID == BOARD_PARSY_SC
#   define  SUB_PARSY   0xC0
#   define  MSEC(x)     (Delay ((x) * 1000))

#else   /* inmos */
#   define  SUB_RESET   0x0000000
#   define  SUB_ANALYSE (SUB_RESET + 4)
#   define  SUB_ERROR   (SUB_RESET)
#   define  PAUSE       {int iii; for (iii=0;iii<10000;iii++); }
#endif

#if __STDC__
#   define  VOLATILE    volatile
#else
#   define  VOLATILE    /**/
#endif


/*
 *
 *      link_t  OpenLink (char *link)
 *
 *  Open link.  If link is NULL or empty, the default link is used, which
 *  is link 2.  Otherwise, link should specify a hex link number (0..3)
 *  to use.  Returns negative number if an error occurs, and a positive
 *  linkid if the link was successfully opened.
 *
 */

#if  __STDC__
link_t  OpenLink (char *link)
#else
link_t  OpenLink (link)
char    *link;
#endif
{
    LinkInfo    *l;                 /* Helios link info */
    LinkConf    linkconf[4];        /* and link config */
    int i;

    /* If a link is already active, we can't open another.              */
    if (activelink != FREELINK)
        return ER_LINK_CANT;

    /* Default link is link 2.                                          */
    if (!link || sscanf (link, "%x", &activelink) != 1)
        activelink = 2;

    /* Check link number specified - error if non-existing.             */
    if (activelink > 3 || activelink < 0) {
        activelink = FREELINK;
        return ER_LINK_CANT;
    }

    /* Find out if link to use is active Helios link.                   */
    l = &(*(GetRoot()->Links))[activelink];
    if (l->Mode == Link_Mode_Intelligent && l->State != Link_State_Dead) {
        fprintf (stderr, "OpenLink: link %d is an active Helios link\n",
                                                            activelink);
        activelink = FREELINK;
        return ER_LINK_SOFT;
    }

    /* Reconfigure links.  Can't seem to just change one link's state.  */
    for (i=0; i<4; i++) {
        linkconf[i].Flags = 0;
        linkconf[i].Mode = Link_Mode_Intelligent;
        linkconf[i].State = Link_State_Running;
        linkconf[i].Id = i;
    }
    linkconf[activelink].Mode = Link_Mode_Dumb;
    if (Reconfigure (linkconf) < 0) {
        activelink = FREELINK;
        return ER_LINK_SOFT;
    }

    /* Return link descriptor.                                          */
    return activelink;
}


/*
 *
 *      int CloseLink (link_t linkid)
 *
 *  Close link.  Must be open to be closed.  If any error occurs (e.g. link
 *  is not open), a negative number is returned.  Otherwise, 1 is returned.
 *
 */

#if __STDC__
int     CloseLink (link_t linkid)
#else
int     CloseLink (linkid)
link_t  linkid;
#endif
{
    LinkConf linkconf[4];
    int i;

    /* If given link id is the one open, flag it as closed.  If not,    */
    /* return ER_LINK_BAD to indicate error.                            */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Reconfigure links to ensure NCM can still be accessed to return  */
    /* processors to pool when logging out of Helios session on         */
    /* Parsytec machine.                                                */
    for (i=0; i<4; i++) {
        linkconf[i].Flags = 0;
        linkconf[i].Mode = Link_Mode_Intelligent;
        linkconf[i].State = Link_State_Running;
        linkconf[i].Id = i;
    }
    linkconf[activelink].State = Link_State_Dead;
    if (Reconfigure (linkconf) < 0)
        return ER_LINK_SOFT;

    /* Link no longer open.                                             */
    activelink = FREELINK;

    /* Return 1 to indicate success.                                    */
    return 1;
}


/*
 *
 *      int ReadLink (link_t linkid, char *buf, unsigned len, int timout)
 *
 *  Read len bytes from specified link into buffer.  Returns number of
 *  bytes read (before timeout), or a negative number if an error occurs
 *  (wrong link or 0-size transfer).
 *
 */

#if __STDC__
int     ReadLink (link_t linkid, char *buf, unsigned len, int timout)
#else
int     ReadLink (linkid, buf, len, timout)
link_t      linkid;
char        *buf;
unsigned    len;
int         timout;
#endif
{
    /* If linkid is not the active link, fail.                          */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Zero-size transfers can't be done.                               */
    if (len < 1)
        return ER_LINK_CANT;

    /* Read from transputer link and return # bytes done.               */
    if (LinkIn (len, linkid, (void *) buf, timout*(OneSec/10)) < 0)
        return 0;
    else
        return len;
}


/*
 *
 *      int WriteLink (link_t linkid, char *buf, unsigned len, int timout)
 *
 *  Write len bytes from specified link into buffer.  Returns number of
 *  bytes written (before timeout), or a negative number if an error occurs
 *  (wrong link or 0-size transfer).
 *
 */

#if __STDC__
int     WriteLink (link_t linkid, char *buf, unsigned len, int timout)
#else
int     WriteLink (linkid, buf, len, timout)
link_t      linkid;
char        *buf;
unsigned    len;
int         timout;
#endif
{
    /* If linkid is not the active link, fail.                          */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Zero-size transfers can't be done.                               */
    if (len < 1)
        return ER_LINK_CANT;

    /* Write to transputer link and return # bytes done.                */
    if (LinkOut (len, linkid, (void *) buf, timout*(OneSec/10)) < 0)
        return 0;
    else
        return len;
}


/*
 *
 *      int ResetLink (link_t linkid)
 *
 *  Reset transputer.  Returns 1 if successful, and a negative number to
 *  indicate an error.
 *
 */

#if __STDC__
int     ResetLink (link_t linkid)
#else
int     ResetLink (linkid)
link_t  linkid;
#endif
{
    VOLATILE int    *p;

    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Reset subsystem with Parsytec nearest neighbour reset.           */
#if BOARDID == BOARD_PARSY_SC
    p = (VOLATILE int *) SUB_PARSY;
    *p = 0;
    *p = 1;
    *p = 2;
    *p = 3;
     *p = 1 << linkid;
    MSEC (1);               /* wait 1 msec */
    *p = 0;
#else

    /* Or reset subsystem with Inmos reset.                             */
    p = (VOLATILE int *) SUB_RESET;    *p = 0;
    p = (VOLATILE int *) SUB_ANALYSE;  *p = 0;  PAUSE;
    p = (VOLATILE int *) SUB_RESET;    *p = 1;  PAUSE;
    p = (VOLATILE int *) SUB_RESET;    *p = 0;  PAUSE;
#endif

    /* Return 1 to indicate success.                                    */
    return 1;
}


/*
 *
 *      int AnalyseLink (link_t linkid)
 *
 *  Analyse transputer.  Returns 1 if successful, and a negative number to
 *  indicate an error.  On Parsytec systems, actually does a reset.
 *
 */

#if __STDC__
int     AnalyseLink (link_t linkid)
#else
int     AnalyseLink (linkid)
link_t  linkid;
#endif
{
    VOLATILE int    *p;

    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* On Parsytec system, do a reset instead.                          */
#if BOARDID == BOARD_PARSY_SC
    ResetLink (linkid);
#else

    /* Do analyse sequence on Inmos subsystem.                          */
    p = (VOLATILE int *) SUB_ANALYSE;  *p = 0;  PAUSE;
    p = (VOLATILE int *) SUB_ANALYSE;  *p = 1;  PAUSE;
    p = (VOLATILE int *) SUB_RESET;    *p = 1;  PAUSE;
    p = (VOLATILE int *) SUB_RESET;    *p = 0;  PAUSE;
    p = (VOLATILE int *) SUB_ANALYSE;  *p = 0;  PAUSE;
#endif

    /* Return 1 to indicate success.                                    */
    return 1;
}


/*
 *
 *      int TestError (link_t linkid)
 *
 *  Returns state of ErrorFlag.  Not supported on Parsytec systems.
 *
 */

#if __STDC__
int      TestError (link_t linkid)
#else
int     TestError (linkid)
link_t  linkid;
#endif
{
    VOLATILE int    *p;

    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Don't know how on Parsytec system.                               */
#if BOARDID == BOARD_PARSY_SC
    return ER_LINK_SOFT;
#else

    /* Check Inmos subsystem.                                           */
    p = (VOLATILE int *) SUB_ERROR;
    return *p & 1;
#endif
}


/*
 *
 *      int TestRead (link_t linkid)
 *
 *  Returns 1 if transputer is ready to receive data.  Not supported.
 *
 */

#if __STDC__
int     TestRead (link_t linkid)
#else
int     TestRead (linkid)
link_t  linkid;
#endif
{
    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Not supported.                                                   */
    return ER_LINK_SOFT;
}


/*
 *
 *      int TestWrite (link_t linkid)
 *
 *  Returns 1 if transputer is ready to receive data.  Not supported.
 *
 */

#if __STDC__
int     TestWrite (link_t linkid)
#else
int     TestWrite (linkid)
link_t  linkid;
#endif
{
    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Not supported.                                                   */
    return ER_LINK_SOFT;
}



