/*
 *
 *  @(#)host.c  1.11    93/08/12    (c) PACT
 *
 *  PSERVER - PACT host server
 *
 *  Copyright (c) 1991-1993 PACT
 *  All Rights Reserved
 *
 *  Host specifics.
 *
 */

#include    <time.h>                    /* time support */
#include    "pserver.h"                 /* general defs */
#include    "error.h"                   /* error manager */
#include    "host.h"                    /* own defs and decls */
#include    "server.h"                  /* server defs */

#ifdef LINUX    /* they say that LINUX is POSIX, but they lie */
#undef _POSIX_SOURCE
#endif

#if OSID==OS_SUNOS || OSID==OS_USV386
#ifdef _POSIX_SOURCE 
  #include <sys/termios.h>
#else
#   include         <termio.h>          /* terminal support */
#endif
#   if  PSERVER_EXTENSION
#       include <sys/utsname.h>         /* uname support */
#   endif
#endif



/*
 *
 *      char *filemodes[2][6]
 *
 *  Array of 12 file mode strings used by file_open().  First 6 are for
 *  binary files, latter 6 for text files.  The 6 modes supported are:
 *  read only, write only & truncate if exist, write only & append if exists,
 *  update, update & truncate if exists, update & append if exists, resp.
 *
 */

#if OSID==OS_DOS
char    *filemodes[2][6] = { { "rb", "wb", "ab", "rb+", "wb+", "ab+" },
                             { "rt", "wt", "at", "rt+", "wt+", "at+" } };
#else   /* UNIX */
char    *filemodes[2][6] = { { "r",  "w",  "a",  "r+",  "w+",  "a+" },
                             { "r",  "w",  "a",  "r+",  "w+",  "a+" } };
#endif


#ifndef HAS_RENAME
/*
 *
 *      int rename (char *old, char *new)
 *
 *  Rename old file into new file.  Return 0 if success, !0 if error.
 *
 */

int     rename (old, new)
char    *old;
char    *new;
{
    return link (old, new) ||
           unlink (old);
}
#endif  /* !HAS_RENAME */


#ifndef HAS_REMOVE
/*
 *
 *      int remove (char *name)
 *
 *  Delete named file.  Return 0 if success, !0 if error.
 *
 */

int     remove (name)
char    *name;
{
    return unlink (name);
}
#endif  /* !HAS_REMOVE */


/*
 *
 *      int host_getkey ()
 *
 *  Return single character from keyboard, without echo.  Wait indefinitely.
 *
 *  Args:   -                       Rets:   BYTE    key
 *
 */

SERVER_FN (host_getkey, 0, 1)
{
    char    c;

    /* Turbo C's kbhit() function tells us if there's a key in the      */
    /* keyboard buffer.  The getch() function gets the character        */
    /* without echo.                                                    */
#if OSID==OS_DOS

    /* Wait for a key.                                                  */
    while (! kbhit ())
        ;

    /* Get character.                                                   */
    c = getch ();

    /* Under UNIX, we must switch the terminal into poll mode, wait for */
    /* a character, and switch the terminal back.                       */
#elif   OSID==OS_SUNOS || OSID==OS_USV386

    #ifdef _POSIX_SOURCE
    struct termios  orgmode, mode;
    #else
    struct termio   orgmode, mode;
    #endif

    /* Get current mode.                                                */
    #ifdef _POSIX_SOURCE
    tcgetattr(0, &orgmode);
    #else
    ioctl (0, TCGETA, &orgmode);
    #endif

    /* Change mode for polling of single character without echo.        */
    mode = orgmode;
    mode.c_iflag &= ~ICRNL;
    mode.c_lflag &= ~ (ICANON | ECHO);
    mode.c_cc[VTIME] = 1;
    mode.c_cc[VMIN] = 0;
    #ifdef _POSIX_SOURCE
      tcsetattr(0, TCSANOW, &mode);
    #else
    ioctl (0, TCSETA, &mode);
    #endif

    /* Wait for character to be entered.                                */
    while (read (0, &c, 1) == 0)
        ;

    /* Restore original terminal mode.                                  */
    #ifdef _POSIX_SOURCE
      tcsetattr(0, TCSANOW, &orgmode);
    #else
    ioctl (0, TCSETA, &orgmode);
    #endif

    /* Fail on other systems.                                           */
#else
    return SP_ERROR;
#endif

    /* Return character typed and success.                              */
    BRET(0) = c;
    return SP_SUCCESS;
}}


/*
 *
 *      int host_pollkey ()
 *
 *  Return single character from keyboard, if available, or return with
 *  non-zero result if no character available.  No echo.
 *
 *  Args:   -                       Rets:   BYTE    key
 *
 */

SERVER_FN (host_pollkey, 0, 1)
{
    char    c;
    int     stat = SP_ERROR;

    /* Turbo C's kbhit() function tells us if there's a key in the      */
    /* keyboard buffer.  The getch() function gets the character        */
    /* without echo.                                                    */
#if OSID==OS_DOS

    /* Get key if present.                                              */
    if (kbhit ()) {
        c = getch ();
        stat = SP_SUCCESS;
    }

    /* Under UNIX, we must switch the terminal into poll mode, see if   */
    /* there is a character, and switch the terminal back.              */
#elif   OSID==OS_SUNOS || OSID==OS_USV386

    #ifdef _POSIX_SOURCE
    struct termios  orgmode, mode;
    #else
    struct termio   orgmode, mode;
    #endif

    /* Get current mode.                                                */
    #ifdef _POSIX_SOURCE
    tcgetattr(0, &orgmode);
    #else
    ioctl (0, TCGETA, &orgmode);
    #endif

    /* Change mode for polling of single character without echo.        */
    mode = orgmode;
    mode.c_iflag &= ~ICRNL;
    mode.c_lflag &= ~ (ICANON | ECHO);
    mode.c_cc[VTIME] = 1;
    mode.c_cc[VMIN] = 0;
    #ifdef _POSIX_SOURCE
      tcsetattr(0, TCSANOW, &mode);
    #else
    ioctl (0, TCSETA, &mode);
    #endif

    /* Read char, if any.                                               */
    if (read (0, &c, 1) == 1)
        stat = SP_SUCCESS;

    /* Restore original terminal mode.                                  */
    #ifdef _POSIX_SOURCE
      tcsetattr(0, TCSANOW, &orgmode);
    #else
    ioctl (0, TCSETA, &orgmode);
    #endif

    /* Fail on other systems.                                           */
#else
     return SP_ERROR;
#endif

    /* Return character typed and success, or error if no character.    */
    BRET(0) = c;
    return stat;
}}


/*
 *
 *      int host_getenv ()
 *
 *  Get environment variable.  If the definition is longer than the
 *  buffer (1024 minus a bit), it is truncated.
 *
 *  Args:   VEC     name            Rets:   VEC     value
 *
 */

SERVER_FN (host_getenv, 1, 1)
{
    char    *value;
    extern  char    *getenv();

    /* Null-terminate environment variable name.                        */
    VDAT(VARG(0))[VLEN(VARG(0))] = '\0';

    /* Get value of environment variable.                               */
    value = getenv (VDAT(VARG(0)));

    /* If defined, set up return value.                                 */
    if (value) {
        int maxstrlen = MAXPACKSZ-VLENSZ-1-VLENSZ;

        /* If definition is too long, warn about truncation.            */
        if (strlen (value) > maxstrlen)
            error ("host_getenv (%s):  value truncated to %d bytes",
                                                    VDAT(VARG(0)), maxstrlen);

        /* Set up return vector.                                        */
        strncpy (VDAT(sbuffer), value, maxstrlen);
        SETVLEN (sbuffer, strlen (value));
        VRET(0) = sbuffer;
    }

    /* Return success.                                                  */
    return value ? SP_SUCCESS : SP_ERROR;
}}


/*
 *
 *      int host_putenv ()
 *
 *  Put environment variable.
 *
 *  Args:   VEC     name            Rets:       -
 *          VEC     value
 *
 */

SERVER_FN (host_putenv, 2, 0)
{
    #ifdef NEXT
      /* putenv does not exist under NEXTSTEP */
      return SP_ERROR;
    #else
    char    *value;
    int     vlen, i;
     extern  int putenv();

    /* Copy value after name, separated by '=', null-terminated.        */
    value = (char *) VDAT(VARG(1));
    vlen = VLEN(VARG(1));
    value[vlen] = '\0';
    VDAT(VARG(0))[VLEN(VARG(0))] = '=';
    for (i=0; i<vlen+1; i++)
        VDAT(VARG(0))[VLEN(VARG(0))+1+i] = value[i];

    /* Get value of environment variable, return success or error.      */
    /* XXX - old string, if any, is never freed! */
    return putenv (strdup (VDAT(VARG(0)))) == 0 ? SP_SUCCESS : SP_ERROR ;
    #endif
}}


/*
 *
 *      int host_time ()
 *
 *  Returns current local time and UTC time (if available).  Both times
 *  are expressed as number of seconds since midnight, Jan 1 1970.  If
 *  UTC time is unavailable, it is set to 0.
 *
 *  Args:   -                           Rets:   INT32   local time
 *                                              INT32   UTC time
 *
 */

SERVER_FN (host_time, 0, 2)
{
    time_t  timer,
            curtime,
            UTCtime;

    /* Get current calendar time, in a host-defined format.  This       */
    /* time() function appears to work on most systems.                 */
    /* Note: We assume that the host returns a time in seconds since    */
    /*       some fixed point.  If it does not, we're in for a lot more */
    /*       work.                                                      */
    (void) time (&timer);

    /* Now use the current calendar time to get a local time and a UTC  */
    /* time.  Default is to just use the calendar time as local time    */
    /* and give no UTC time.                                            */
    curtime = timer;
    UTCtime = 0;

    /* On many systems, including SV Unix and Turbo C, there's a        */
    /* function tzset() which stores the timezone offset in seconds in  */
    /* a global variable called 'timezone'.                             */
#if OSID==OS_DOS || OSID==OS_USV386
{   extern  long    timezone;
    extern  void    tzset ();
    UTCtime = timer;
    tzset ();
    curtime = UTCtime - timezone;
}

    /* SunOS >4.0 has Standard C localtime() and gmtime() functions,    */
    /* but these return (UNIX) time in seconds since Jan 1, 1970.       */
#elif OSID==OS_SUNOS && __STDC__
{
    curtime = mktime (localtime (&timer));
    UTCtime = mktime (gmtime    (&timer));
}

    /* The Standard C localtime() and gmtime() functions usually return */
    /* the number of seconds since Jan 1, 1900.                         */
#elif __STDC__
{
    curtime = mktime (localtime (&timer));
    UTCtime = mktime (gmtime    (&timer));
    curtime -= 25567*24*60*60;          /* subtract 70 years */
    UTCtime -= 25567*24*60*60;
}
#endif

    /* Return local time and UTC time.                                  */
    WRET(0) = curtime;
    WRET(1) = UTCtime;

    /* Return success.                                                  */
    return SP_SUCCESS;
}}


/*
 *
 *      int host_system ()
 *
 *  Run a command.  If the command string is zero length, the existance
 *  of a command processor (shell) is tested and the result is zero if
 *  it exists.  Otherwise the command is run in a host-defined manner and
 *  the status returned is host-defined.
 *
 *  Args:   VEC     command         Rets:   INT32   status
 *
 */

SERVER_FN (host_system, 1, 1)

    /* Null-terminate command string.                                   */
    VDAT(VARG(0))[VLEN(VARG(0))] = '\0';

    /* Special handling of zero length command: test shell and return   */
    /* success or failure.                                              */
    if ((VDAT(VARG(0)))[0] == '\0') {
        WRET(0) = system ("");
        return WRET(0) ? SP_ERROR : SP_SUCCESS;
    }

    /* Run command in shell and return status.                          */
    else {
        WRET(0) = system (VDAT(VARG(0)));
        return SP_SUCCESS;
    }
}


#if PSERVER_EXTENSION

/*
 *
 *      int host_getpid ()
 *
 *  Return process id, process group or parent process id.
 *
 *  Args:   BYTE    type            Rets:   INT32   id
 *
 */

SERVER_FN (host_getpid,  1, 1)

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    /* On UNIX, get process id, process group, or parent process id.    */
#if OSID==OS_SUNOS || OSID==OS_USV386
    switch (BARG(0)) {
    case SP_PID:    WRET(0) = getpid ();    break;
    case SP_PGRP:   WRET(0) = getpgrp ();   break;
    case SP_PPID:   WRET(0) = getppid ();   break;
    default:        error ("host_getpid(%d): illegal/unsupported id", BARG(0));
                    return SP_ERROR;
    }

    /* Return success.                                                  */
    return SP_SUCCESS;

    /* No such thing on MS-DOS.                                         */
#else
    return SP_ERROR;
#endif
}


/*
 *
 *      int host_getuid ()
 *
 *  Return user id, effective user id, group id, or effective group id.
 *
 *  Args:   BYTE    type            Rets:   INT32   id
 *
 */

SERVER_FN (host_getuid, 1, 1)

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    /* On UNIX, get user id, effective user id, group id or         */
    /* effective group id.                                          */
#if OSID==OS_SUNOS || OSID==OS_USV386
    switch (BARG(0)) {
    case SP_UID:    WRET(0) = getuid ();    break;
    case SP_EUID:   WRET(0) = geteuid ();   break;
    case SP_GID:    WRET(0) = getgid ();    break;
    case SP_EGID:   WRET(0) = getegid ();   break;
    default:        error ("host_getuid(%d): illegal/unsupported id", BARG(0));
                    return SP_ERROR;
    }

    /* Return success.                                                  */
    return SP_SUCCESS;

    /* No such thing on MS-DOS.                                         */
#else
    return SP_ERROR;
#endif
}


/*
 *
 *      int host_setuid ()
 *
 *   Set user id or group id.
 *
 *  Args:   BYTE    type            Rets:   -
 *          INT32   id
 *
 */

SERVER_FN (host_setuid, 2, 0)

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    /* On UNIX, set user id or group id.                                */
#if OSID==OS_SUNOS || OSID==OS_USV386
    switch (BARG(0)) {
    case SP_UID:    WRET(0) = setuid ();    break;
    case SP_GID:    WRET(0) = setgid ();    break;
    default:        error ("host_getuid(%d): illegal/unsupported id", BARG(0));
                    return SP_ERROR;
    }

    /* Return success.                                                  */
    return SP_SUCCESS;

    /* No such thing on MS-DOS.                                         */
#else
    return SP_ERROR;
#endif
}


/*
 *
 *      int host_kill ()
 *
 *  Send signal to process or group of processes.
 *
 *  Args:   INT32   pid         Rets:   -
 *          INT32   sig
 *
 */

SERVER_FN (host_kill,   2, 0)

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    /* On UNIX, send signal to process (w/ kill()), return OK or not.   */
#if OSID==OS_SUNOS || OSID==OS_USV386
    return kill ((int) WARG(0), (int) WARG(1)) == 0
                                ? SP_SUCCESS : SP_ERROR;

    /* No such thing on MS-DOS.                                         */
#else
    return SP_ERROR;
#endif
}


/*
 *
 *      int host_uname ()
 *
 *  Return various host characteristic strings.
 *
 *  Args:   -                       Rets:   VEC     sysname
 *                                           VEC     nodename
 *                                          VEC     release
 *                                          VEC     version
 *                                          VEC     machine
 *
 */

SERVER_FN (host_uname,  0, 5)
{
    /* On UNIX, do uname() call and copy results into vectors.          */
#if OSID==OS_SUNOS || OSID==OS_USV386
    byte_t  *s;
    struct  utsname u;

    if (! pserverext)
        return SP_UNIMPLEMENTED;

    if (uname (&u) == -1)
        return SP_ERROR;

    s = sbuffer;
    SETVLEN (s, sizeof(u.sysname));
    strncpy (VDAT(s), u.sysname, sizeof(u.sysname));
    VRET(0) = s;
    s += VLENSZ + sizeof (u.sysname);
    SETVLEN (s, sizeof(u.nodename));
    strncpy (VDAT(s), u.nodename, sizeof(u.nodename));
    VRET(1) = s;
    s += VLENSZ + sizeof (u.nodename);
    SETVLEN (s, sizeof(u.release));
    strncpy (VDAT(s), u.release, sizeof(u.release));
    VRET(2) = s;
    s += VLENSZ + sizeof (u.release);
    SETVLEN (s, sizeof(u.version));
    strncpy (VDAT(s), u.version, sizeof(u.version));
    VRET(3) = s;
    s += VLENSZ + sizeof (u.version);
    SETVLEN (s, sizeof(u.machine));
    strncpy (VDAT(s), u.machine, sizeof(u.machine));
    VRET(4) = s;

    /* Return success.                                                  */
    return SP_SUCCESS;

    /* Don't know what to do on MS-DOS - should be able to return       */
    /* something.                                                       */
#else
    return SP_UNIMPLEMENTED;
#endif
}}

#endif

