/*
 *
 *  @(#)main.c  1.25    94/04/10    (c) PACT
 *
 *  PSERVER - PACT host server
 *
 *  Copyright (c) 1991-1994 PACT
 *  All Rights Reserved
 *
 *  Main module.
 *
 *  Aug-16-1994, Nicolas Droux
 *    Ported to Linux
 *  Aug-25-1994, Nicolas Droux
 *    Modified for extended PPC CS-Lib pserver  
 *
 */

#include    "pserver.h"                 /* general defs */
#include    "error.h"                   /* error manager */
#include    "linkio.h"                  /* link interface definition */
#include    "server.h"                  /* server */
#include    <setjmp.h>                  /* longjmps */
#include    <signal.h>                  /* signals */

#ifdef CS_LIB
#include "cslib.h"
#endif

#if !__STDC__
extern  char    *getenv ();
extern  char    *malloc ();
#endif

#ifdef LINUX
#define getopt my_getopt
#endif

/*
 *
 *      RELEASE
 *
 *  Release number is externally defined, and defaults to "int" (internal).
 *
 */

#ifndef RELEASE
#   define  RELEASE "int"
#endif


/*
 *
 *      Define default search path, default extensions.
 *
 */

#if OSID == OS_DOS
#   define  DIRSEPC     '\\'
#   define  DEFTPATH    "\\pcc\\bin"
#else
#   define  DIRSEPC     '/'
#   define  DEFTPATH    "/pcc/bin"
#endif
#define DEFEXT  ".run"


/*
 *
 *      bool_t verbose
 *      bool_t debug
 *
 *  Global flags indicating verbose and debug modes.
 *
 */

bool_t  verbose = FALSE;
bool_t  debug   = FALSE;


/*
 *
 *      tint_t status
 *      jmp_buf exitenv
 *
 *  Some static data that must be preserved across the longjmp().
 *
 */

static  tint_t      status = 0;         /* pserver return status */
static  jmp_buf     exitenv;            /* to longjmp() out of break handler */
static  link_t      linkid = -1;        /* link id to work with */


/*
 *
 *      char *resetmode
 *
 *  Reset mode.
 *
 */

static  char        *resetmode = NULL;


/*
 *
 *      void usage (char *progname)
 *
 *  Print pserver usage and exit.
 *
 */

#if __STDC__
static  void    usage (char *progname)
#else
static  void    usage (progname)
char    *progname;
#endif
{
    fprintf (stderr,
"PACT host server / transputer    Rel %-4s    Copyright (c) 1991-1994 PACT\n",
    RELEASE);
    fprintf (stderr, "Usage: %s [options] program arguments ...\n", progname);
    fprintf (stderr, "\
  -a n          analyse root processor and peek n bytes of memory\n\
  -c ddd...     select a different processor as root\n\
");
#if OSID == OS_USV386
    fprintf (stderr, "\
  -d            fail if no free interface (don't wait)\n\
");
#endif
    fprintf (stderr, "\
  -e            terminate upon raising of the ErrorFlag\n\
");
#if OSID == OS_DOS && PSERVER_EXTENSION
    fprintf (stderr, "\
  -i srv=int    specify software interrupt to take upon specified request\n\
");
#endif
    fprintf (stderr, "\
  -l n          specify link to use\n\
");
#if  PSERVER_EXTENSION
    fprintf (stderr, "\
  -p            %sable pserver extensions\n\
",                                            strcmp(progname,"iserver") ? "dis" : "en");
#endif
    fprintf (stderr, "\
  -r            do not reset root processor\n\
");
#if SMART_WAIT
    fprintf (stderr, "\
  -s n          smart wait delay in seconds (def: 5)\n\
");
#endif
#if INACTIVE_MAX
    fprintf (stderr, "\
  -T n          terminate after link is inactive for n seconds\n\
");
#endif
    fprintf (stderr, "\
  -t            report times spent on transputer\n\
  -V            print version info\n\
  -v            verbose mode\n\
  -x            terminate after boot, do not serve\n\
");

    exit (3);
    /*NOTREACHED*/
}


/*
 *
 *      bool_t route (link_t linkid, char *route)
 *
 *  Setup specified route by booting intermediate processors with little
 *  message-passing program.
 *
 */

#if __STDC__
static  bool_t  route (link_t linkid, char *rootroute)
#else
static  bool_t  route (linkid, rootroute)
link_t  linkid;
char    *rootroute;
#endif
{
    static  char    booter[] = {
        0x8f, 0xd0, 0x27, 0x2f, 0x2f, 0x2f, 0x2f, 0x28,
        0x64, 0x40, 0x23, 0xfc, 0xd0, 0xd0, 0x21, 0x40,
        0x23, 0xf3, 0xd8, 0x24, 0xf2, 0x21, 0xf8, 0x24,
        0xf2, 0x21, 0xfc, 0x24, 0xf2, 0x27, 0x2f, 0x2f,
        0x2f, 0x2f, 0x2f, 0x6d, 0x44, 0xe0, 0x24, 0xf2,
        0x27, 0x2f, 0x2f, 0x2f, 0x2f, 0x2f, 0x6d, 0x48,
        0xe0, 0x40, 0x25, 0xf4, 0x22, 0xf9, 0x25, 0xf7,
        0x40, 0xd2, 0x48, 0xd3, 0x72, 0x24, 0xf2, 0xfa,
        0x21, 0xf2, 0x12, 0x4a, 0x22, 0xf1, 0x78, 0x41,
        0xfe, 0x28, 0x4f, 0xd6, 0x16, 0x23, 0x4f, 0x21,
        0xfb, 0xd7, 0x77, 0x41, 0x24, 0xfa, 0x65, 0x47,
        0x21, 0xfb, 0x23, 0x43, 0x21, 0xfb, 0x76, 0x24,
        0xfa, 0x44, 0x78, 0xfa, 0xd4, 0x12, 0x74, 0x44,
        0xf7, 0x72, 0x27, 0x2f, 0x2f, 0x2f, 0x2f, 0x2f,
        0x6f, 0xc0, 0xaf, 0x21, 0x4a, 0x21, 0xfb, 0x76,
        0x25, 0xf2, 0x23, 0xff, 0x41, 0x25, 0xf2, 0x44,
        0xf8, 0xd2, 0x13, 0x74, 0x44, 0xf7, 0x72, 0x78,
        0x21, 0x40, 0x25, 0xf2, 0x73, 0xf7, 0x72, 0xf6
     };
#if 0 /* @@ */
#   include "msgpass.h"
#else
    static  char    msgpasser[] = {
        0x00, 0x00, 0x00, 0x80, 0x73, 0x00, 0x00, 0x00,
        0x40, 0xd5, 0x15, 0x74, 0x41, 0xf7, 0x75, 0xc5,
        0x24, 0xa0, 0x40, 0xd5, 0x15, 0x74, 0x41, 0xf7,
        0x75, 0x44, 0x25, 0xf2, 0x24, 0xf2, 0xfa, 0xd3,
        0x75, 0x24, 0xf2, 0xfa, 0xd5, 0x42, 0xd1, 0x64,
        0x40, 0x21, 0xfb, 0xd0, 0x11, 0x60, 0x46, 0x10,
        0xd7, 0x77, 0xfa, 0xd6, 0x76, 0x22, 0x40, 0x24,
        0xfa, 0x4b, 0x76, 0xfd, 0x12, 0x74, 0x41, 0xf7,
        0x75, 0x72, 0xfe, 0x60, 0x07, 0x10, 0xf3, 0x11,
        0x72, 0x41, 0xf7, 0x77, 0x71, 0xfe, 0x60, 0x07,
        0x15, 0xf3, 0x75, 0x44, 0x25, 0xf2, 0x24, 0xf2,
        0xfa, 0xd3, 0x75, 0x24, 0xf2, 0xfa, 0xd5, 0x76,
        0x41, 0x25, 0xf2, 0xd6, 0x77, 0x75, 0x76, 0xfb,
        0x11, 0x73, 0x41, 0xf7, 0x75, 0x24, 0xf2, 0xff,
        0x75, 0x27, 0x43, 0xff, 0x72, 0x75, 0x27, 0x43,
        0xfb, 0x65, 0x0a
    };
#endif
    int res;

    /* Try to establish path to alternative root processor, if any.     */
    if (rootroute && rootroute[0]) {

        /* Boot root processor.                                         */
        res = WriteLink (linkid, booter, sizeof(booter), 5);
        if (res < 0)
            error ("error booting router: %s", lerror(res));
        else if (res != sizeof(booter))
            error ("timeout booting router after 0x%x bytes", res);
        else {
            char    ack;

            /* Expect to get byte 1 back from booter.                   */
            res = ReadLink (linkid, &ack, 1, 5);
            if (res < 0)
                error ("error booting router: %s", lerror(res));
            else if (res != 1)
                error ("router protocol error: no ack");
            else if (ack != 1)
                error ("router protocol error: ack %d instead of 1", ack);
            else {

                /* Send message passer code to root processor.          */
                res = WriteLink (linkid, msgpasser, sizeof(msgpasser), 5);
                if (res < 0)
                    error ("error booting router: %s", lerror(res));
                else if (res != sizeof(msgpasser))
                    error ("timeout booting router after 0x%x bytes", sizeof(booter)+res);
                else {
                    char    *r;

                    /* Now establish path.                              */
                    for (r=rootroute; *r; r++) {
                        char    linkspec;
                        static  int     resetbit = -1;

                        /* Find out reset mode once.                    */
                        if (resetbit == -1) {
                            if (resetmode || (resetmode = getenv ("TRESET")))
                                if (! strncmp (resetmode, "subsystem", strlen(resetmode)))
                                    resetbit = 0x40;
                                 else if (! strncmp (resetmode, "parsytec", strlen(resetmode)))
                                    resetbit = 0x80;
                                else if (! strncmp (resetmode, "none", strlen(resetmode)))
                                    resetbit = 0;
                                else {
                                    fprintf (stderr, "unknown TRESET reset mode: '%s'\n", resetmode);
                                    resetbit = 0;
                                }
                            else
                                resetbit = 0;
                        }

                        /* Indicate if this will be last link spec.     */
                        if (* (r + 1) == '\0') {
                            linkspec = 5;
                            res = WriteLink (linkid, &linkspec, 1, 5);
                            if (res != 1) {
                                if (res < 0)
                                    error ("error setting up route: %s", lerror(res));
                                else
                                    error ("route %s failed after %d processors",
                                                        rootroute, r-rootroute);
                                break;
                            }
                        }

                        /* Send link indicator (0..3).                  */
                        linkspec = (*r - '0') | resetbit;
                        res = WriteLink (linkid, &linkspec, 1, 5);
                        if (res != 1) {
                            if (res < 0)
                                error ("error setting up route: %s", lerror(res));
                            else
                                error ("route %s failed after %d processors",
                                                        rootroute, r-rootroute);
                            break;
                        }
                    }

                    /* Return success if entire path set up.            */
                    return *r == '\0';
                }
            }
        }

        /* If we didn't return success earlier, we didn't make it.      */
        return FALSE;
    }

    /* No route so succeeded.                                           */
    return TRUE;
}


/*
 *
 *      char *analyse (link_t linkid, utint_t *psize, char *rootroute)
 *
 *  Analyse (selected) root processor and return block containing peeked
 *  data, or NULL if errors occurred.  *Psize is set to the actual size
 *  peeked, which may be less than specified because of memory limitations
 *  (in which this will have been diagnosed).
 *
 */

#if __STDC__
static   char    *analyse (link_t linkid, utint_t *psize, char *rootroute)
#else
static  char    *analyse (linkid, psize, rootroute)
link_t  linkid;
utint_t *psize;
char    *rootroute;
#endif
{
    char    peek = 1;                   /* code to do peek with */
    tint_t  addr = 0x80000000;          /* start address - 0x8000 if T2xx! */
    char    *peeked = NULL;             /* peeked data */
    size_t  size;                       /* real size */
    tint_t  value;                      /* value returned */
    tint_t  i;                          /* loop var */
    int     res;

    /* Be verbose if requested.                                         */
    if (verbose)
        fprintf (stderr, "analysing root transputer\n");

    /* Try to analyse the root processor.                               */
    if ((res = AnalyseLink (linkid)) < 0)
        error ("error analysing transputer: %s", lerror(res));
    else {

        /* Set up route to selected root processor.                     */
        if (route (linkid, rootroute)) {

            /* More info.                                               */
            if (verbose)
                fprintf (stderr, "peeking 0x%lx (%ldK) bytes\n",
                                (long) *psize, ((long) *psize + 512) / 1024);

            /* Allocate buffer to hold the info.                        */
            size = *psize;
            if (size != *psize)
                size = 0xFFFC;
            while (! (peeked = malloc ((int) size)) && size > 1024)
                size -= 1024;
            if (! peeked)
                error ("can't even allocate 1K for peek buffer");
            else {

                /* Diagnose if less than requested.                     */
                if (size != *psize)
                    error ("can only peek 0x%lx (%ldK) bytes",
                                        (long) size, (long) (size+512)/1024);

                /* Peek memory starting at mostneg int.                 */
                for (i=0; i<size; i+=4, addr+=4) {
                    int x1, x2, x3;
                    x1 = WriteLink (linkid, &peek,  1, 5);
                    x2 = WriteLink (linkid, &addr,  4, 5);
                    x3 = ReadLink  (linkid, &value, 4, 5);
                    if (x1 < 0 || x2 < 0 || x3 < 0) {
                        error ("error peeking address %x: %s", addr,
                                    lerror (x1<0 ? x1 : x2<0 ? x2 : x3));
                        break;
                    }
                    else if (x1 != 1 || x2 != 4 || x3 != 4) {
                        error ("timeout peeking address %x (%d/%d/%d)",
                                                        addr, x1, x2, x3);
                        break;
                    }

                    /* Store peeked value in block.                     */
                     * (tint_t *) (&peeked[i]) = value;

                    /* In verbose mode, report every kilobyte peeked.   */
                    if (verbose && isatty (fileno (stderr)) && (i+4)%1024 == 0)
                        fprintf (stderr, "%4dKb peeked\r", (i+4)/1024);
                }

                /* In verbose mode, print final size peeked.            */
                if (size >= 512 && verbose && isatty (fileno (stderr)))
                    fprintf (stderr, "%4dKb peeked\n", (size+512)/1024);
            }
        }
    }

    /* Set real size peeked and return peeked data, or NULL.            */
    *psize = size;
    return peeked;
}


/*
 *
 *      bool_t boot (link_t linkid, char *prog)
 *
 *  Boot transputer with program with given name.  Returns TRUE if failure.
 *
 */

#if __STDC__
static  bool_t  boot (link_t linkid, char *prog)
#else
static  bool_t  boot (linkid, prog)
link_t  linkid;
char    *prog;
#endif
{
    FILE    *f = NULL;
    char    *tpath = NULL,
            *paths,
            *path,
            *name;
    bool_t  err = FALSE;

    /* Build list of directories to search for executable.  As is usual */
    /* under MS-DOS, the current directory is always searched first.    */
    tpath = getenv ("TPATH");
    if (! tpath)
        tpath = "";
    paths = malloc (strlen(".")+1 + strlen(tpath)+1 + strlen(DEFTPATH) + 1);
    if (! paths) {
        error ("can't allocate memory for executable search path");
        return TRUE;
    }
    strcat (strcat (strcat (strcpy (paths, ".;"), tpath), ";"), DEFTPATH);

    /* Try all dirs.  Try with and without .run extension.              */
    for (path=strtok(paths,";"); path && !f; path=strtok(NULL,";")) {
        name = malloc (strlen(path)+ 1 + strlen(prog) + strlen (DEFEXT) + 1);
        if (! name) {
            error ("can't allocate memory for executable search path");
            return TRUE;
        }
        if (path[strlen(path)-1] == DIRSEPC)
            path[strlen(path)-1] = '\0';
        if (path[0] && ! (path==paths && path[0]=='.' && path[1]=='\0'))
            sprintf (name, "%s%c%s", path, DIRSEPC, prog);
         else
            strcpy (name, prog);
        strcat (name, DEFEXT);
        if (! (f = fopen (name, "rb"))) {
            name[strlen(name)-strlen(DEFEXT)] = '\0';
            if (! (f = fopen (name, "rb")))
                free (name);
        }
    }

    /* List of dirs is no longer needed.                                */
    free (paths);

    /* If still not succesfully opened, something's wrong.              */
    if (! f) {
        error ("can't find %s", prog);
        err = TRUE;
    }

    /* Found program, now copy to link.                                 */
    else {
        long    total = 0;
        long    lastK = 0;

        /* Print name of file opened.                                   */
        if (verbose)
            fprintf (stderr, "booting from %s\n", name);

        /* Read file and copy to link.  In verbose mode, we may want to */
        /* try and keep track of what we're sending and where, so as to */
        /* give an idea of what's happening if/when the boot fails.     */
        while (! feof (f) && ! ferror (f) && ! err) {
            char    buf[100];
            int     x;
            x = fread (buf, 1, sizeof(buf), f);
            if (ferror (f)) {
                error ("error reading %s", name);
                err = TRUE;
            }
            else if (x > 0) {
                int y = WriteLink (linkid, buf, x, 10);
                total += y;
                if (y != x) {
                    if (y < 0)
                        error ("error booting %s: %s", name, lerror(y));
                    else
                        error ("timeout booting %s after 0x%lx bytes", name, total);
                    err = TRUE;
                }
            }
            /* In verbose mode, report every kilobyte booted.           */
            if (verbose && isatty (fileno (stderr)) && total/10240 > lastK/10) {
                lastK = total/1024;
                fprintf (stderr, "%4dKb booted\r", lastK);
            }
        }

        /* In verbose mode, print total size booted.                    */
        if (verbose && isatty (fileno (stderr)))
            fprintf (stderr, "%4ldKb booted\n", (total+512)/1024);

        /* Close file and free filename.                                */
        (void) fclose (f);
        free (name);
    }

     /* Return status.                                                   */
    return err;
}


/*
 *
 *      void breakhandler (...)
 *
 *  Break handler is called for interrupt signals.
 *
 */

static  void    breakhandler ()
{
    /* No interrupts until program termination.                         */
    (void) signal (SIGINT, SIG_IGN);

    /* Diagnostic.                                                      */
    if (verbose)
        error ("interrupted by user");
    status = -3;

    /* Longjmp back to exit part of main program.                       */
    longjmp (exitenv, 1);
}


/*
 *
 *      int main (int argc, char **argv)
 *
 *  Parse command line and go link.  Return exit status.
 *
 */

#if __STDC__
int main (int argc, char **argv)
#else
int main (argc, argv)
int     argc;
char    **argv;
#endif
{
    extern  char    *version;           /* version info from 'version' */
    extern  int     getopt();           /* option parser */
    extern  char    *optarg;            /* option-argument, if any */
    extern  int     optind;             /* next argv entry to check */
    bool_t  errflg = FALSE;             /* command-line errors */
    char    options[64];                /* options for getopt */
    int     i;                          /* loop var */
    char    *progname;                  /* program name */
    char    *peeked = NULL;             /* block containing peeked memory */
    utint_t peeksz = 0;                 /* size of block of memory to peek */
#   define  DEFPEEKSZ   (8*1024)        /* default: 8K */
    bool_t  termerr = FALSE;            /* terminate if ErrorFlag is set */
    bool_t  reset = TRUE;               /* reset transputer */
    bool_t  serve = TRUE;               /* serve link */
    bool_t  reporttimes = FALSE;        /* report transputer system and user time */
    char    *link = NULL;               /* name of link to work with */
    char    *rootroute = NULL;          /* route to root, if set */
    bool_t  waitforlink = TRUE;         /* default is to wait for link */
    bool_t  sigign = FALSE;             /* signals ignored? */
    int     res;

    #ifdef CS_LIB
    /* pserver is being compiled to be used as an interface between */
    /* a PPC client application running on a remote host and a      */
    /* PPC parallel program running on the transputer network.      */
    init_for_cs_lib();
    #endif

    /* Determine program name.  If iserver, disable pserver extensions. */
     progname = strrchr(argv[0],DIRSEPC) ? strrchr(argv[0],DIRSEPC)+1 : argv[0];
    pserverext = strcmp (progname, "iserver");

    /* MS-DOS doesn't allow stderr redirection, so make stderr=stdout.  */
#if OSID == OS_DOS
    *stderr = *stdout;
#endif

    /* Some linkIO modules want to be able to look at arguments.        */
#if BOARDID == BOARD_MCP1000
    argcv (argc, argv);
#endif

    /* Build list of options for this host/interface.                   */
    options[0] = '\0';
    strcat (options, "a:c:ei:l:prR:s:T:tVvxz?");
#if OSID == OS_USV386
    strcat (options, "d");
#endif
#if BOARDID == BOARD_MCP1000
    strcat (options, "NP:S");
#endif

    /* Parse command line options.                                      */
    while ((i = getopt (argc, argv, options)) != -1)
        switch (i) {

        case 'a':                       /* analyse and peek */
            peeksz = strtol (optarg, (char **) NULL, 16);
            if (peeksz == 0)
                peeksz = DEFPEEKSZ;
            break;

        case 'c':                       /* 'channel' to root */
            if (strspn (optarg, "0123") != strlen (optarg)) {
                error ("illegal link specifier %c in route %s", optarg[strspn(optarg,"0123")], optarg);
                errflg = TRUE;
            }
            else
                rootroute = optarg;
            break;

        case 'e':                       /* terminate if ErrorFlag set */
            termerr = !termerr;
            break;

        case 'i':                       /* software interrupt:  req==int */
#if OSID == OS_DOS
#   if  PSERVER_EXTENSION
            if (! strchr (optarg, '=')) {
                error ("illegal software interrupt specifier (missing '=')");
                errflg = TRUE;
            }
            else {
                int server, intno;
                server = strtol (strtok (optarg, "="), (char **) NULL, 10);
                intno  = strtol (strtok (NULL,   ""),  (char **) NULL, 10);
                addserver (server, intno);
            }
#   else
            error ("pserver extensions disabled");
            errflg = TRUE;
#   endif   /* PSERVER_EXTENSION */
#else
            error ("extra servers through interrupt only supported under MS-DOS");
            errflg = TRUE;
#endif   /* OS_ID == OS_DOS */
            break;

        case 'l':                       /* link to use */
            link = optarg;
            break;

        case 'p':                       /* toggle pserver extensions */
#if PSERVER_EXTENSION
            pserverext = !pserverext;
#else
            error ("pserver extensions disabled");
            errflg = TRUE;
#endif
            break;

        case 'r':                       /* reset transputer */
            reset = !reset;
            break;

        case 'R':                       /* resetmode (undocumented) */
            resetmode = optarg;
            break;

        case 's':                       /* smart wait delay */
#if SMART_WAIT
            smartwaitsecs = strtol (optarg, (char **) NULL, 10);
#else
            error ("smart waiting disabled");
            errflg = TRUE;
#endif
            break;

        case 'T':                       /* max secs inactive link */
#if INACTIVE_MAX
            inactivesecs = strtol (optarg, (char **) NULL, 10);
#else
            error ("max inactive link check disabled");
            errflg = TRUE;
#endif
            break;

        case 't':                       /* report transputer times */
            reporttimes = !reporttimes;
            break;

        case 'V':                       /* version info */
            fprintf (stderr, "%s: %s\n", argv[0], version);
            if (argc == 2)
                exit (1);
            break;

        case 'v':                       /* verbose mode */
            verbose = !verbose;
            break;

        case 'x':                       /* do not serve */
            serve = !serve;
            break;

        case 'z':                       /* debug mode */
            debug = !debug;
            break;

        case '?':                       /* help */
            errflg = TRUE;
             break;

        /* Options for USV/386/B004 interface:                      */
        case 'd':                       /* wait for free transputer */
            waitforlink = ! waitforlink;
            break;

        /* Options for MCP1000 interface (used by Transtech sw):    */
        case 'P':                       /* select link */
            link = optarg;
            break;
        case 'N': case 'S':             /* ignored */
            break;
        }

    /* Print usage if pserver invoked without arguments or any errors   */
    /* parsing options.                                                 */
    if (argc == 1 || errflg)
        usage (progname);

    /* Remaining arguments are passed to transputer as command line     */
    /* argsuments upon request.                                         */
    setargs (argc, argv, optind);

    /* Set up for longjmp out of breakhandler to terminate.             */
    if (setjmp (exitenv) == 0) {

        /* Catch interrupt unless it's currently ignored (background).  */
        sigign = signal (SIGINT, SIG_IGN) == SIG_IGN;
        if (! sigign)
            signal (SIGINT, breakhandler);

        /* Select wait/fail mode for PACT UNIX/386 device driver.       */
#if OSID == OS_USV386 && !defined(NEXT)
        WaitLink (waitforlink);
#endif

        /* Get default link from TRANSPUTER environment variable.       */
        if (! link)
            link = getenv ("TRANSPUTER");

        /* Open link to transputer network.                             */
        if ((linkid = OpenLink (link)) < 0) {
            error ("error opening link: %s", lerror (linkid));
            status = -1;
        }
        else {

            /* Analyse and peek block of memory if specified, then give */
            /* it to the server to give away upon request.              */
            if (peeksz)
                peeked = analyse (linkid, &peeksz, rootroute);
            setcore (peeked, peeksz);

            /* Reset transputer.                                        */
            if (reset) {
                if (verbose)
                    fprintf (stderr, "transputer reset\n");
                if ((res = ResetLink (linkid)) < 0)
                    error ("error resetting transputer: %s", lerror(res));
            }

            /* Set up route to root processor ONLY if just reset.       */
            if (! reset || route (linkid, rootroute)) {

                /* Boot program, if any, and serve.                     */
                 if (optind < argc)
                    if (boot (linkid, argv[optind])) {
                        status = -2;
                        serve = FALSE;
                    }
                if (serve) {
                    if (verbose)
                        fprintf (stderr, "server started\n");
                    status = server (linkid, termerr, verbose, debug);
                }
            }
        }
    }

    /* If open, close link to transputer without interruption.          */
    if (linkid >= 0) {
        (void) signal (SIGINT, SIG_IGN);
        if ((res = CloseLink (linkid)) < 0) {
            error ("error closing link: %s", lerror(res));
            if (status == 0)
                status = -1;
        }
        linkid = -1;
    }

    /* Allow interrupts to proceed as before.                           */
    if (! sigign)
        (void) signal (SIGINT, SIG_DFL);

    /* If selected, report transputer user and system time, a la        */
    /* UNIX time(1).                                                    */
    if (reporttimes) {
        if (ttimes.user == -1 && ttimes.sys == -1 && ttimes.real == -1)
            fprintf (stderr, "\n        ?.? real         ?.? user         ?.? sys\n");
        else
            fprintf (stderr, "\n%11.1f real %11.1f user %11.1f sys\n",
                        (double) ttimes.real / 1000,
                        (double) ttimes.user / 1000,
                        (double) ttimes.sys  / 1000);
    }

    /* Return exit status returned by transputer program (or negative   */
    /* number if server failed to open/boot/close link, or was          */
    /* interrupted).                                                    */
    return status;
}

