/*
 *
 *  @(#)mcp1000.c   1.2 93/08/01    (c) PACT
 *
 *  PSERVER - PACT host server
 *
 *  Copyright (c) 1991, 1992 PACT
 *  All Rights Reserved
 *
 *  MCP1000 interface module.
 *
 */

#include    <stdio.h>
#include    <fcntl.h>
#include    <string.h>
#include    <sys/time.h>
#include    <niche/nadriver.h>
#include    "linkio.h"


/*
 *
 *      Boolean type.
 *
 */

typedef int bool_t;
#undef  FALSE
#define FALSE   0
#undef  TRUE
#define TRUE    1


/*
 *
 *  Currently active link and its name.
 *
 */

#define FREELINK    (-1)
static  link_t      activelink = FREELINK;
static  char        linkname[80] = "";


/*
 *
 *  Remember current timeout, so it can be changed when necessary.
 *
 */

static int      curtimout = -1;
struct timeval  timelimit;


/*
 *
 *  Networking (niche) support.
 *
 */

#include "nts_lib.h"

NTS_params nts_values;
int     Argc = 0;
char    **Argv = NULL;


/*
 *
 *      void argcv (int argc, char **argv)
 *
 *  Called by main prior to parsing command line arguments, to allow
 *  linkIO module to access command line arguments for any special
 *  purposes.
 *
 */

void    argcv (argc, argv)
int     argc;
char    **argv;
{
    Argc = argc;
    Argv = argv;
}


/*
 *
 *      bool_t parsename (char *name)
 *
 *  Parse link name and return FALSE if name specifies a device on this machine
 *  ("/xxx") or TRUE if it specifies a unit on some machine (incl. default).
 *
 */

static  bool_t  parsename (name)
char    *name;
{
    char    *colon;

    /* No name or empty name specifies any link (unit 0).               */
    if (! name || name[0] == '\0')
        return FALSE;

    /* Name that starts with '/' specifies explicit link name on this   */
    /* machine.                                                         */
    if (name[0] == '/')
        return TRUE;

    /* Colon (':') with number specifies unit number.                   */
    colon = strchr (name, ':');
    if (colon) {
        *colon = '\0';
        nts_values.unit = atoi (colon+1);
    }

    /* Everything preceding colon (if any) specifies host name.         */
    strcpy (nts_values.host, name);

    /* Return FALSE to indicate a unit specification.                   */
    return FALSE;
}


/*
 *
 *      link_t  OpenLink (char *link)
 *
 *  Open link.  If link is NULL or empty, any free link can be used.
 *  Otherwise, link should specify a device on this machine ("/") or
 *  a unit on some host in the network ("[host:]unit") (where unit is
 *   a decimal number).  Returns a negative number if an error occurs,
 *  and a positive linkid if the link was successfully opened.
 *
 */

#if __STDC__
link_t  OpenLink (char *link)
#else
link_t  OpenLink (link)
char    *link;
#endif
{
    int     i;

    nts_values.host[0] = '\0';
    strcpy (nts_values.type, "nap");
    nts_values.strategy = NTS_TERMINAL;
    nts_values.unit = -1;

    /* Link already open?                                               */
    if (activelink != FREELINK)
        if (!link || link[0] == '\0' ||
            !strncmp (link, linkname, sizeof(linkname)-1))
            return activelink;
        else
            return ER_LINK_CANT;

    /* Simple device open if name is device name.                       */
    if (parsename (link))
        activelink = open (link, O_RDWR);

#ifdef  NTS_LIB
    /* Open explicit unit on some machine.  Wants to look at command    */
    /* line arguments.                                                  */
    else if (nts_values.unit >= 0)
        (void) nts_open_link (Argc, Argv, &nts_values, &activelink);

    /* If no unit was specified, try to find a free transputer.         */
    else {
        nts_values.unit = 0;
        while (nts_values.unit < 16 && activelink < 0) {
            if (nts_open_link (Argc, Argv, &nts_values, &activelink))
                nts_values.unit++;
        }
        if (activelink >= 0)
            fprintf (stderr, "opened site %s:%s%d\n", nts_values.host,
                                        nts_values.type, nts_values.unit);
    }
#endif

    /* Return error if no link could be opened.                         */
    if (activelink < 0) {
        activelink = FREELINK;
        return ER_LINK_CANT;
    }

    /* Be sure to remember link name if succesful.                      */
    if (link) {
        strncpy (linkname, link, sizeof(linkname));
        linkname[sizeof(linkname)-1] = '\0';
    }

    /* Return link ID.                                                  */
    return activelink;
}


/*
 *
 *      int CloseLink (link_t linkid)
 *
 *  Close link.  Must be open to be closed.  If any error occurs (e.g. link
 *  is not open), a negative number is returned.  Otherwise, 1 is returned.
 *
 */

#if __STDC__
int     CloseLink (link_t linkid)
#else
int     CloseLink (linkid)
link_t  linkid;
#endif
{
    /* Check if link is actually open.                                  */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Close device.                                                    */
    (void) close (activelink);

    /* Free link.                                                       */
    activelink = FREELINK;

    /* Return 1 to indicate success.                                    */
    return 1;
}


/*
 *
 *      int ReadLink (link_t linkid, char *buf, unsigned len, int timout)
 *
 *  Read len bytes from specified link into buffer.  Returns number of
 *  bytes read (before timeout), or a negative number if an error occurs
 *  (wrong link or 0-size transfer).  The timout parameter specifies the
 *  number of tenths of seconds the entire transfer is allowed to take.
 *
 */

#if __STDC__
int     ReadLink (link_t linkid, char *buf, unsigned len, int timout)
#else
int     ReadLink (linkid, buf, len, timout)
link_t      linkid;
char        *buf;
unsigned    len;
int         timout;
#endif
{
    fd_set  fds;

    /* If linkid is not the active link, fail.                          */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Zero-size transfers can't be done.                               */
    if (len < 1)
        return ER_LINK_CANT;

    /* Change timelimit structure if timeout changed.                   */
    if (timout != 0 && timout != curtimout) {
        timelimit.tv_sec = timout / 10;
         timelimit.tv_usec = 100000 * (timout % 10);
        curtimout = timout;
    }

    /* See if link can be read from, then go do the read.               */
    FD_ZERO (&fds);
    FD_SET (activelink, &fds);      /* was: fdmask = 1 << activelink; */
    switch (select (32, &fds, NULL, NULL, &timelimit)) {
    case 0:  return 0;
    case 1:  return read (activelink, buf, len);
    default: return ER_LINK_SOFT;
    }
}


/*
 *
 *      int WriteLink (link_t linkid, char *buf, unsigned len, int timout)
 *
 *  Write len bytes from specified link into buffer.  Returns number of
 *  bytes written (before timeout), or a negative number if an error occurs
 *  (wrong link or 0-size transfer).  The timout parameter specifies the
 *  number of tenths of seconds the entire transfer is allowed to take.
 *
 */

#if __STDC__
int     WriteLink (link_t linkid, char *buf, unsigned len, int timout)
#else
int     WriteLink (linkid, buf, len, timout)
link_t      linkid;
char        *buf;
unsigned    len;
int         timout;
#endif
{
    fd_set  fds;

    /* If linkid is not the active link, fail.                          */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Zero-size transfers can't be done.                               */
    if (len < 1)
        return ER_LINK_CANT;

    /* Change timelimit structure if timeout changed.                   */
    if (timout != 0 && timout != curtimout) {
        timelimit.tv_sec = timout / 10;
        timelimit.tv_usec = 100000 * (timout % 10);
        curtimout = timout;
    }

    /* See if link can be written to, then go do the write.             */
    FD_ZERO (&fds);
    FD_SET (activelink, &fds);      /* was: fdmask = 1 << activelink; */
    switch (select (32, NULL, &fds, NULL, &timelimit)) {
    case 0:  return 0;
    case 1:  return write (activelink, buf, len);
    default: return ER_LINK_SOFT;
    }
}


/*
 *
 *       int ResetLink (link_t linkid)
 *
 *  Reset transputer.  Returns 1 if successful, and a negative number to
 *  indicate an error.
 *
 */

#if __STDC__
int     ResetLink (link_t linkid)
#else
int     ResetLink (linkid)
link_t  linkid;
#endif
{
    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Tell driver to reset the transputer.  Return error or 1.         */
    return (ioctl (activelink, SITE_RESET) == -1 ||
            ioctl (activelink, DISABLE_ERRORS) == -1) ? ER_LINK_SOFT : 1;
}


/*
 *
 *      int AnalyseLink (link_t linkid)
 *
 *  Analyse transputer.  Returns 1 if successful, and a negative number to
 *  indicate an error.
 *
 */

#if __STDC__
int     AnalyseLink (link_t linkid)
#else
int     AnalyseLink (linkid)
link_t  linkid;
#endif
{
    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Tell driver to reset the transputer.  Return error or 1.         */
    return (ioctl (activelink, SITE_ANALYSE) == -1 ||
            ioctl (activelink, DISABLE_ERRORS) == -1) ? ER_LINK_SOFT : 1;
}


/*
 *
 *      int TestError (link_t linkid)
 *
 *  Returns state of ErrorFlag.
 *
 */

#if __STDC__
int     TestError (link_t linkid)
#else
int     TestError (linkid)
link_t  linkid;
#endif
{
    int status;

    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Ask driver for error status.  Return error or status.            */
    return (ioctl (activelink, ENABLE_ERRORS) == -1 ||
            ioctl (activelink, SITE_ERROR, &status) == -1) ? ER_LINK_SOFT
                                                           : status;
}


/*
 *
 *      int TestRead (link_t linkid)
 *
 *  Returns TRUE if transputer is ready to receive data.
 *
 */

#if __STDC__
int     TestRead (link_t linkid)
#else
int     TestRead (linkid)
link_t  linkid;
#endif
{
    int status;

    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Ask driver for read status.  Return error or status.             */
#if 0   /* WHY? */
    return ioctl (activelink, LINK_INPUT_PENDING, &status) == -1 ? ER_LINK_SOFT
                                                                 : status;
#else
    return 1;
#endif
}


/*
 *
 *      int TestWrite (link_t linkid)
 *
 *  Returns TRUE if transputer is ready to receive data.
 *
 */

#if __STDC__
int     TestWrite (link_t linkid)
#else
int     TestWrite (linkid)
link_t  linkid;
#endif
{
    /* If specified link is not active return error status.             */
    if (linkid != activelink)
        return ER_LINK_BAD;

    /* Ask driver for write status.  Return error or status.            */
#if 0
    /* XXX */
#else
     return 1;
#endif
}

