/*
 *
 *  @(#)msdos.c 1.12    92/12/20    (c) PACT
 *
 *  PSERVER - PACT host server
 *
 *  Copyright (c) 1991, 1992 PACT
 *  All Rights Reserved
 *
 *  MS-DOS specifics.
 *
 */

#include    "pserver.h"                 /* general defs */
#include    "error.h"                   /* error manager */
#include    "msdos.h"                   /* own defs and decls */
#include    "server.h"                  /* server defs */


#if OSID==OS_DOS

#include    <dos.h>


/*
 *
 *      int msdos_sendblock (byte_t *req, byte_t *ans)
 *
 *  Send block of data to some address in PC's memory.  Return result.
 *  Answer vector may overlap request.
 *
 *  Args:   INT32   address             Rets:   INT16 len
 *          VEC     data
 *
 */

static  int msdos_sendblock (byte_t *req, byte_t *ans)
{
    unsigned long   addr;
    char            far* ptr;
    int             len;
    byte_t          *a;

    /* Get address and convert to far pointer.  Get length.             */
    addr = INT32 (req);  req += 4;
    ptr = (char far*) addr;
    len = VLEN (req);  req += VLENSZ;

    /* Start of data in answer.                                         */
    a = VDAT (ans);

    /* Return length.                                                   */
    SETINT16 (a, len);

    /* Set length of answer.                                            */
    SETVLEN (ans, 2);

    /* Copy data from packet to specified address.                      */
    while (len-- > 0)
        *ptr++ = *req++;

    /* Return success.                                                  */
    return SP_SUCCESS;
}


/*
 *
 *      int msdos_recvblock (byte_t *req, byte_t *ans)
 *
 *  Receive block of data from some address in PC's memory.  Return result.
 *  Answer vector may overlap request.
 *
 *  Args:   INT32   address             Rets:   VEC     data
 *          INT16   len
 *
 */

static  int msdos_recvblock (byte_t *req, byte_t *ans)
{
    unsigned long   addr;
    char            far* ptr;
    int             len;
    byte_t          *a;

    /* Get address and convert to far pointer.  Get length.             */
    addr = INT32 (req);  req += 4;
    ptr = (char far*) addr;
    len = INT16 (req);

    /* Check max size.                                                  */
    if (len > MAXPACKSZ-VLENSZ-1-VLENSZ)
        len = MAXPACKSZ-VLENSZ-1-VLENSZ;

    /* Start of data in answer.                                         */
    a = VDAT (ans);

    /* Store vector length in answer block.                             */
    SETVLEN (a, len);  a += VLENSZ;

    /* Set length of answer.                                            */
    SETVLEN (ans, VLENSZ + len);

    /* Copy data from specified address to packet.                      */
    while (len-- > 0)
        *a++ = *ptr++;

    /* Return success.                                                  */
    return SP_SUCCESS;
}


/*
 *
 *      int msdos_int86x (byte_t *req, byte_t *ans)
 *
 *  Call interrupt.  Return result.
 *  Answer vector may overlap request.
 *
 *  Args:   INT16   interrupt           Rets:   BYTE    cflag
 *          INT32   ax                          INT32   ax
 *          INT32   bx                          INT32   bx
 *          INT32   cx                          INT32   cx
 *          INT32   dx                          INT32   dx
 *          INT32   di                          INT32   di
 *          INT32   si                          INT32   si
 *          INT32   cs                          INT32   cs
 *          INT32   ds                          INT32   ds
 *          INT32   es                          INT32   es
 *          INT32   ss                          INT32   ss
 *
 *  Note: The above INT32's are interpreted as INT16 data + INT16 padding.
 *
 */

static  int msdos_int86x (byte_t *req, byte_t *ans)
{
    int             intno;
    struct SREGS    Segregs,
                    *segregs = &Segregs;
    union REGS      Inregs,
                    *inregs = &Inregs,
                    Outregs,
                    *outregs = &Outregs;
    byte_t          *a;

    /* Get interrupt and registers.                                     */
    intno        = INT16 (req);  req += 2;
    inregs->x.ax = INT16 (req);  req += 4;
    inregs->x.bx = INT16 (req);  req += 4;
    inregs->x.cx = INT16 (req);  req += 4;
    inregs->x.dx = INT16 (req);  req += 4;
    inregs->x.di = INT16 (req);  req += 4;
    inregs->x.si = INT16 (req);  req += 4;
    segregs->cs  = INT16 (req);  req += 4;
    segregs->ds  = INT16 (req);  req += 4;
    segregs->es  = INT16 (req);  req += 4;
    segregs->ss  = INT16 (req);

    /* Do interrupt.                                                    */
    (void) int86x (intno, inregs, outregs, segregs);

    /* Start of data in answer.                                         */
    a = VDAT (ans);

    /* Store registers.                                                 */
    memset (a, 0, 10*4+2);
    *a++ = outregs->x.cflag;
    SETINT16 (a, outregs->x.ax);  a += 4;
    SETINT16 (a, outregs->x.bx);  a += 4;
    SETINT16 (a, outregs->x.cx);  a += 4;
    SETINT16 (a, outregs->x.dx);  a += 4;
    SETINT16 (a, outregs->x.di);  a += 4;
    SETINT16 (a, outregs->x.si);  a += 4;
    SETINT16 (a, segregs->cs);    a += 4;
    SETINT16 (a, segregs->ds);    a += 4;
    SETINT16 (a, segregs->es);    a += 4;
    SETINT16 (a, segregs->ss);

    /* Set length of answer.                                            */
    SETVLEN (ans, VLENSZ + 10*4+2);

    /* Return success.                                                  */
    return SP_SUCCESS;
}


/*
 *
 *      int msdos_segread (byte_t *req, byte_t *ans)
 *
 *  Read segment registers.  Return result.
 *  Answer vector may overlap request.
 *
 *  Args:   -                           Rets:   INT32   cs
 *                                              INT32   ds
 *                                              INT32   es
 *                                              INT32   ss
 *
 *  Note: The above INT32's are interpreted as INT16 data + INT16 padding.
 *
 */

static  int msdos_segread (byte_t *req, byte_t *ans)
{
    struct SREGS    Segregs,
                    *segregs = &Segregs;
    byte_t          *a;

    /* Read registers.                                                  */
    segread (segregs);

    /* Start of data in answer.                                         */
    a = VDAT (ans);

    /* Store registers.                                                 */
    memset (a, 0, 16);
    SETINT16 (a, segregs->cs);  a += 4;
    SETINT16 (a, segregs->ds);  a += 4;
    SETINT16 (a, segregs->es);  a += 4;
    SETINT16 (a, segregs->ss);  a += 4;

    /* Set length of answer.                                            */
    SETVLEN (ans, VLENSZ + 16);

    /* Return success.                                                  */
    return SP_SUCCESS;
}


/*
 *
 *      int msdos_outportb (byte_t *req, byte_t *ans)
 *
 *  Write value (byte) to port.  Return result.
 *  Answer vector may overlap request.
 *
 *  Args:   INT16   port                Rets:   -
 *          BYTE    value
 *
 */

static  int msdos_outportb (byte_t *req, byte_t *ans)
{
    int     port;
    int     value;

    /* Get port and value.                                              */
    port  = INT16 (req);  req += 2;
    value = BYTE  (req);

    /* Write value to port.                                             */
    outp (port, value);

    /* Zero-length answer.                                              */
    SETVLEN (ans, 0);

    /* Return success.                                                  */
    return SP_SUCCESS;
}


/*
 *
 *       int msdos_inportb (byte_t *req, byte_t *ans)
 *
 *  Read value (byte) from port.  Return result.
 *  Answer vector may overlap request.
 *
 *  Args:   INT16   port                Rets:   BYTE    value
 *
 */

static  int msdos_inportb (byte_t *req, byte_t *ans)
{
    int     port;
    int     value;

    /* Get port.                                                        */
    port  = INT16 (req);

    /* Read port.                                                       */
    value = inp (port);

    /* Store value read in answer.                                      */
    ans[VLENSZ] = value;

    /* Answer is one byte long.                                         */
    SETVLEN (ans, 1);

    /* Return success.                                                  */
    return SP_SUCCESS;
}


#if     PSERVER_EXTENSION
/*
 *
 *      Data describing all registered extra servers, the requests they
 *      serve and the software interrupt to invoke them.
 *
 */

#include    <dos.h>

#define NOFSERVERS      16

static  struct server {
    int     id;
    int     intno;
} servers[NOFSERVERS] = { 0 };


/*
 *
 *      bool_t addserver (int serverid, int intno)
 *
 *  Registers additional server.  Returns TRUE if error, FALSE if OK.
 *
 */

bool_t  addserver (int serverid, int intno)
{
    int i;

    /* Server id 0 is special, and translated to -1 internally.         */
    if (serverid == 0)
        serverid = -1;

    /* First make sure request is unique.                               */
     for (i=0; i<NOFSERVERS; i++)
        if (servers[i].id == serverid)
            return TRUE;

    /* Find free slot in servers[] table.                               */
    for (i=0; i<NOFSERVERS; i++)
        if (servers[i].id == 0)
            break;
    if (i == NOFSERVERS)
        return TRUE;

    /* Add server to table and return OK.                               */
    servers[i].id    = serverid;
    servers[i].intno = intno;
    return FALSE;
}


/*
 *
 *      int msdos_server (byte_t *req, byte_t *ans)
 *
 *  Calls extra server thru software interrupt, if registered.  This
 *  provides an easy way to add special servers without having to change
 *  and recompile the pserver.  A special server can set up a software
 *  interrupt and invoke the pserver with the -i option to register a
 *  server.  Transputer runtime functions provide for calling an extra
 *  server and returning from them.
 *
 *  Args:   INT16   serverid            Rets:   VEC whatever
 *          VEC     whatever
 *
 */

static  int msdos_server (byte_t *req, byte_t *ans)
{
    struct REGPACK  regs;
    int             i;
    int             serverid;

    /* Get id of server to invoke.  Serverid 0 is translated to -1.     */
    serverid = INT16 (req);  req += 2;
    if (serverid == 0)
        serverid = -1;

    /* See if such a server is registered.                              */
    for (i=0; i<NOFSERVERS; i++)
        if (servers[i].id == serverid)
            break;

    /* Return error if not.                                             */
    if (i == NOFSERVERS) {
        error ("server %d not registered", serverid);
        return SP_ERROR;
    }

    /* Invoke server thru interrupt.  Server gets vector and may        */
    /* return vector.  Passes length of request and receives length of  */
    /* reply in ax.                                                     */
    regs.r_ax = VLEN (req);
    regs.r_es = FP_SEG (VDAT (req));
    regs.r_bx = FP_OFF (VDAT (req));
    regs.r_si = FP_SEG (VDAT (VDAT (ans)));
    regs.r_di = FP_OFF (VDAT (VDAT (ans)));
    intr (servers[i].intno, &regs);

     /* We return data as returned by alternate server, preceded by      */
    /* 2-byte length.                                                   */
    if (regs.r_ax < 0 || regs.r_ax > MAXPACKSZ-VLENSZ-1-VLENSZ-VLENSZ) {
        error ("alternate server %d returned packet that is too large", serverid);
        regs.r_ax = regs.r_ax<0 ? 0 : MAXPACKSZ-VLENSZ-1-VLENSZ-VLENSZ;
    }
    SETVLEN (VDAT (ans), regs.r_ax);

    /* Now set length of block we'll be returning (len+data).           */
    SETVLEN (ans, regs.r_ax + VLENSZ);

    /* Return success if/when extra server returns.                     */
    return SP_SUCCESS;
}
#endif  /* PSERVER_EXTENSION */

#endif  /* DOS */


/*
 *
 *      int msdos_msdos ()
 *
 *  Perform MS-DOS specific function.
 *
 *  Args:   BYTE    code                Rets:   RAW     data
 *          RAW     data
 *
 */

static  char    msdosbuf[MAXPACKSZ];

SERVER_FN (msdos_msdos, 2, 1)

#if OSID==OS_DOS
    /* Return buffer with data.                                         */
    VRET(0) = msdosbuf;

    /* Provide service.  Input from request packet, output to buffer.   */
    switch (BARG(0)) {
    case 0:     return msdos_sendblock (VARG(1), VRET(0));
    case 1:     return msdos_recvblock (VARG(1), VRET(0));
    case 2:     return msdos_int86x    (VARG(1), VRET(0));
    case 3:     return msdos_segread   (VARG(1), VRET(0));
    case 4:     return msdos_outportb  (VARG(1), VRET(0));
    case 5:     return msdos_inportb   (VARG(1), VRET(0));
#if     PSERVER_EXTENSION
    case 167:   if (pserverext)
                    return msdos_server (VARG(1), VRET(0));
                /* fall thru if no extensions */
#endif
    default:    error ("MS-DOS request %d unknown", BARG(0));
                return SP_UNIMPLEMENTED;
    }
#else
    error ("MS-DOS request %d not supported", BARG(0));
    return SP_UNIMPLEMENTED;
#endif
}

