/*
 *
 *  @(#)server.c    1.24    93/09/10    (c) PACT
 *
 *  PSERVER - PACT host server
 *
 *  Copyright (c) 1991-1993 PACT
 *  All Rights Reserved
 *
 *  Main server loop.
 *
 */

#include    "pserver.h"                 /* general defs */
#include    "debug.h"                   /* debug support */
#include    "error.h"                   /* error manager */
#include    "file.h"                    /* file support */
#include    "host.h"                    /* host support */
#include    "linkio.h"                  /* link interface definition */
#include    "msdos.h"                   /* MS-DOS support */
#include    "server.h"                  /* own defs and decls */

#include    <time.h>


/*
 *
 *      USE_TESTREAD
 *
 *  Define this if TestRead() is to return TRUE before ReadLink() is called.
 *  On some platforms, with some linkIO modules and/or device drivers, this
 *  is essential (Parsytec!), while on others the TestRead() function isn't
 *  even implemented fully.  According to linkIO specs, just calling
 *  ReadLink() to wait for and accept data from the link should always work.
 *  Use TestRead() when opssible, though, as it may be faster.
 *
 */

#if (defined(PARSYTEC) || BOARDID==BOARD_PARSY_SC || OSID==OS_USV386) && \
    !defined(LINUX) && !defined(NEXT)
#   define  USE_TESTREAD
#else
#   undef   USE_TESTREAD
#endif


/*
 *
 *      sbuffer [MAXPACKSZ]
 *
 *  A multi-purpose server buffer, big enough to hold an entire packet.
 *
 */

byte_t  sbuffer[MAXPACKSZ];


/*
 *
 *      bool_t pserverext
 *
 *  Pserver extensions are enabled if this is set to TRUE.  If enabled
 *  the server version number returned to the transputer program is
 *  100 more than normal so the transputer runtime knows it can use the
 *  extensions.
 *
 */

#if PSERVER_EXTENSION
bool_t  pserverext = TRUE;
#endif


/*
 *
 *      miscellaneous variables used only in this module
 *
 */

        bool_t  terminate = FALSE;      /* terminate server when set to TRUE */
        tint_t  termstat = 0;           /* status returned by exit() */

static  int     nofclargs = 0;
static  char    **clargs = NULL;        /* command line arguments */
static  int     nofclargst = 0;
static  char    **clargst = NULL;

static  char    *core = NULL;           /* core peeked at startup */
static  utint_t coresz = 0;

#if OSID==OS_DOS
union conv  C;                          /* used for conversions */
#endif


/*
 *
 *      int smartwaitsecs
 *
 *  If smart waiting is enabled, the main server loop that waits for
 *  requests from the transputer will insert delays after the transputer
 *  has been quiet for smartwaitsecs seconds.  This is especially nice
 *  on multi-user workstations, so as not to spend 90% of the CPU cycles
 *  waiting for a quiet transputer.
 *
 */

#if SMART_WAIT
int     smartwaitsecs = 5;
#endif


/*
 *
 *      int inactivesecs
 *
 *  If >0, this indicates the number of seconds the transputer link
 *  may be inactive before the server gives up and quits.
 *
 */

#if INACTIVE_MAX
int     inactivesecs = 0;
#endif


/*
 *
 *      struct ttimes ttimes
 *
 *  PACT runtime reports system, user and real time spent on root transputer,
 *  which can be displayed by pserver if so desired.
 *
 */

#if PSERVER_EXTENSION
struct ttimes   ttimes = { -1, -1, -1};
#endif


/*
 *
 *      bool_t request (link_t linkid, byte_t *packet, bool_t debug)
 *
 *  Process request.  Extracts arguments from message, calls server function
 *  with arguments, packs results into packet and returns TRUE on error.
 *
 */

#if __STDC__
static  bool_t  request (link_t linkid, byte_t *packet, bool_t debug)
#else
static  bool_t  request (linkid, packet, debug)
link_t  linkid;
byte_t  *packet;
bool_t  debug;
#endif
{
    static  struct {
        int     tag;
        int     (*fn) ();
        char    *argdesc;               /* arguments passed, excl. tag */
        char    *resdesc;               /* results passed back, excl. res */
    } reqs[] = {
        SP_ZERO,        NULL,           "",         "",
        SP_OPEN,        file_open,      "vbb",      "w",
        SP_CLOSE,       file_close,     "w",        "",
        SP_READ,        file_read,      "wh",       "v",
        SP_WRITE,       file_write,     "wv",       "h",
        SP_GETBLOCK,    file_getblock,  "wh",       "v",
        SP_PUTBLOCK,    file_putblock,  "wv",       "h",
        SP_GETS,        file_gets,      "wh",       "v",
        SP_PUTS,        file_puts,      "wv",       "",
        SP_FLUSH,       file_flush,     "w",        "",
        SP_SEEK,        file_seek,      "www",      "",
        SP_TELL,        file_tell,      "w",        "w",
        SP_EOF,         file_eof,       "w",        "",
        SP_FERROR,      file_ferror,    "w",        "wv",
        SP_REMOVE,      file_remove,    "v",        "",
        SP_RENAME,      file_rename,    "vv",       "",
        SP_ISATTY,      file_isatty,    "w",        "",
        SP_GETKEY,      host_getkey,    "",         "b",
        SP_POLLKEY,     host_pollkey,   "",         "b",
        SP_GETENV,      host_getenv,    "v",        "v",
        SP_TIME,        host_time,      "",         "ww",
        SP_SYSTEM,      host_system,    "v",        "w",
        SP_EXIT,        server_exit,    "w",        "",
        SP_COMMAND,     server_command, "b",        "v",
        SP_CORE,        server_core,    "wh",       "v",
        SP_ID,          server_id,      "",         "bbbb",
        SP_MSDOS,       msdos_msdos,    "br",       "r",
        SP_SUN,         NULL,           "",         "",
        SP_MSC,         NULL,           "",         "",
        SP_VMS,         NULL,           "",         "",
        SP_ALSYS,       NULL,           "",         "",
        SP_KPAR,        NULL,           "",         "",
#if PSERVER_EXTENSION
        SP_ARGC,        server_argc,    "",         "ww",
        SP_ARGVx,       server_argvx,   "w",        "v",
         SP_ABORT,       server_abort,   "w",        "",
        SP_DBG_INIT,    debug_init,     "w",        "",
        SP_DBG_BPT,     debug_bpt,      "",         "",
        SP_TIMES,       server_times,   "www",      "",
        SP_OPEN2,       file_open2,     "vww",      "w",
        SP_ACCESS,      file_access,    "vw",       "",
        SP_FCNTL,       file_fcntl,     "bwwbbwww", "wbbwww",
        SP_STAT,        file_stat,      "vw",       "wwwwwwwwwww",
        SP_GETPID,      host_getpid,    "b",        "w",
        SP_GETUID,      host_getuid,    "b",        "w",
        SP_SETUID,      host_setuid,    "bw",       "",
        SP_KILL,        host_kill,      "ww",       "",
        SP_LINK,        file_link,      "vv",       "",
        SP_UNLINK,      file_unlink,    "v",        "",
        SP_UNAME,       host_uname,     "",         "vvvvv",
        SP_PUTENV,      host_putenv,    "vv",       "",
#endif
    };
    int     msgsize;
    byte_t  *req;
    int     i;
    bool_t  err = FALSE;

    /* Determine message size and start of request data.                */
    msgsize = VLEN (packet);
    req = VDAT (packet);

    /* Find tag in list.  Error if not found.                           */
    for (i=0; i<sizeof(reqs)/sizeof(reqs[0]); i++)
        if (reqs[i].tag == req[0])
            break;
    if (i >= sizeof(reqs)/sizeof(reqs[0])) {
        error ("unrecognized service request: %d", req[0]);
        VDAT(packet)[0] = SP_UNIMPLEMENTED;
        SETVLEN (packet, 2);
    }
    else {
        int     res = SP_UNIMPLEMENTED;
        sarg_t  arguments[6];
        int     nofarguments;
        sarg_t  results[6];
        int     nofresults;
        int     j;
        byte_t  *r;

        /* Determine number of arguments and return values.             */
        nofarguments = strlen (reqs[i].argdesc);
        nofresults   = strlen (reqs[i].resdesc);
        assert (nofarguments < sizeof(arguments)/sizeof(arguments[0]));
        assert (nofresults   < sizeof(results)  /sizeof(results[0]));

        /* Extract and set up arguments to pass to function.            */
        r = &req[1];
        for (j=0; j<nofarguments && !err; j++) {
            int sz;

            /* Determine argument size.                                 */
            switch (reqs[i].argdesc[j]) {
            case 'b':   sz = 1;                         break;
            case 'h':   sz = 2;                         break;
            case 'w':   sz = 4;                         break;
            case 'v':   sz = VLENSZ + VLEN (r);         break;
            case 'r':   sz = msgsize - (r-req);         break;
            default:    abort();
            }

             /* Make sure message contains argument.                     */
            if (r - req > msgsize-sz) {
                error ("request %d: argument %d missing from message", req[0], j);
                err = TRUE;
            }
            else {

                /* Extract argument from message.                       */
                switch (reqs[i].argdesc[j]) {
                case 'b':   arguments[j].b = BYTE(r);   break;
                case 'h':   arguments[j].h = INT16(r);  break;
                case 'w':   arguments[j].w = INT32(r);  break;
                case 'v':   arguments[j].v = r;         break;
                case 'r':   arguments[j].v = r;         break;
                }

                /* Bump message pointer.                                */
                r += sz;
            }
        }
        if (! err) {

            /* Check if entire message has been processed.              */
#if 0       /* Not compatible with iserver, and some transputer         */
            /* executables have been found to send more than their      */
            /* share...                                                 */
            /* REHK 930704 - open et. al. now also has optional extra args */
            if (r-req != msgsize && r-req != msgsize-1 && msgsize > MINPACKSZ-VLENSZ)
                error ("request %d: message %d bytes too long", req[0], msgsize - (r-req));
#endif

            /* Print info in debug mode.                                */
            if (debug) {
                fprintf (stderr, "request %3d;  %d arguments", req[0], nofarguments);
                if (nofarguments > 0) {
                    fprintf (stderr, ":");
                    for (j=0; j<nofarguments; j++) {
                        switch (reqs[i].argdesc[j]) {
                        case 'b':
                            fprintf (stderr, " BYTE %#x", arguments[j].b);
                            break;
                        case 'h':
                            fprintf (stderr, " INT16 %#x", arguments[j].h);
                            break;
                        case 'w':
                            fprintf (stderr, " INT32 %#lx", (long) arguments[j].w);
                            break;
                        case 'v':
                            fprintf (stderr, " VEC %#x", VLEN (arguments[j].v));
                            break;
                        case 'r':
                            fprintf (stderr, " RAW");
                            break;
                        }
                        if (j+1 < nofarguments)
                            fprintf (stderr, ",");
                    }
                }
                fprintf (stderr, "\n");
            }

            /* Call function with argument vector and room for results. */
            if (reqs[i].fn)
                res = (*reqs[i].fn) (nofarguments, arguments, nofresults, results);
            else
                error ("unsupported service request: %d", req[0]);

            /* Print info in debug mode.                                */
            if (debug) {
                fprintf (stderr, "              %d results:   BYTE %#x", res?1:nofresults+1, res);
                if (res == SP_SUCCESS && nofresults > 0) {
                    fprintf (stderr, ",");
                    for (j=0; j<nofresults; j++) {
                        switch (reqs[i].resdesc[j]) {
                        case 'b':
                            fprintf (stderr, " BYTE %#x", results[j].b);
                            break;
                        case 'h':
                            fprintf (stderr, " INT16 %#x", results[j].h);
                            break;
                        case 'w':
                            fprintf (stderr, " INT32 %#lx", (long) results[j].w);
                            break;
                        case 'v':
                            fprintf (stderr, " VEC %#x", VLEN (results[j].v));
                            break;
                        case 'r':
                            fprintf (stderr, " RAW");
                            break;
                        }
                        if (j+1 < nofresults)
                            fprintf (stderr, ",");
                    }
                }
                fprintf (stderr, "\n");
            }

            /* If result byte is 1 (not implemented), print error msg.  */
            if (res == SP_UNIMPLEMENTED /* && req[0] != SP_MSDOS */)
                error ("request %d: not implemented", req[0]);

            /* Pack results into packet.  Always start with result byte.*/
            /* If result byte is 0 (OK), add results as specified in    */
            /* descriptor.                                              */
            VDAT(packet)[0] = res;
            msgsize = 1;
            if (res == SP_SUCCESS) {
                r = &packet[3];
                for (j=0; j<nofresults; j++) {
                    int     sz;
                    tint_t  w;

                    /* Determine result size.                           */
                    switch (reqs[i].resdesc[j]) {
                    case 'b':   sz = 1;  w = results[j].b;          break;
                    case 'h':   sz = 2;  w = results[j].h;          break;
                    case 'w':   sz = 4;  w = results[j].w;          break;
                    case 'v':   sz = VLENSZ;  w = VLEN (results[j].v);  break;
                    case 'r':   sz = 0;                             break;
                    default:    abort();
                    }

                    /* Store/copy result into message and bump size.    */
                    msgsize += sz;
                    while (sz-- > 0) {
                        *r++ = w & 0xFF;
                        w >>= 8;
                    }
                    if (reqs[i].resdesc[j] == 'v' || reqs[i].resdesc[j] == 'r') {
#ifdef  PSERVER_EXTENSION
                        if ((short) VLEN (results[j].v) != -1) {
#endif
                             msgsize += VLEN (results[j].v);
                            if (msgsize <= MAXPACKSZ-VLENSZ) {
                                memcpy (r, VDAT(results[j].v), VLEN (results[j].v));
                                r += w;
                            }
#ifdef  PSERVER_EXTENSION
                        }
#endif
                    }
                }
            }

            /* If packet size > MAXPACKSZ, return error to transputer.  */
            if (msgsize > MAXPACKSZ-VLENSZ) {
                error ("request %d: illegal reply packet size %d", msgsize+VLENSZ);
                VDAT(packet)[0] = 130;      /* some error code */
                msgsize = 1;
            }

            /* Store message size, rounded to even, at start of packet. */
            msgsize = (msgsize + 1) & ~1;
            SETVLEN (packet, msgsize);
        }
    }

    /* Return TRUE upon errors.                                         */
    return err;
}


/*
 *
 *      tint_t server (link_t linkid, bool_t termerr, bool_t verbose, bool_t debug)
 *
 *  Serve transputer program until it terminates.  If termerr is TRUE,
 *  the server terminates as soon as the ErrorFlag is set.  If debug
 *  is TRUE, a log of requests is printed.
 *
 */

#if __STDC__
tint_t  server (link_t linkid, bool_t termerr, bool_t verbose, bool_t debug)
#else
tint_t  server (linkid, termerr, verbose, debug)
link_t  linkid;
bool_t  termerr;
bool_t  verbose;
bool_t  debug;
#endif
{
    byte_t      packet[MAXPACKSZ+1];    /* one extra so we can always   */
                                        /* add a string terminator      */
#define MSGSIZE VLEN(packet)
#define REQ     (VDAT(packet)[0])
    int         res;

    /* Clear terminate flag.  May be set by server functions if they    */
    /* figure it's time to stop serving (e.g. exit).                    */
    terminate = FALSE;

    /* Loop here until terminated.                                      */
    while (! terminate) {
#if SMART_WAIT && INACTIVE_MAX
        time_t  lastactivetime = (smartwaitsecs>0 || inactivesecs>0)
                                                        ? time(NULL) : 0;
#elif   SMART_WAIT
         time_t  lastactivetime = smartwaitsecs>0 ? time(NULL) : 0;
#elif   INACTIVE_MAX
        time_t  lastactivetime = inactivesecs>0 ? time(NULL) : 0;
#endif

        /* Wait for transputer to send something to us.                 */
#ifndef USE_TESTREAD
        while ((res = ReadLink (linkid, packet, MINPACKSZ, 5)) == 0) {
#else
        while (! TestRead (linkid)) {
#endif

            /* Make sure to pay attention to Ctrl-Break under DOS.      */
#if OSID==OS_DOS
            kbhit ();
#endif

            /* And checking the ErrorFlag if requested.                 */
            if (termerr && TestError (linkid))
                break;

            /* Time out with fatal error after inactivity for too long. */
#if INACTIVE_MAX
            if (inactivesecs > 0 && time(NULL)-lastactivetime > inactivesecs)
                break;
#endif

            /* And not spending all our time busy waiting when there's  */
            /* little traffic.                                          */
#if SMART_WAIT
            if (smartwaitsecs > 0 && time(NULL)-lastactivetime > smartwaitsecs)
                sleep (1);
#endif
        }

        /* Time out with fatal error after inactivity for too long.     */
#if INACTIVE_MAX
        if (inactivesecs > 0 && time(NULL)-lastactivetime > inactivesecs) {
            error ("server terminated because transputer link was inactive for %d secs", time(NULL)-lastactivetime);
            break;
        }
#endif

        /* Terminated due to ErrorFlag being set?                       */
        if (termerr && TestError (linkid)) {
            error ("server terminated because transputer ErrorFlag got set");
            break;
        }

#ifdef  USE_TESTREAD
        /* Not terminated, so request pending.                          */
        res = ReadLink (linkid, packet, MINPACKSZ, 20);
#endif

        /* Negative result is fatal error.                              */
        if (res < 0) {
            error ("error serving: %s", lerror (res));
            break;
        }

        /* Something is wrong if we didn't get MINPACKSZ bytes.         */
        if (res != MINPACKSZ) {
            if (res <= VLENSZ)
                error ("timeout after %d bytes of request packet", res);
            else
                error ("request %d: timeout after %d bytes of request packet", REQ, res);
             break;
        }

        /* Check message size:  must be at least 6, and even, and no    */
        /* more than 1022.                                              */
        if (MSGSIZE < MINPACKSZ-VLENSZ || (MSGSIZE & 1) || MSGSIZE > MAXPACKSZ-VLENSZ) {
            error ("request %d: illegal request packet size %d", REQ, MSGSIZE+VLENSZ);
            break;
        }

        /* Get remainder of message, if any.                            */
        if (MSGSIZE > MINPACKSZ-VLENSZ) {
            res = ReadLink (linkid, &packet[MINPACKSZ], MSGSIZE-(MINPACKSZ-VLENSZ), 5);

            /* Negative result is fatal error.                          */
            if (res < 0) {
                error ("error serving: %s", lerror (res));
                break;
            }

            /* Timeout is not allowed here.                             */
            if (res != MSGSIZE-(MINPACKSZ-VLENSZ)) {
                error ("request %d: timeout after %d bytes of request packet", REQ, 8+res, MSGSIZE);
                break;
            }
        }

        /* Process request message.                                     */
        if (request (linkid, packet, debug))
            break;

        /* Send result packet to link.                                  */
        /* XXX - if this fails to send results to exit(), we may        */
        /*       not be able to distinguish this below...               */
        res = WriteLink (linkid, packet, MSGSIZE+VLENSZ, 5);

        /* Negative result is fatal error.                              */
        if (res < 0) {
            error ("error serving: %s", lerror (res));
            break;
        }

        /* Timeout is not allowed here.                                 */
        if (res != MSGSIZE+VLENSZ) {
            error ("request %d: timeout after %d bytes of reply packet", REQ, res);
            break;
        }
    }

    /* Return status returned by transputer program, or -1 if stopped   */
    /* because of errors, -2 if stopped because of ErrorFlag.           */
    /* XXX - must do better, 'cause -1 and -2 are valid status returns */
    /* XXX - should also return !0 if any unimplemented requests */
    if (terminate)
        return termstat;
    else if (termerr && TestError (linkid))
        return -2;
    else
        return -1;
}


/*
 *
 *      void setargs (int argc, char *argv[], int argp)
 *
 *   These are all command line arguments with which main was entered.  The
 *  ones destined for the transputer program start at argv[argp].
 *
 */

#if __STDC__
void    setargs (int argc, char *argv[], int argp)
#else
void    setargs (argc, argv, argp)
int     argc;
char    *argv[];
int     argp;
#endif
{
    /* Store info about all args and about the unparsed args globally.  */
    nofclargs = argc;
    clargs = argv;
    nofclargst = argc-argp;
    clargst = &argv[argp];
}


/*
 *
 *      void setcore (char *data, utint_t size)
 *
 *  Stores data that was peeked when the server started for later
 *  retrieval by transputer program.
 *
 */

#if __STDC__
void    setcore (char *data, utint_t size)
#else
void    setcore (data, size)
char    *data;
utint_t size;
#endif
{
    /* Simply store in global vars for server_core().                   */
    core = data;
    coresz = size;
}


/*
 *
 *      int server_exit ()
 *
 *  Terminate the server with given exit status.  Special values 999999999
 *  and -999999999 are recognized and translated into host specific 'success'
 *  and 'failure' results, resp.
 *
 *  Args:   INT32   status          Rets:   -
 *
 */

SERVER_FN (server_exit, 1, 0)

    /* Set terminate flag and return status.                            */
    terminate = TRUE;
    termstat  = WARG (0);

    /* Recognize special value 0 for success (as per ANSI) and special  */
    /* Inmos values for success and failure.                            */
    if (termstat == 0 || termstat == 999999999)
         termstat = EXIT_SUCCESS;
    else if (termstat == -999999999)
        termstat = EXIT_FAILURE;

    /* Succeeded.                                                       */
    return SP_SUCCESS;
}


/*
 *
 *      int server_command ()
 *
 *  Retrieve command line as a string.  The options recognized by the
 *  server may or may not be removed, depending on an argument.
 *
 *  Args:   BYTE    all             Rets:   VEC string
 *
 */

SERVER_FN (server_command, 1, 1)
{
    char    **argv;
    int     argc;
    int     i;
    char    *s;
    int     len;

    /* Do all arguments or just the ones meant for the transputer.      */
    if (BARG(0)) {
        argv = clargs;
        argc = nofclargs;
    }
    else {
        argv = clargst + 1;
        argc = nofclargst - 1;
    }

    /* Print all arguments into string, and return that.                */
    s = (char *) VDAT(sbuffer);
    *s = '\0';
    len = 0;
    for (i=0; i<argc; i++) {
        if (i != 0)
            len++;
        len += strlen (argv[i]);
        if (strchr (argv[i], ' ') || strchr (argv[i], '\t'))
            len += 2;
        if (len > MAXPACKSZ-VLENSZ-1-VLENSZ-1) {
            error ("server_command(): command line too long at '%s'", argv[i]);
            break;
        }
        if (i != 0)
            strcat (s, " ");
        if (strchr (argv[i], ' ') || strchr (argv[i], '\t'))
            strcat (strcat (strcat (s, "\""), argv[i]), "\"");
        else
            strcat (s, argv[i]);
    }
    SETVLEN (sbuffer, strlen(s));
    VRET(0) = sbuffer;

    /* Return success.                                                  */
    return SP_SUCCESS;
}}


/*
 *
 *      int server_core ()
 *
 *  Return the memory that was peeked from the root transputer's memory
 *  when the server was started, if any.
 *
 *  Args:   INT32   offset          Rets:   VEC core
 *          INT16   length
 *
 */

SERVER_FN (server_core, 2, 1)

    /* Error if no core peeked, or offset > size.                       */
    if (!core || WARG(0) >= coresz)
        return SP_ERROR;

    /* Return length bytes of core, starting at offset.  Max 1024-some. */
    if (WARG(0) + HARG(1) > coresz)
        HARG(1) = coresz - WARG(0);
    if (HARG(1) > MAXPACKSZ-VLENSZ-1-VLENSZ)
        HARG(1) = MAXPACKSZ-VLENSZ-1-VLENSZ;

    /* Copy data from core buffer to sbuffer and return that.           */
    memcpy (VDAT(sbuffer), &core[WARG(0)], (int) HARG(1));
    SETVLEN (sbuffer, HARG(1));
    VRET(0) = sbuffer;

    /* Return success.                                                  */
    return SP_SUCCESS;
}


/*
 *
 *      int server_id ()
 *
 *  Return info about the server.
 *
 *  PSERVER extension:  Add 100 to server version # to indicate pserver.
 *                      If running non-PACT compiled transputer program
 *                      which uses server version info, may need to disable
 *                      pserver extensions.
 *
 *  Args:   -                       Rets:   BYTE    version
 *                                          BYTE    host
 *                                          BYTE    os
 *                                          BYTE    board
 *
 */

SERVER_FN (server_id, 0, 4)

    /* Return all kinds of information.                                 */
#if PSERVER_EXTENSION
    BRET(0) = SERVERID + (pserverext ? 100 : 0);
#else
    BRET(0) = SERVERID;
#endif
    BRET(1) = HOSTID;
    BRET(2) = OSID;
    BRET(3) = BOARDID;

    /* Return success.                                                  */
     return SP_SUCCESS;
}


#if PSERVER_EXTENSION

/*
 *
 *      int server_argc ()
 *
 *  Returns number of command line arguments (argc).
 *  This is a PSERVER extension.
 *
 *  Args:   -                       Rets:   INT32   argc
 *                                          INT32   total-length
 *
 */

SERVER_FN (server_argc, 0, 2)

    /* Return argc and total length and success if pserver extensions   */
    /* enabled.                                                         */
    if (pserverext) {
        int i;
        int len=0;
        WRET(0) = nofclargst;
        for (i=0; i<nofclargst; i++)
            len += strlen (clargst[i]) + 1;
        WRET(1) = len;
        return SP_SUCCESS;
    }

    /* Return unimplemented if pserver extensions are disabled.         */
    else
        return SP_UNIMPLEMENTED;
}


/*
 *
 *      int server_argvx ()
 *
 *  Return command line argument argv[x].
 *  This is a PSERVER extension.
 *
 *  Args:   INT32   x               Rets:   VEC     argv[x]
 *
 */

SERVER_FN (server_argvx, 1, 1)

    /* Return argv[x] and success if pserver extensions enabled.        */
    if (pserverext) {
        int     argc;
        char    *argv;

        /* Return error if argument number out of range.                */
        argc = WARG (0);
        if (argc < 0 || argc > nofclargst)
            return SP_ERROR;

        /* Check length of argv[x].  Diagnose and truncate if too big.  */
        argv = clargst[argc];
        if (strlen(argv) > MAXMSGSZ-1-VLENSZ-1) {
            error ("server_argvx(): command line argument %d truncated to %d chars", argc, MAXMSGSZ-1-VLENSZ);
            argv[MAXMSGSZ-1-VLENSZ-1-1] = '\0';
         }

        /* Copy argv[x] to sbuffer, set size, and return sbuffer.       */
        strcpy (VDAT(sbuffer), argv);
        SETVLEN (sbuffer, strlen(argv)+1);
        VRET(0) = sbuffer;

        /* Return success.                                              */
        return SP_SUCCESS;
    }

    /* Return unimplemented if pserver extensions are disabled.         */
    else
        return SP_UNIMPLEMENTED;
}


/*
 *
 *      int server_abort ()
 *
 *  Abort server.  As exit, but with special message and return status.
 *  This is a PSERVER extension.
 *
 *  Args:   INT32   stat            Rets:   -
 *
 */

SERVER_FN (server_abort, 1, 0)

    /* Display diagnostic and terminate with special status if enabled. */
    if (pserverext) {

        /* Set terminate flag and return status.                        */
        terminate = TRUE;
        termstat  = WARG(0);

        /* Print abort message, depending on abort status.              */
        switch (WARG(0)) {
        case 167:   error ("transputer aborted");
                    break;
        case 168:   error ("transputer aborted - out of memory");
                    break;
        default:    error ("transputer aborted with status %d\n", WARG(0));
                    break;
        }

        /* Succeeded.                                                   */
        return SP_SUCCESS;
    }

    /* Return unimplemented if pserver extensions are disabled.         */
    else
        return SP_UNIMPLEMENTED;
}

/*
 *
 *      int server_times ()
 *
 *  Generated by PACT runtime to tell pserver about times spent in 'user'
 *  and 'system' code, as well as 'real' (= total), to be printed if desired.
 *  This is a PSERVER extension.
 *
 *  Args:   INT32   real            Rets:   -
 *          INT32   user
 *           INT32   sys
 *
 */

SERVER_FN (server_times, 3, 0)

    /* Store user and system times for later use if enabled.            */
    if (pserverext) {

        /* Store times.                                                 */
        ttimes.real = WARG(0);
        ttimes.user = WARG(1);
        ttimes.sys  = WARG(2);

        /* Succeeded.                                                   */
        return SP_SUCCESS;
    }

    /* Return unimplemented if pserver extensions are disabled.         */
    else
        return SP_UNIMPLEMENTED;
}

#endif  /* PSERVER_EXTENSION */
