/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* Server channels.                                                        */
/* This module implements the server channels. Server channels are used    */
/* to send and receive data respectively to and from a client application. */
/* These functions are not directly used by a PPC program but are called   */
/* from the channel implementation of the supported parallel               */
/* architectures.                                                          */
/* This module is only used by UNIX and PVM PPC programs. For PACT-C       */
/* programs, see chanservpact.c.                                           */
/* Note that there are only two possible channels, one for input, the      */
/* other for output.                                                       */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 16.02.1994  Droux       Created                                         */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: chanservunix.h                                                */
/* Tested on :                                                             */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#ifndef chanservunix_h
#define chanservunix_h

#include <ppc/csp/chan.h>

typedef enum                                               /* channels ids */
{
  FROM_CLIENT,                                   /* from client channel id */
  TO_CLIENT                                        /* to client channel id */
} channels_ids;

/*-------------------------------------------------------------------------*/
/* servunix_init_channels:                                                 */
/* Initializes the two channels from and to the client application with    */
/* the given values.                                                       */
/* This function is called by the chanunix module.                         */
/*-------------------------------------------------------------------------*/
void servunix_init_channels(
  int from_client,
  int to_client
);

/*-------------------------------------------------------------------------*/
/* servunix_chan_write:                                                    */
/* Send the given data to the client.                                      */
/* <nitems> corresponds to the number of items of type <type> to be        */
/* transmitted.                                                            */
/*-------------------------------------------------------------------------*/
int servunix_chan_write (
  channel id, 
  void *vbuf, 
  int nitems, 
  int type
);

/*-------------------------------------------------------------------------*/
/* servunix_chan_read:                                                     */
/* Read the given data items from the specified channel.                   */
/* <nitems> corresponds to the number of items of type <type> to be        */
/* received.                                                               */
/*-------------------------------------------------------------------------*/
/* this is exactely the same function as servunix_chan_read. XDR           */
/* will write or read to and from the channel according to the XDR         */
/* direction declared when the channels are initialized.                   */
#define servunix_chan_read servunix_chan_write

/*-------------------------------------------------------------------------*/
/* servunix_start_write:                                                   */
/* Starts a channel transmission of data. Must be called before any call   */
/* to unix_chan_write for each message transmission.                       */
/* Does nothing under here. Implemented for compatibility with the PVM     */
/* lower-half module.                                                      */
/*-------------------------------------------------------------------------*/
void servunix_start_write (void);

/*-------------------------------------------------------------------------*/
/* servunix_end_write:                                                     */
/* Terminates a channel transmission of data. Must be called after all     */
/* calls to unix_chan_write for each message transmission.                 */
/* Does nothing here. Implemented for compatibility with the PVM           */
/* lower-half module.                                                      */
/*-------------------------------------------------------------------------*/
void servunix_end_write (
  channel id
);

/*-------------------------------------------------------------------------*/
/* servunix_start_read:                                                    */
/* Starts a channel reception of data. Must be called before any call      */
/* to unix_chan_read for each message reception.                           */
/* Does nothing here. Implemented for compatibility with the PVM           */
/* lower-half module.                                                      */
/*-------------------------------------------------------------------------*/
void servunix_start_read (
  channel id
);

#endif
