/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Client-Server Library.                                                  */
/* The functions implemented by this module are used by a PACT-C server    */
/* program to send and receive typed data respectively to and from a       */
/* remote client through the extended pserver.                             */
/* These functions follow the XDR specifications, i.e. the format of the   */
/* data received or sent respectively from and to the client are           */
/* converted from and to the XDR format to and from the transputer format. */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 30.08.1994  Droux       Created                                         */
/*-------------------------------------------------------------------------*/
/* filename: chservpact.h                                                  */
/* Tested on :                                                             */
/*   PACT-C v93.1, T800, T805                                              */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

/*-------------------------------------------------------------------------*/
/* scalar output                                                           */
/*-------------------------------------------------------------------------*/

void cslib_xdr_send_int (int *n);
#define cslib_xdr_send_u_int(x) cslib_xdr_send_int((int *)x)
#define cslib_xdr_send_long(x) cslib_xdr_send_int((int *)x)
#define cslib_xdr_send_u_long(x) cslib_xdr_send_int((int *)x)

void cslib_xdr_send_short (short *n);
#define cslib_xdr_send_u_short(x) cslib_xdr_send_short ((short *)x)

void cslib_xdr_send_float (float *n);
void cslib_xdr_send_double (double *n);
void cslib_xdr_send_char (char *n);

/*-------------------------------------------------------------------------*/
/* vector output                                                           */
/*-------------------------------------------------------------------------*/

void cslib_xdr_send_int_vector (int *n, int l);
#define cslib_xdr_send_u_int_vector(x, l) cslib_xdr_send_int_vector((int *)x, l)
#define cslib_xdr_send_long_vector(x, l) cslib_xdr_send_int_vector((int *)x, l)
#define cslib_xdr_send_u_long_vector(x, l) cslib_xdr_send_int_vector((int *)x, l)

void cslib_xdr_send_short_vector (short *n, int l);
#define cslib_xdr_send_u_short_vector(x, l) cslib_xdr_send_short_vector((short *)x, l)

void cslib_xdr_send_float_vector (float *n, int l);
void cslib_xdr_send_double_vector (double *n, int l);
void cslib_xdr_send_char_vector (char *n, int l);

/*-------------------------------------------------------------------------*/
/* scalar input                                                            */
/*-------------------------------------------------------------------------*/

void cslib_xdr_get_int (int *n);
#define cslib_xdr_get_u_int(x) cslib_xdr_get_int((int *)x)
#define cslib_xdr_get_long(x) cslib_xdr_get_int((int *)x)
#define cslib_xdr_get_u_long(x) cslib_xdr_get_int((int *)x)

void cslib_xdr_get_short (short *n);
#define cslib_xdr_get_u_short(x) cslib_xdr_get_short((short *)x)

void cslib_xdr_get_float (float *n);
void cslib_xdr_get_double (double *n);
void cslib_xdr_get_char (char *n);

/*-------------------------------------------------------------------------*/
/* vector input                                                            */
/*-------------------------------------------------------------------------*/

void cslib_xdr_get_int_vector (int *n, int l);
#define cslib_xdr_get_u_int_vector(x, l) cslib_xdr_get_int_vector((int *)x, l)
#define cslib_xdr_get_long_vector(x, l) cslib_xdr_get_int_vector((int *)x, l)
#define cslib_xdr_get_u_long_vector(x, l) cslib_xdr_get_int_vector((int *)x, l)

void cslib_xdr_get_short_vector (short *n, int l);
#define cslib_xdr_get_u_short_vector(x, l) cslib_xdr_get_short_vector((short *)x, l)

void cslib_xdr_get_float_vector (float *n, int l);
void cslib_xdr_get_double_vector (double *n, int l);
void cslib_xdr_get_char_vector (char *n, int l);
