/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Alternate construction PACT-C implementation.                           */
/* This source file contains the implementation of the functions defined   */
/* in the header file alt.h.                                               */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 27.01.1994  Droux       Created                                         */
/* 15.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: altpact.c                                                     */
/* Tested on :                                                             */
/*   PACT-C v92.1, T805                                                    */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <par.h>
#include <time.h>
#include <ppc/csp/chan.h>
#include <ppc/csp/chanpact.h>
#include <ppc/csp/alt.h>

/*-------------------------------------------------------------------------*/
/* alt_init:                                                               */
/* Initializes the alternate construct.                                    */
/* This functions is not called by the user directly but it is generated   */
/* by the macro alt_begin().                                               */
/*-------------------------------------------------------------------------*/
void alt_init (alt_status *act_alt)
{
  act_alt->nb_chans = act_alt->the_timeout = 0;
}

/*-------------------------------------------------------------------------*/
/* alt_wait:                                                               */
/* Activate the alternate constrct. Waits for data comming from one of the */
/* channels given in the specified alt status variable. Exists if some     */
/* data is arrived or if the timeout is reached. Returns TIMEOUT in this   */
/* case or the winner channel id otherwise.                                */
/*-------------------------------------------------------------------------*/
void alt_wait (alt_status *act_alt, channel *winner)
{
  int c;                       /* channel counter for PACT-C alt construct */
  char |*chans [MAX_ALT_CHANS];
  int nb_chans = act_alt->nb_chans;
  
  for (c = 0; c < nb_chans; c++)                    /* get PACT-C channels */
    chans[c] = get_sync_pactc_chan(act_alt->chans[c]);

  alt (c = 0; c < nb_chans; c++)
  {
    guard chans[c]:
      *winner = act_alt->chans[c];
      break;
    timeout act_alt->the_timeout:
      *winner = TIMEOUT;
      break;
  }
}

/*-------------------------------------------------------------------------*/
/* alt_add_channel:                                                        */
/* Add the given channel to the list of channels to be listen.             */
/*-------------------------------------------------------------------------*/
void alt_add_channel (alt_status *act_alt, channel c)
{
  act_alt->chans[act_alt->nb_chans++] = c;
}

/*-------------------------------------------------------------------------*/
/* alt_add_timeout:                                                        */
/* Specifies the timeout of the alternate construct. If this function is   */
/* not called, then the alternate constrct terminates only if something is */
/* available on one of the specified channels.                             */
/*-------------------------------------------------------------------------*/
void alt_add_timeout (alt_status *act_alt, long t)
{
  act_alt->the_timeout = t;
}
