/*-------------------------------------------------------------------------*/
/* Mathematics on Parallel and Distributed Systems                         */
/* Engineering College of Biel, Switzerland.                               */
/* Computer Science Dpt.                                                   */
/* Swiss National Science Foundation project 5003-034331.                  */
/* Pierre-Andre Chevalier, Walter Businger, Nicolas Droux, Werner Hett.    */
/*-------------------------------------------------------------------------*/
/* Description:                                                            */
/* PPC: Portable Parallel C                                                */
/* Alternate construction UNIX implementation.                             */
/* This source file contains the implementation of the functions defined   */
/* in the header file alt.h.                                               */
/*-------------------------------------------------------------------------*/
/* History:                                                                */
/* Date        By          Description                                     */
/* ----------  ----------- ----------------------------------------------- */
/* 07.12.1993  Droux       Created                                         */
/* 08.07.1994  Droux       New structure of source directory               */
/*-------------------------------------------------------------------------*/
/* filename: altunix.c                                                     */
/* Tested on :                                                             */
/*   NEXTSTEP 2.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, NeXT Computers                                          */
/*   NEXTSTEP 3.1, Intel Computers                                         */
/*   NEXTSTEP 3.2, NeXT Computers                                          */
/*   NEXTSTEP 3.2, Intel Computers                                         */
/*   SunOS Release 4.1.3, SPARC                                            */
/*   SunOS Release 5.3 (Solaris 2), SPARC                                  */
/*   Linux 0.99.pl9, Intel Computers                                       */
/*-------------------------------------------------------------------------*/
/* Copyright (c) 1994 Engineering College of Biel-Bienne, Switzerland.     */
/*                                                                         */
/* This program is free software; you can redistribute it and/or modify it */
/* under the terms of the GNU General Public License as published by the   */
/* Free Software Foundation; either version 2 of the License, or (at your  */
/* option) any later version.                                              */
/*                                                                         */
/* This program is distributed in the hope that it will be useful, but     */
/* WITHOUT ANY WARRANTY; without even the implied warranty of              */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    */
/* See the GNU General Public License for more details.                    */
/*-------------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/time.h>
#ifdef SUN4
#include <stddef.h>
#else
#include <stdlib.h>
#endif
#include <ppc/csp/chan.h>
#include <ppc/csp/alt.h>
#include <ppc/csp/chanunixlh.h>

/*-------------------------------------------------------------------------*/
/* alt_init:                                                               */
/* Initializes the alternate construct.                                    */
/* This functions is not called by the user directly but it is generated   */
/* by the macro alt_begin().                                               */
/*-------------------------------------------------------------------------*/
void alt_init (alt_status *act_alt)
{
  act_alt->nb_chans = act_alt->the_timeout = 0;
}

/*-------------------------------------------------------------------------*/
/* alt_wait:                                                               */
/* Activate the alternate constrct. Waits for data comming from one of the */
/* channels given in the specified alt status variable. Exists if some     */
/* data is arrived or if the timeout is reached. Returns TIMEOUT in this   */
/* case or the winner channel id otherwise.                                */
/*-------------------------------------------------------------------------*/
void alt_wait (alt_status *act_alt, channel *winner)
{
  static last_returned = 0;
  fd_set readfds;
  fd_set writefds;
  fd_set exceptfds;
  struct timeval timeout;
  void *timeout_ptr;
  int nfound;
  int n;
  int max_fd;
  
  FD_ZERO(&readfds);                                    /* clear all flags */
  FD_ZERO(&writefds);
  FD_ZERO(&exceptfds);
  
  if (act_alt->the_timeout)                  /* determine timeout to apply */
  {
    timeout.tv_sec = act_alt->the_timeout/SECONDS;
    timeout.tv_usec = act_alt->the_timeout%SECONDS;
    timeout_ptr = &timeout;
  }
  else
    timeout_ptr = NULL;
  
  max_fd = 0;
  for (n = 0; n < act_alt->nb_chans; n++)    /* add channels to be checked */
  {
    int act_fd = unix_read_fd(act_alt->chans[n]);
    FD_SET(act_fd, &readfds);
    if (act_fd > max_fd)
      max_fd = act_fd;
  }
     
  nfound = select(max_fd + 1,                /* wait for something to come */
    &readfds, &writefds, &exceptfds, 
    timeout_ptr);
    
  if (!nfound)
    *winner = TIMEOUT;
  else
  {
    int idx;
    for (n = 0; n < act_alt->nb_chans; n++)
    {
      idx = (n + last_returned) % act_alt->nb_chans;
      if (FD_ISSET(unix_read_fd(act_alt->chans[idx]), &readfds))
      {
        last_returned = idx + 1;
	*winner = act_alt->chans[idx];
	break;
      }
    }
  }
}

/*-------------------------------------------------------------------------*/
/* alt_add_channel:                                                        */
/* Add the given channel to the list of channels to be listen.             */
/*-------------------------------------------------------------------------*/
void alt_add_channel (alt_status *act_alt, channel c)
{
  act_alt->chans[act_alt->nb_chans++] = c;
}

/*-------------------------------------------------------------------------*/
/* alt_add_timeout:                                                        */
/* Specifies the timeout of the alternate construct. If this function is   */
/* not called, then the alternate constrct terminates only if something is */
/* available on one of the specified channels.                             */
/*-------------------------------------------------------------------------*/
void alt_add_timeout (alt_status *act_alt, long t)
{
  act_alt->the_timeout = t;
}
